@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -47,9 +47,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -76,11 +74,15 @@ char debughelp[] = "toggle/set debugging mode";
 char dirhelp[] = "list contents of remote directory";
 char disconhelp[] = "terminate ftp session";
 char domachelp[] = "execute macro";
+char epsv4help[] ="toggle the use of EPRT/EPSV for IPv4";
 char formhelp[] = "set file transfer format";
 char globhelp[] = "toggle metacharacter expansion of local file names";
 char hashhelp[] = "toggle printing `#' for each buffer transferred";
 char helphelp[] = "print local help information";
 char idlehelp[] = "get (set) idle timer on remote side";
+char ipanyhelp[] = "allow all address families";
+char ipv4help[] = "select only IPv4 addresses";
+char ipv6help[] = "select only IPv6 addresses";
 char lcdhelp[] = "change local working directory";
 char lshelp[] = "list contents of remote directory";
 char macdefhelp[] = "define a macro";
@@ -147,6 +149,7 @@ static struct cmd cmdtab[] = {
   {"debug", debughelp, 0, 0, 0, setdebug},
   {"dir", dirhelp, 1, 1, 1, ls},
   {"disconnect", disconhelp, 0, 1, 1, disconnect},
+  {"epsv4", epsv4help, 0, 0, 0, setepsv4},
   {"form", formhelp, 0, 1, 1, setform},
   {"get", receivehelp, 1, 1, 1, get},
   {"glob", globhelp, 0, 0, 0, setglob},
@@ -154,6 +157,9 @@ static struct cmd cmdtab[] = {
   {"help", helphelp, 0, 0, 1, help},
   {"idle", idlehelp, 0, 1, 1, site_idle},
   {"image", binaryhelp, 0, 1, 1, setbinary},
+  {"ipany", ipanyhelp, 0, 0, 0, setipany},
+  {"ipv4", ipv4help, 0, 0, 0, setipv4},
+  {"ipv6", ipv6help, 0, 0, 0, setipv6},
   {"lcd", lcdhelp, 0, 0, 0, lcd},
   {"ls", lshelp, 1, 1, 1, ls},
   {"macdef", macdefhelp, 0, 0, 0, macdef},
