@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -87,6 +87,8 @@ FTP_EXTERN int code;		/* return/reply code for ftp command */
 FTP_EXTERN int crflag;		/* if 1, strip car. rets. on ascii gets */
 FTP_EXTERN char pasv[64];	/* passive port for proxy data connection */
 FTP_EXTERN int passivemode;	/* passive mode enabled */
+FTP_EXTERN int doepsv4;		/* EPSV/EPRT for IPv4 enabled */
+FTP_EXTERN int usefamily;	/* Precondition on an adress family */
 FTP_EXTERN char *altarg;	/* argv[1] with no shell-like preprocessing  */
 FTP_EXTERN char ntin[17];	/* input translation table */
 FTP_EXTERN char ntout[17];	/* output translation table */
