@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -51,9 +51,7 @@
  * FTP User Program -- Command Interface.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 /*#include <sys/ioctl.h>*/
 #include <sys/types.h>
@@ -77,13 +75,10 @@
 #include "ftp_var.h"
 
 #include "libinetutils.h"
+#include "unused-parameter.h"
 
-#if HAVE_LIBREADLINE
-# include <readline/readline.h>
-# include <readline/history.h>
-#else
-# include "readline.h"
-#endif
+#include <readline/readline.h>
+#include <readline/history.h>
 
 
 #define DEFAULT_PROMPT "ftp> "
@@ -110,6 +105,8 @@ static struct argp_option argp_options[] = {
   {"prompt", OPT_PROMPT, "PROMPT", OPTION_ARG_OPTIONAL, "print a command line PROMPT "
    "(optionally), even if not on a tty", GRP+1},
   {"verbose", 'v', NULL, 0, "verbose output", GRP+1},
+  {"ipv4", '4', NULL, 0, "contact IPv4 hosts", GRP+1},
+  {"ipv6", '6', NULL, 0, "contact IPv6 hosts", GRP+1},
 #undef GRP
   {NULL}
 };
@@ -156,6 +153,14 @@ parse_opt (int key, char *arg, struct argp_state *state)
       passivemode = 0;
       break;
 
+    case '4':
+      usefamily = AF_INET;
+      break;
+
+    case '6':
+      usefamily = AF_INET6;
+      break;
+
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -180,6 +185,8 @@ main (int argc, char *argv[])
   interactive = 1;
   autologin = 1;
   passivemode = 0;		/* passive mode not active */
+  doepsv4 = 0;			/* use EPRT/EPSV for IPv4 */
+  usefamily = AF_UNSPEC;	/* allow any address family */
 
   /* Parse command line */
   iu_argp_init ("ftp", default_program_authors);
@@ -249,14 +256,14 @@ main (int argc, char *argv[])
     }
 }
 
-RETSIGTYPE
-intr (int sig ARG_UNUSED)
+void
+intr (int sig _GL_UNUSED_PARAMETER)
 {
   longjmp (toplevel, 1);
 }
 
-RETSIGTYPE
-lostpeer (int sig ARG_UNUSED)
+void
+lostpeer (int sig _GL_UNUSED_PARAMETER)
 {
   if (connected)
     {
@@ -336,10 +343,8 @@ cmdscanner (int top)
 	  break;
 	}
 
-#if HAVE_LIBHISTORY
       if (line && *line)
 	add_history (line);
-#endif
 
       if (l == 0)
 	break;
@@ -381,7 +386,7 @@ cmdscanner (int top)
 int slrflag;
 
 void
-makeargv ()
+makeargv (void)
 {
   char **argp;
 
@@ -400,7 +405,7 @@ makeargv ()
  * handle quoting and strings
  */
 char *
-slurpstring ()
+slurpstring (void)
 {
   int got_one = 0;
   char *sb = stringbase;
