@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-  2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+  2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
   Foundation, Inc.
 
   This file is part of GNU Inetutils.
@@ -110,7 +110,8 @@ extern char tmpline[];
 extern off_t restart_point;
 
 /* Exported from server_mode.c.  */
-extern int server_mode (const char *pidfile, struct sockaddr_in *phis_addr);
+extern int server_mode (const char *pidfile, struct sockaddr_in *phis_addr,
+			char *argv[]);
 
 /* Credential for the request.  */
 struct credentials
