@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-  2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+  2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
   Foundation, Inc.
 
   This file is part of GNU Inetutils.
@@ -51,9 +51,7 @@
  * FTP server.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <alloca.h>
 #include <sys/param.h>
@@ -104,6 +102,7 @@
 #include <progname.h>
 #include <libinetutils.h>
 #include "extern.h"
+#include "unused-parameter.h"
 
 #ifndef LINE_MAX
 # define LINE_MAX 2048
@@ -155,7 +154,7 @@ static off_t file_size;
 static off_t byte_count;
 static sig_atomic_t transflag;	/* Flag where in a middle of transfer.  */
 static const char *pid_file = PATH_FTPDPID;
-#if !defined(CMASK) || CMASK == 0
+#if !defined CMASK || CMASK == 0
 # undef CMASK
 # define CMASK 027
 #endif
@@ -183,7 +182,7 @@ off_to_str (off_t off)
     next_buf = bufs;
 
   if (sizeof (off) > sizeof (long))
-    sprintf (*next_buf, "%qd", (long long int) off);
+    sprintf (*next_buf, "%lld", (long long int) off);
   else if (sizeof (off) == sizeof (long))
     sprintf (*next_buf, "%ld", off);
   else
@@ -375,7 +374,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       if (timeout > maxtimeout)
 	timeout = maxtimeout;
       break;
-      
+
     case 'u':		/* Set umask.  */
       {
 	long val = 0;
@@ -387,7 +386,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
 	  defumask = val;
 	break;
       }
-      
+
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -424,11 +423,10 @@ main (int argc, char *argv[], char **envp)
   /* Parse the command line */
   iu_argp_init ("ftpd", default_program_authors);
   argp_parse (&argp, argc, argv, 0, &index, NULL);
-  
+
   /* Bail out, wrong usage */
-  argc -= index;
-  if (argc != 0)
-    error (1, 0, "surplus arguments; try `%s --help' for more info",
+  if (argc - index != 0)
+    error (EXIT_FAILURE, 0, "surplus arguments; try `%s --help' for more info",
 	   program_name);
 
   /* LOG_NDELAY sets up the logging connection immediately,
@@ -440,8 +438,20 @@ main (int argc, char *argv[], char **envp)
      fd = accept(). tcpd is check if compile with the support  */
   if (daemon_mode)
     {
-      if (server_mode (pid_file, &his_addr) < 0)
-	exit (1);
+#ifndef HAVE_FORK
+      /* Shift out the daemon option in subforks  */
+      int i;
+      for (i = 0; i < argc; ++i)
+	if (strcmp (argv[i], "-D") == 0)
+	  {
+	    int j;
+	    for (j = i; j < argc; ++j)
+	      argv[j] = argv[j + 1];
+	    argv[--argc] = NULL;
+	  }
+#endif
+      if (server_mode (pid_file, &his_addr, argv) < 0)
+	exit (EXIT_FAILURE);
     }
   else
     {
@@ -450,7 +460,7 @@ main (int argc, char *argv[], char **envp)
 		       &addrlen) < 0)
 	{
 	  syslog (LOG_ERR, "getpeername (%s): %m", program_name);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
 
@@ -470,11 +480,11 @@ main (int argc, char *argv[], char **envp)
 		     &addrlen) < 0)
       {
 	syslog (LOG_ERR, "getsockname (%s): %m", program_name);
-	exit (1);
+	exit (EXIT_FAILURE);
       }
   }
 
-#if defined (IP_TOS) && defined (IPTOS_LOWDELAY) && defined (IPPROTO_IP)
+#if defined IP_TOS && defined IPTOS_LOWDELAY && defined IPPROTO_IP
   /* To  minimize delays for interactive traffic.  */
   {
     int tos = IPTOS_LOWDELAY;
@@ -515,7 +525,7 @@ main (int argc, char *argv[], char **envp)
   if (display_file (PATH_NOLOGIN, 530) == 0)
     {
       reply (530, "System not available.");
-      exit (0);
+      exit (EXIT_SUCCESS);
     }
 
   /* Display a Welcome message if exists,
@@ -566,7 +576,7 @@ curdir (void)
   return (cred.guest ? path + 1 : path);
 }
 
-static RETSIGTYPE
+static void
 sigquit (int signo)
 {
   syslog (LOG_ERR, "got signal %s", strsignal (signo));
@@ -574,8 +584,8 @@ sigquit (int signo)
 }
 
 
-static RETSIGTYPE
-lostconn (int signo ARG_UNUSED)
+static void
+lostconn (int signo _GL_UNUSED_PARAMETER)
 {
   if (debug)
     syslog (LOG_DEBUG, "lost connection");
@@ -821,7 +831,7 @@ pass (const char *passwd)
 	    {
 	      syslog (LOG_NOTICE, "repeated login failures from %s",
 		      cred.remotehost);
-	      exit (0);
+	      exit (EXIT_SUCCESS);
 	    }
 	  return;
 	}
@@ -923,7 +933,7 @@ store (const char *name, const char *mode, int unique)
   if (unique && stat (name, &st) == 0)
     {
       const char *name_unique = gunique (name);
-      
+
       if (name_unique)
         name = name_unique;
       else
@@ -967,7 +977,7 @@ store (const char *name, const char *mode, int unique)
 	  /* We must do this seek to "current" position
 	     because we are changing from reading to
 	     writing.  */
-	  if (fseek (fout, 0L, SEEK_CUR) < 0)
+	  if (fseeko (fout, 0L, SEEK_CUR) < 0)
 	    {
 	      perror_reply (550, name);
 	      goto done;
@@ -1020,6 +1030,9 @@ getdatasock (const char *mode)
   /* anchor socket to avoid multi-homing problems */
   data_source.sin_family = AF_INET;
   data_source.sin_addr = ctrl_addr.sin_addr;
+#if HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
+  data_source.sin_len = sizeof (struct sockaddr_in);
+#endif
   for (tries = 1;; tries++)
     {
       if (bind (s, (struct sockaddr *) &data_source,
@@ -1031,7 +1044,7 @@ getdatasock (const char *mode)
     }
   seteuid ((uid_t) cred.uid);
 
-#if defined (IP_TOS) && defined (IPTOS_THROUGHPUT) && defined (IPPROTO_IP)
+#if defined IP_TOS && defined IPTOS_THROUGHPUT && defined IPPROTO_IP
   {
     int on = IPTOS_THROUGHPUT;
     if (setsockopt (s, IPPROTO_IP, IP_TOS, (char *) &on, sizeof (int)) < 0)
@@ -1065,7 +1078,7 @@ dataconn (const char *name, off_t size, const char *mode)
   if (pdata >= 0)
     {
       struct sockaddr_in from;
-      socklen_t s; 
+      socklen_t s;
       socklen_t fromlen = sizeof (from);
 
       signal (SIGALRM, toolong);
@@ -1081,7 +1094,7 @@ dataconn (const char *name, off_t size, const char *mode)
 	}
       close (pdata);
       pdata = s;
-#if defined (IP_TOS) && defined (IPTOS_THROUGHPUT) && defined (IPPROTO_IP)
+#if defined IP_TOS && defined IPTOS_THROUGHPUT && defined IPPROTO_IP
       /* Optimize throughput.  */
       {
 	int tos = IPTOS_THROUGHPUT;
@@ -1406,7 +1419,7 @@ void
 statcmd (void)
 {
   struct sockaddr_in *sin;
-  u_char *a, *p;
+  unsigned char *a, *p;
 
   lreply (211, "%s FTP server status:", hostname);
   if (!no_version)
@@ -1454,8 +1467,8 @@ statcmd (void)
       printf ("     PORT");
       sin = &data_dest;
     printaddr:
-      a = (u_char *) & sin->sin_addr;
-      p = (u_char *) & sin->sin_port;
+      a = (unsigned char *) & sin->sin_addr;
+      p = (unsigned char *) & sin->sin_port;
 #define UC(b) (((int) b) & 0xff)
       printf (" (%d,%d,%d,%d,%d,%d)\r\n", UC (a[0]),
 	      UC (a[1]), UC (a[2]), UC (a[3]), UC (p[0]), UC (p[1]));
@@ -1675,8 +1688,8 @@ dologout (int status)
   _exit (status);
 }
 
-static RETSIGTYPE
-myoob (int signo ARG_UNUSED)
+static void
+myoob (int signo _GL_UNUSED_PARAMETER)
 {
   char *cp;
 
