@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-  2009, 2010 Free Software Foundation, Inc.
+  2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -17,9 +17,7 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <signal.h>
 #include <sys/types.h>
@@ -39,6 +37,9 @@
 # include <tcpd.h>
 #endif
 
+#include <libinetutils.h>
+#include "unused-parameter.h"
+
 static void reapchild (int);
 
 #define DEFPORT 21
@@ -83,8 +84,8 @@ check_host (struct sockaddr *sa)
 }
 #endif
 
-static RETSIGTYPE
-reapchild (int signo ARG_UNUSED)
+static void
+reapchild (int signo _GL_UNUSED_PARAMETER)
 {
   int save_errno = errno;
 
@@ -94,7 +95,7 @@ reapchild (int signo ARG_UNUSED)
 }
 
 int
-server_mode (const char *pidfile, struct sockaddr_in *phis_addr)
+server_mode (const char *pidfile, struct sockaddr_in *phis_addr, char *argv[])
 {
   int ctl_sock, fd;
   struct servent *sv;
@@ -132,6 +133,9 @@ server_mode (const char *pidfile, struct sockaddr_in *phis_addr)
   memset (&server_addr, 0, sizeof server_addr);
   server_addr.sin_family = AF_INET;
   server_addr.sin_port = htons (port);
+#if HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
+  server_addr.sin_len = sizeof (struct sockaddr_in);
+#endif
 
   if (bind (ctl_sock, (struct sockaddr *) &server_addr, sizeof server_addr))
     {
@@ -178,5 +182,10 @@ server_mode (const char *pidfile, struct sockaddr_in *phis_addr)
   if (!check_host ((struct sockaddr *) phis_addr))
     return -1;
 #endif
+
+#ifndef HAVE_FORK
+  _exit (execvp (argv[0], argv));
+#endif
+
   return fd;
 }
