@@ -1,6 +1,6 @@
 /* changeif.c -- change the configuration of a network interface
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -19,9 +19,7 @@
 
 /* Written by Marcus Brinkmann.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <stdio.h>
 #include <errno.h>
@@ -50,7 +48,7 @@
   err = ioctl (sfd, SIOCSIF##type, ifr);				\
   if (err < 0)								\
     {									\
-      error (0, errno, "%s failed", "SIOCSIF" #type); 		        \
+      error (0, errno, "%s failed", "SIOCSIF" #type);			\
       return -1;							\
     }
 
@@ -72,7 +70,7 @@ set_address (int sfd, struct ifreq *ifr, char *address)
 
   if (!host)
     {
-      error (0, 0, "can not resolve `%s': %s", address, hstrerror (h_errno));
+      error (0, 0, "cannot resolve `%s': %s", address, hstrerror (h_errno));
       return -1;
     }
   if (host->h_addrtype != AF_INET)
@@ -141,7 +139,7 @@ int
 set_mtu (int sfd, struct ifreq *ifr, int mtu)
 {
 #ifndef SIOCSIFMTU
-  error (0, 0, 
+  error (0, 0,
          "don't know how to set the interface mtu on this system");
   return -1;
 #else
@@ -187,7 +185,7 @@ set_metric (int sfd, struct ifreq *ifr, int metric)
 int
 set_flags (int sfd, struct ifreq *ifr, int setflags, int clrflags)
 {
-#if !defined (SIOCGIFFLAGS) || !defined (SIOCSIFFLAGS)
+#if !defined SIOCGIFFLAGS || !defined SIOCSIFFLAGS
   error (0, 0,
          "don't know how to set the interface flags on this system");
   return -1;
