@@ -1,6 +1,6 @@
 /* flags.c -- network interface flag handling
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -19,9 +19,7 @@
 
 /* Written by Marcus Brinkmann.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <stdio.h>
 
@@ -116,7 +114,7 @@ struct if_flag
 #ifdef IFF_LINK1		/* Per link layer defined bit.  */
     {"LINK1", IFF_LINK1},
 #endif
-#if defined(IFF_LINK2) && defined(IFF_ALTPHYS)
+#if defined IFF_LINK2 && defined IFF_ALTPHYS
 # if IFF_LINK2 == IFF_ALTPHYS
     /* IFF_ALTPHYS == IFF_LINK2 on FreeBSD.  This entry is used as a
        fallback for if_flagtoname conversion, if no relevant EXPECT_
@@ -203,7 +201,7 @@ struct if_flag
 #ifdef IFF_VMIFNET		/* Used to identify a virtual MAC address.  */
     {"VMIFNET", IFF_VMIFNET},
 #endif
-#if defined(IFF_D1) && defined(IFF_SNAP)
+#if defined IFF_D1 && defined IFF_SNAP
 # if IFF_D1 == IFF_SNAP
     /* IFF_SNAP == IFF_D1 on OSF 4.0g systems.  This entry is used as a
        fallback for if_flagtoname conversion, if no relevant EXPECT_
@@ -400,11 +398,20 @@ static struct if_flag_char flag_char_tab[] = {
   { IFF_DYNAMIC,     'd' },
 #endif
   { IFF_PROMISC,     'P' },
+#ifdef IFF_NOTRAILERS
   { IFF_NOTRAILERS,  'N' },
+#endif
   { IFF_NOARP,       'O' },
   { IFF_POINTOPOINT, 'P' },
+#ifdef IFF_SLAVE
   { IFF_SLAVE,       's' },
+#endif
+#ifdef IFF_MASTER
   { IFF_MASTER,      'm' },
+#endif
+#ifdef IFF_SIMPLEX
+  { IFF_SIMPLEX,     'S' },
+#endif
   { IFF_RUNNING,     'R' },
   { IFF_UP,          'U' },
   { 0 }
