@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -27,7 +27,6 @@ struct if_nameindex
   int if_index;
 };
 extern unsigned int if_nametoindex (const char *ifname);
-extern char *if_indextoname (unsigned int ifindex, char *ifname);
 extern struct if_nameindex *if_nameindex (void);
 extern void if_freenameindex (struct if_nameindex *ptr);
 # endif
