@@ -1,6 +1,6 @@
 /* ifconfig.c -- network interface configuration utility
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -19,9 +19,7 @@
 
 /* Written by Marcus Brinkmann.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/stat.h>
 #include <stdio.h>
@@ -44,7 +42,9 @@
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <net/if.h>
+#include <netinet/in.h>
 #include <arpa/inet.h>
+#include <progname.h>
 #include "ifconfig.h"
 
 int
@@ -53,14 +53,14 @@ main (int argc, char *argv[])
   int err = 0;
   int sfd;
   struct ifconfig *ifp;
-
+  set_program_name (argv[0]);
   parse_cmdline (argc, argv);
 
   sfd = socket (AF_INET, SOCK_STREAM, 0);
   if (sfd < 0)
     {
       error (0, errno, "socket error");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   for (ifp = ifs; ifp < ifs + nifs; ifp++)
