@@ -1,6 +1,6 @@
 /* options.c -- process the command line options
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -19,9 +19,7 @@
 
 /* Written by Marcus Brinkmann.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <stdio.h>
 #include <errno.h>
@@ -141,10 +139,10 @@ struct format formats[] = {
    "Traditional UNIX interface listing.  Default for Solaris and HPUX.",
    "${format}{check-existence}"
    "${ifdisplay?}{"
-   "${name}: flags=${flags}{number}<${flags}{string}{,}>"
+   "${name}: flags=${flags}{number}{%hx}<${flags}{string}{,}>"
    "${mtu?}{ mtu ${mtu}}${\\n}"
    "${addr?}{${\\t}inet ${addr}"
-   " netmask ${netmask}{0}{%02x}${netmask}{1}{%02x}"
+   " netmask ${netmask}{0}{%#02x}${netmask}{1}{%02x}"
    "${netmask}{2}{%02x}${netmask}{3}{%02x}"
    "${brdaddr?}{ broadcast ${brdaddr}}${\\n}}"
    "${exists?}{hwtype?}{${hwtype?}{${\\t}${hwtype}"
@@ -376,15 +374,10 @@ parse_opt_set_default_format (const char *format)
     format = system_default_format ? system_default_format : "default";
 
   for (frm = formats; frm->name; frm++)
-    {
-      if (!strcmp (format, frm->name))
-	{
-	  default_format = frm->templ;
-	  return;
-	}
-    }
-
-  default_format = format;
+    if (!strcmp (format, frm->name)) break;
+  if (frm == NULL || frm->templ == NULL)
+    error (EXIT_FAILURE, 0, "%s: unknown output format", format);
+  default_format = frm->templ;
 }
 
 static int
@@ -573,7 +566,6 @@ parse_cmdline (int argc, char *argv[])
 {
   int index;
   struct ifconfig *ifp = ifs;
-  set_program_name (argv[0]);
 
   parse_opt_set_default_format (NULL);
   iu_argp_init ("ifconfig", program_authors);
@@ -595,7 +587,9 @@ parse_cmdline (int argc, char *argv[])
       /* No interfaces specified.  Get a list of all interfaces.  */
       struct if_nameindex *ifnx, *ifnxp;
 
-      ifnx = ifnxp = if_nameindex ();
+      ifnx = ifnxp = system_if_nameindex ();
+      if (!ifnx)
+	error (EXIT_FAILURE, 0, "could not get list of interfaces");
       while (ifnxp->if_index != 0 || ifnxp->if_name != NULL)
 	{
 	  struct ifconfig *ifp;
