@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -19,12 +19,10 @@
 
 #include <config.h>
 
-#if defined(__linux__)
+#if defined __linux__
 # include "system/linux.c"
 #elif defined(__sun__)
 # include "system/solaris.c"
-#elif defined(__hpux__)
-# include "system/hpux.c"
 #elif defined(__QNX__)
 # include "system/qnx.c"
 #else
