@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -23,6 +23,8 @@
 # define IFCONFIG_SYSTEM_H
 
 
+# include <if_index.h>
+
 /* Option parsing.  */
 
 extern struct argp_child system_argp_child;
@@ -85,13 +87,12 @@ int system_configure (int sfd, struct ifreq *ifr,
 # endif
 
 
+extern struct if_nameindex* (*system_if_nameindex) (void);
 
-# if defined(__linux__)
+# if defined __linux__
 #  include "system/linux.h"
 # elif defined(__sun__)
 #  include "system/solaris.h"
-# elif defined(__hpux__)
-#  include "system/hpux.h"
 # elif defined(__QNX__)
 #  include "system/qnx.h"
 # else
