@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -46,7 +46,7 @@ struct system_ifconfig
    for in generic format strings with ${exists?}{foobar?}.  */
 # undef SYSTEM_FORMAT_HANDLER
 
-/* Prototype system_fh_* functions here. 
+/* Prototype system_fh_* functions here.
    void system_fh_newline (format_data_t, int, char **);
 */
 
