@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -19,8 +19,8 @@
 
 /* Written by Marcus Brinkmann.  */
 
-#ifndef IFCONFIG_SYSTEM_SOLARIS_H
-# define IFCONFIG_SYSTEM_SOLARIS_H
+#ifndef IFCONFIG_SYSTEM_QNX_H
+# define IFCONFIG_SYSTEM_QNX_H
 
 # include "../printif.h"
 # include "../options.h"
