@@ -1,6 +1,6 @@
 /* solaris.c -- Solaris specific code for ifconfig
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -19,9 +19,7 @@
 
 /* Written by Marcus Brinkmann.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/stat.h>
 #include <stdio.h>
@@ -178,3 +176,9 @@ system_configure (int sfd, struct ifreq *ifr, struct system_ifconfig *ifs)
   return 0;
 #endif
 }
+
+
+
+/* System hooks. */
+
+struct if_nameindex* (*system_if_nameindex) (void) = if_nameindex;
