@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -17,27 +17,25 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/types.h>
 #include <unistd.h>
 
-u_short
-icmp_cksum (u_char * addr, int len)
+unsigned short
+icmp_cksum (unsigned char * addr, int len)
 {
   register int sum = 0;
-  u_short answer = 0;
-  u_short *wp;
+  unsigned short answer = 0;
+  unsigned short *wp;
 
-  for (wp = (u_short *) addr; len > 1; wp++, len -= 2)
+  for (wp = (unsigned short *) addr; len > 1; wp++, len -= 2)
     sum += *wp;
 
   /* Take in an odd byte if present */
   if (len == 1)
     {
-      *(u_char *) & answer = *(u_char *) wp;
+      *(unsigned char *) & answer = *(unsigned char *) wp;
       sum += answer;
     }
 
