@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -17,14 +17,11 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/time.h>
-#include <signal.h>
 
 #include <netinet/in_systm.h>
 #include <netinet/in.h>
@@ -34,7 +31,7 @@
 #include <icmp.h>
 
 int
-icmp_generic_encode (u_char * buffer, size_t bufsize, int type, int ident,
+icmp_generic_encode (unsigned char * buffer, size_t bufsize, int type, int ident,
 		     int seqno)
 {
   icmphdr_t *icmp;
@@ -53,11 +50,11 @@ icmp_generic_encode (u_char * buffer, size_t bufsize, int type, int ident,
 }
 
 int
-icmp_generic_decode (u_char * buffer, size_t bufsize,
+icmp_generic_decode (unsigned char * buffer, size_t bufsize,
 		     struct ip **ipp, icmphdr_t ** icmpp)
 {
   size_t hlen;
-  u_short cksum;
+  unsigned short cksum;
   struct ip *ip;
   icmphdr_t *icmp;
 
@@ -77,20 +74,20 @@ icmp_generic_decode (u_char * buffer, size_t bufsize,
   /* Recompute checksum */
   cksum = icmp->icmp_cksum;
   icmp->icmp_cksum = 0;
-  icmp->icmp_cksum = icmp_cksum ((u_char *) icmp, bufsize - hlen);
+  icmp->icmp_cksum = icmp_cksum ((unsigned char *) icmp, bufsize - hlen);
   if (icmp->icmp_cksum != cksum)
     return 1;
   return 0;
 }
 
 int
-icmp_echo_encode (u_char * buffer, size_t bufsize, int ident, int seqno)
+icmp_echo_encode (unsigned char * buffer, size_t bufsize, int ident, int seqno)
 {
   return icmp_generic_encode (buffer, bufsize, ICMP_ECHO, ident, seqno);
 }
 
 int
-icmp_echo_decode (u_char * buffer, size_t bufsize,
+icmp_echo_decode (unsigned char * buffer, size_t bufsize,
 		  struct ip **ipp, icmphdr_t ** icmpp)
 {
   return icmp_generic_decode (buffer, bufsize, ipp, icmpp);
