@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -17,15 +17,13 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
+
+#include <stddef.h>
 
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/time.h>
-#include <signal.h>
-#include <stddef.h>
 
 #include <netinet/in_systm.h>
 #include <netinet/in.h>
@@ -35,7 +33,7 @@
 #include <icmp.h>
 
 int
-icmp_timestamp_encode (u_char * buffer, size_t bufsize, int ident, int seqno)
+icmp_timestamp_encode (unsigned char * buffer, size_t bufsize, int ident, int seqno)
 {
   icmphdr_t *icmp;
   struct timeval tv;
