@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-  2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -46,11 +46,9 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
 # include <sys/types.h>
 # ifdef ENCRYPTION
 #  include <sys/socket.h>
@@ -70,12 +68,12 @@
 
 # define SERVICE_NAME	"rcmd"
 
-# if defined(SHISHI)
-int kcmd (Shishi **, int *, char **, u_short, char *, char **,
+# if defined SHISHI
+int kcmd (Shishi **, int *, char **, unsigned short, char *, char **,
 	  char *, int *, char *, char *, Shishi_key **,
 	  struct sockaddr_in *, struct sockaddr_in *, long);
 # else
-int kcmd (int *, char **, u_short, char *, char *, char *, int *,
+int kcmd (int *, char **, unsigned short, char *, char *, char *, int *,
 	  KTEXT, char *, char *, CREDENTIALS *, Key_schedule,
 	  MSG_DAT *, struct sockaddr_in *, struct sockaddr_in *, long);
 # endif
@@ -86,9 +84,9 @@ int kcmd (int *, char **, u_short, char *, char *, char *, int *,
  *	if fd2p is non-NULL, another socket is filled in for it
  */
 
-# if defined(SHISHI)
+# if defined SHISHI
 int
-krcmd (Shishi ** h, char **ahost, u_short rport, char **remuser, char *cmd,
+krcmd (Shishi ** h, char **ahost, unsigned short rport, char **remuser, char *cmd,
        int *fd2p, char *realm)
 {
   int sock = -1, err = 0;
@@ -112,7 +110,7 @@ krcmd (Shishi ** h, char **ahost, u_short rport, char **remuser, char *cmd,
 
 # elif defined(KERBEROS)
 int
-krcmd (char **ahost, u_short rport, char *remuser, char *cmd, int *fd2p,
+krcmd (char **ahost, unsigned short rport, char *remuser, char *cmd, int *fd2p,
        char *realm)
 {
   int sock = -1, err = 0;
@@ -140,9 +138,9 @@ krcmd (char **ahost, u_short rport, char *remuser, char *cmd, int *fd2p,
 
 # ifdef ENCRYPTION
 
-#  if defined(SHISHI)
+#  if defined SHISHI
 int
-krcmd_mutual (Shishi ** h, char **ahost, u_short rport, char **remuser,
+krcmd_mutual (Shishi ** h, char **ahost, unsigned short rport, char **remuser,
 	      char *cmd, int *fd2p, char *realm, Shishi_key ** key)
 {
   int sock = -1, err = 0;
@@ -168,7 +166,7 @@ krcmd_mutual (Shishi ** h, char **ahost, u_short rport, char **remuser,
 
 #  elif defined(KERBEROS)
 int
-krcmd_mutual (char **ahost, u_short rport, char *remuser, char *cmd,
+krcmd_mutual (char **ahost, unsigned short rport, char *remuser, char *cmd,
 	      int *fd2p, char *realm, CREDENTIALS * cred, Key_schedule sched)
 {
   int sock, err;
