@@ -1,6 +1,6 @@
 /*
-  Copyright (C) 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -17,6 +17,10 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
+#ifndef HAVE_CONFIG_H
+#error "<config.h> has not been included; please included it"
+#endif
+
 #include "argp-version-etc.h"
 
 void utmp_init (char *line, char *user, char *id);
@@ -29,6 +33,13 @@ void logwtmp_keep_open (char *line, char *name, char *host);
 
 extern const char *default_program_authors[];
 
-#define iu_argp_init(name, authors) 				\
+#define iu_argp_init(name, authors)				\
   argp_program_bug_address = "<" PACKAGE_BUGREPORT ">";		\
   argp_version_setup (name, authors);
+
+#ifdef HAVE_FORK
+# define fork_exit(s) exit(s)
+#else
+# define fork() vfork()
+# define fork_exit(s) _exit(s)
+#endif
