@@ -1,6 +1,7 @@
 /* localhost.c - A slightly more convenient wrapper for gethostname
   Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-  2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation,
+  Inc.
 
   This file is part of GNU Inetutils.
 
@@ -19,14 +20,9 @@
 
 /* Written by Miles Bader.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <unistd.h>
-#ifdef HAVE_MALLOC_H
-# include <malloc.h>
-#endif
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
@@ -82,7 +78,7 @@ localhost (void)
     /* Determine FQDN */
     {
       struct hostent *hp = gethostbyname (buf);
-      
+
       if (hp)
 	{
 	  struct in_addr addr;
