@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-  2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -57,9 +57,7 @@
 			Jim Guyton 10/85
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -75,7 +73,10 @@
 
 #include "tftpsubs.h"
 
+/* Some systems define PKTSIZE in <arpa/tftp.h>.  */
+#ifndef PKTSIZE
 #define PKTSIZE SEGSIZE+4	/* should be moved to tftp.h */
+#endif
 
 struct bf
 {
@@ -98,12 +99,12 @@ int prevchar = -1;		/* putbuf: previous char (cr check) */
 static struct tftphdr *rw_init (int);
 
 struct tftphdr *
-w_init ()
+w_init (void)
 {
   return rw_init (0);
 }				/* write-behind */
 struct tftphdr *
-r_init ()
+r_init (void)
 {
   return rw_init (1);
 }				/* read-ahead */
@@ -257,7 +258,7 @@ write_behind (FILE * file, int convert)
       if (prevchar == '\r')
 	{			/* if prev char was cr */
 	  if (c == '\n')	/* if have cr,lf then just */
-	    fseek (file, -1, 1);	/* smash lf on top of the cr */
+	    fseeko (file, -1, 1);	/* smash lf on top of the cr */
 	  else if (c == '\0')	/* if have cr,nul then */
 	    goto skipit;	/* just skip over the putc */
 	  /* else just fall through and allow it */
@@ -287,7 +288,7 @@ synchnet (int f)
 {
   int i, j = 0;
   char rbuf[PKTSIZE];
-  struct sockaddr_in from;
+  struct sockaddr_storage from;
   socklen_t fromlen;
 
   while (1)
