@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-  2008, 2009, 2010 Free Software Foundation, Inc.
+  2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -40,17 +40,18 @@
 /* Written by Wietse Venema.  With port to GNU Inetutils done by Alain
    Magloire.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/time.h>
 #include <time.h>
-#if defined(UTMPX) && defined(HAVE_UTMPX_H)
+#if defined UTMPX && defined HAVE_UTMPX_H
 # define __USE_GNU
 # include <utmpx.h>
 #else
+# if HAVE_UTIL_H
+#  include <util.h>
+# endif
 # include <utmp.h>
 #endif
 #include <string.h>
@@ -75,13 +76,13 @@ utmp_logout (char *line)
       ut->ut_exit.e_termination = 0;
       ut->ut_exit.e_exit = 0;
       gettimeofday (&tv, 0);
-      ut->ut_tv.tv_sec = tv_sec;
-      ut->ut_tv.tv_usec = tv_usec;
+      ut->ut_tv.tv_sec = tv.tv_sec;
+      ut->ut_tv.tv_usec = tv.tv_usec;
       pututxline (ut);
       updwtmpx (PATH_WTMPX, ut);
     }
   endutxent ();
-#else
+#elif HAVE_DECL_GETUTENT
   struct utmp utx;
   struct utmp *ut;
 
@@ -118,5 +119,8 @@ utmp_logout (char *line)
 # endif
     }
   endutent ();
+#else
+  if (logout (line))
+    logwtmp (line, "", "");
 #endif
 }
