@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-  2009, 2010 Free Software Foundation, Inc.
+  2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -49,9 +49,7 @@
 /* This code is derived from software contributed to Berkeley by
    Michael Fischbein. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -62,22 +60,19 @@
 #include "extern.h"
 
 int
-namecmp (a, b)
-     const FTSENT *a, *b;
+namecmp (const FTSENT *a, const FTSENT *b)
 {
   return (strcmp (a->fts_name, b->fts_name));
 }
 
 int
-revnamecmp (a, b)
-     const FTSENT *a, *b;
+revnamecmp (const FTSENT *a, const FTSENT *b)
 {
   return (strcmp (b->fts_name, a->fts_name));
 }
 
 int
-modcmp (a, b)
-     const FTSENT *a, *b;
+modcmp (const FTSENT *a, const FTSENT *b)
 {
   if (b->fts_statp->st_mtime > a->fts_statp->st_mtime)
     return (1);
@@ -88,8 +83,7 @@ modcmp (a, b)
 }
 
 int
-revmodcmp (a, b)
-     const FTSENT *a, *b;
+revmodcmp (const FTSENT *a, const FTSENT *b)
 {
   if (b->fts_statp->st_mtime > a->fts_statp->st_mtime)
     return (-1);
@@ -100,8 +94,7 @@ revmodcmp (a, b)
 }
 
 int
-acccmp (a, b)
-     const FTSENT *a, *b;
+acccmp (const FTSENT *a, const FTSENT *b)
 {
   if (b->fts_statp->st_atime > a->fts_statp->st_atime)
     return (1);
@@ -112,8 +105,7 @@ acccmp (a, b)
 }
 
 int
-revacccmp (a, b)
-     const FTSENT *a, *b;
+revacccmp (const FTSENT *a, const FTSENT *b)
 {
   if (b->fts_statp->st_atime > a->fts_statp->st_atime)
     return (-1);
@@ -124,8 +116,7 @@ revacccmp (a, b)
 }
 
 int
-statcmp (a, b)
-     const FTSENT *a, *b;
+statcmp (const FTSENT *a, const FTSENT *b)
 {
   if (b->fts_statp->st_ctime > a->fts_statp->st_ctime)
     return (1);
@@ -136,8 +127,7 @@ statcmp (a, b)
 }
 
 int
-revstatcmp (a, b)
-     const FTSENT *a, *b;
+revstatcmp (const FTSENT *a, const FTSENT *b)
 {
   if (b->fts_statp->st_ctime > a->fts_statp->st_ctime)
     return (-1);
@@ -148,8 +138,7 @@ revstatcmp (a, b)
 }
 
 int
-sizecmp (a, b)
-     const FTSENT *a, *b;
+sizecmp (const FTSENT *a, const FTSENT *b)
 {
   if (b->fts_statp->st_size > a->fts_statp->st_size)
     return (1);
@@ -160,8 +149,7 @@ sizecmp (a, b)
 }
 
 int
-revsizecmp (a, b)
-     const FTSENT *a, *b;
+revsizecmp (const FTSENT *a, const FTSENT *b)
 {
   if (b->fts_statp->st_size > a->fts_statp->st_size)
     return (-1);
