@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-  2009, 2010 Free Software Foundation, Inc.
+  2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -46,9 +46,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -68,7 +66,7 @@ static size_t fts_maxarglen (char *const *);
 static void fts_padjust (FTS *, void *);
 static int fts_palloc (FTS *, size_t);
 static FTSENT *fts_sort (FTS *, FTSENT *, int);
-static u_short fts_stat (FTS *, struct dirent *, FTSENT *, int);
+static unsigned short fts_stat (FTS *, struct dirent *, FTSENT *, int);
 
 #ifndef MAX
 # define MAX(a, b)	(((a) > (b)) ? (a) : (b))
@@ -92,10 +90,7 @@ static u_short fts_stat (FTS *, struct dirent *, FTSENT *, int);
 #define BREAD		3	/* fts_read */
 
 FTS *
-fts_open (argv, options, compar)
-     char *const *argv;
-     register int options;
-     int (*compar) (const FTSENT **, const FTSENT **);
+fts_open (char *const *argv, register int options, int (*compar) (const FTSENT **, const FTSENT **))
 {
   register FTS *sp;
   register FTSENT *p, *root;
@@ -215,9 +210,7 @@ mem1:free (sp);
 }
 
 static void
-fts_load (sp, p)
-     FTS *sp;
-     register FTSENT *p;
+fts_load (FTS *sp, register FTSENT *p)
 {
   register int len;
   register char *cp;
@@ -231,7 +224,7 @@ fts_load (sp, p)
    */
   len = p->fts_pathlen = p->fts_namelen;
   memmove (sp->fts_path, p->fts_name, len + 1);
-  if ((cp = rindex (p->fts_name, '/')) && (cp != p->fts_name || cp[1]))
+  if ((cp = strrchr (p->fts_name, '/')) && (cp != p->fts_name || cp[1]))
     {
       len = strlen (++cp);
       memmove (p->fts_name, cp, len + 1);
@@ -242,8 +235,7 @@ fts_load (sp, p)
 }
 
 int
-fts_close (sp)
-     FTS *sp;
+fts_close (FTS *sp)
 {
   register FTSENT *freep, *p;
   int saved_errno = 0;
@@ -300,8 +292,7 @@ fts_close (sp)
 	    p->fts_path[0] == '/' ? 0 : p->fts_pathlen)
 
 FTSENT *
-fts_read (sp)
-     register FTS *sp;
+fts_read (register FTS *sp)
 {
   register FTSENT *p;
   register FTSENT *tmp;
@@ -521,10 +512,7 @@ next:tmp = p;
  * reasons.
  */
 int
-fts_set (sp, p, instr)
-     FTS *sp;
-     FTSENT *p;
-     int instr;
+fts_set (FTS *sp, FTSENT *p, int instr)
 {
   if (instr && instr != FTS_AGAIN && instr != FTS_FOLLOW &&
       instr != FTS_NOINSTR && instr != FTS_SKIP)
@@ -537,9 +525,7 @@ fts_set (sp, p, instr)
 }
 
 FTSENT *
-fts_children (sp, instr)
-     register FTS *sp;
-     int instr;
+fts_children (register FTS *sp, int instr)
 {
   register FTSENT *p;
   int fd;
@@ -622,9 +608,7 @@ fts_children (sp, instr)
  * been found, cutting the stat calls by about 2/3.
  */
 static FTSENT *
-fts_build (sp, type)
-     register FTS *sp;
-     int type;
+fts_build (register FTS *sp, int type)
 {
   struct dirent *dp;
   register FTSENT *p, *head;
@@ -645,7 +629,7 @@ fts_build (sp, type)
    * Open the directory for reading.  If this fails, we're done.
    * If being called from fts_read, set the fts_info field.
    */
-#if defined(HAVE_OPENDIR2) && defined(DTF_HIDEW) 
+#if defined HAVE_OPENDIR2 && defined DTF_HIDEW
   if (ISSET (FTS_WHITEOUT))
     oflag = DTF_NODUP | DTF_REWIND;
   else
@@ -868,12 +852,8 @@ fts_build (sp, type)
   return (head);
 }
 
-static u_short
-fts_stat (sp, dp, p, follow)
-     FTS *sp;
-     register FTSENT *p;
-     struct dirent *dp;
-     int follow;
+static unsigned short
+fts_stat (FTS *sp, struct dirent *dp, register FTSENT *p, int follow)
 {
   register FTSENT *t;
   register dev_t dev;
@@ -884,7 +864,7 @@ fts_stat (sp, dp, p, follow)
   /* If user needs stat info, stat buffer already allocated. */
   sbp = ISSET (FTS_NOSTAT) ? &sb : p->fts_statp;
 
-#if defined(DT_WHT) && defined(S_IFWHT)
+#if defined DT_WHT && defined S_IFWHT
   /*
    * Whited-out files don't really exist.  However, there's stat(2) file
    * mask for them, so we set it so that programs (i.e., find) don't have
@@ -963,10 +943,7 @@ fts_stat (sp, dp, p, follow)
 }
 
 static FTSENT *
-fts_sort (sp, head, nitems)
-     FTS *sp;
-     FTSENT *head;
-     register int nitems;
+fts_sort (FTS *sp, FTSENT *head, register int nitems)
 {
   register FTSENT **ap, *p;
 
@@ -998,10 +975,7 @@ fts_sort (sp, head, nitems)
 }
 
 static FTSENT *
-fts_alloc (sp, name, namelen)
-     FTS *sp;
-     const char *name;
-     register int namelen;
+fts_alloc (FTS *sp, const char *name, register int namelen)
 {
   register FTSENT *p;
   size_t len;
@@ -1036,8 +1010,7 @@ fts_alloc (sp, name, namelen)
 }
 
 static void
-fts_lfree (head)
-     register FTSENT *head;
+fts_lfree (register FTSENT *head)
 {
   register FTSENT *p;
 
@@ -1056,9 +1029,7 @@ fts_lfree (head)
  * plus 256 bytes so don't realloc the path 2 bytes at a time.
  */
 static int
-fts_palloc (sp, more)
-     FTS *sp;
-     size_t more;
+fts_palloc (FTS *sp, size_t more)
 {
   sp->fts_pathlen += more + 256;
   sp->fts_path = realloc (sp->fts_path, (size_t) sp->fts_pathlen);
@@ -1070,9 +1041,7 @@ fts_palloc (sp, more)
  * already returned.
  */
 static void
-fts_padjust (sp, addr)
-     FTS *sp;
-     void *addr;
+fts_padjust (FTS *sp, void *addr)
 {
   FTSENT *p;
 
@@ -1094,8 +1063,7 @@ fts_padjust (sp, addr)
 }
 
 static size_t
-fts_maxarglen (argv)
-     char *const *argv;
+fts_maxarglen (char *const *argv)
 {
   size_t len, max;
 
