@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-  2009, 2010 Free Software Foundation, Inc.
+  2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -49,15 +49,12 @@
 /* This code is derived from software contributed to Berkeley by
    Michael Fischbein.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 
-#include <dirent.h>
 #include <errno.h>
 #include "fts.h"
 #include <stdio.h>
@@ -66,9 +63,7 @@
 #include <unistd.h>
 #include <pwd.h>
 #include <grp.h>
-#ifdef HAVE_TERMIOS_H
-# include <termios.h>
-#endif
+#include <termios.h>
 
 #include <intprops.h>
 #include <inttostr.h>
@@ -118,9 +113,7 @@ int f_whiteout;			/* show whiteout entries */
 int rval;
 
 int
-ls_main (argc, argv)
-     int argc;
-     char *argv[];
+ls_main (int argc, char **argv)
 {
   static char dot[] = ".", *dotav[] = { dot, NULL };
   struct winsize win;
@@ -355,9 +348,7 @@ static int output;		/* If anything output. */
  * a superset (may be exact set) of the files to be displayed.
  */
 static void
-traverse (argc, argv, options)
-     int argc, options;
-     char *argv[];
+traverse (int argc, char **argv, int options)
 {
   FTS *ftsp;
   FTSENT *p, *chp;
@@ -366,7 +357,7 @@ traverse (argc, argv, options)
   if ((ftsp = fts_open (argv, options, f_nosort ? NULL : mastercmp)) == NULL)
     {
       fprintf (stderr, "%s: fts_open: %s", argv[0], strerror (errno));
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   display (NULL, fts_children (ftsp, 0));
@@ -418,7 +409,7 @@ traverse (argc, argv, options)
   if (errno)
     {
       fprintf (stderr, "fts_read: %s", strerror (errno));
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
@@ -428,15 +419,14 @@ traverse (argc, argv, options)
  * points to the parent directory of the display list.
  */
 static void
-display (p, list)
-     FTSENT *p, *list;
+display (FTSENT *p, FTSENT *list)
 {
   struct stat *sp;
   DISPLAY d;
   FTSENT *cur;
   NAMES *np;
   unsigned long long maxsize;
-  u_long btotal, maxblock, maxinode, maxlen, maxnlink;
+  unsigned long btotal, maxblock, maxinode, maxlen, maxnlink;
   int bcfile, flen, glen, ulen, maxflags, maxgroup, maxuser;
   int entries, needstats;
   char *user = NULL, *group = NULL, buf[INT_BUFSIZE_BOUND (uintmax_t)];
@@ -542,7 +532,7 @@ display (p, list)
 				ulen + glen + flen + 3)) == NULL)
 		{
 		  fprintf (stderr, "malloc: %s", strerror (errno));
-		  exit (1);
+		  exit (EXIT_FAILURE);
 		}
 
 	      np->user = &np->data[0];
@@ -598,8 +588,7 @@ display (p, list)
  * All other levels use the sort function.  Error entries remain unsorted.
  */
 static int
-mastercmp (a, b)
-     const FTSENT **a, **b;
+mastercmp (const FTSENT **a, const FTSENT **b)
 {
   int a_info, b_info;
 
