@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-  2009, 2010 Free Software Foundation, Inc.
+  2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -67,7 +67,7 @@ extern int f_typedir;		/* add type character for directories */
 typedef struct
 {
   FTSENT *list;
-  u_long btotal;
+  unsigned long btotal;
   int bcfile;
   int entries;
   int maxlen;
