@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-  2009, 2010 Free Software Foundation, Inc.
+  2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -46,14 +46,11 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include <stddef.h>
 #include <string.h>
 
 #define SAPPEND(s) {							\
@@ -69,9 +66,7 @@
  *	are set, return the default string.
  */
 char *
-flags_to_string (flags, def)
-     u_int flags;
-     char *def;
+flags_to_string (u_int flags, char *def)
 {
   static char string[128];
   char *prefix;
@@ -99,9 +94,7 @@ flags_to_string (flags, def)
  *	to the offending token.
  */
 int
-string_to_flags (stringp, setp, clrp)
-     char **stringp;
-     u_int *setp, *clrp;
+string_to_flags (char **stringp, u_int *setp, u_int *clrp)
 {
   if (setp)
     *setp = 0;
