@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-  2009, 2010 Free Software Foundation, Inc.
+  2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -49,9 +49,7 @@
 /* This code is derived from software contributed to Berkeley by
    Michael Fischbein.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -65,8 +63,7 @@
 #include "extern.h"
 
 int
-putname (name)
-     char *name;
+putname (char *name)
 {
   int len;
 
@@ -76,8 +73,8 @@ putname (name)
 }
 
 void
-usage ()
+usage (void)
 {
   fprintf (stderr, "usage: ls [-1ACFLRSTWacdfiklmnopqrstux] [file ...]\n");
-  exit (1);
+  exit (EXIT_FAILURE);
 }
