@@ -1,6 +1,7 @@
 /*
   Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-  2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation,
+  Inc.
 
   This file is part of GNU Inetutils.
 
@@ -66,7 +67,7 @@
  * or implied warranty.
  */
 
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
 TN_Authenticator *findauthenticator (int, int);
 
 void auth_init (char *, int);
