@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-  Foundation, Inc.
+  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free
+  Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -67,11 +67,11 @@
  * or implied warranty.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
-#if defined(AUTHENTICATION)
+#include "unused-parameter.h"
+
+#if defined AUTHENTICATION
 # include <stdio.h>
 # include <sys/types.h>
 # include <signal.h>
@@ -539,7 +539,7 @@ auth_send (unsigned char *data, int cnt)
    *  Therefore, exit on client end.
    */
   printf ("Unable to securely authenticate user ... exit\n");
-  exit (0);
+  exit (EXIT_SUCCESS);
 # endif	/* KANNAN */
 }
 
@@ -651,8 +651,8 @@ auth_finished (TN_Authenticator * ap, int result)
   validuser = result;
 }
 
-static RETSIGTYPE
-auth_intr (int sig ARG_UNUSED)
+static void
+auth_intr (int sig _GL_UNUSED_PARAMETER)
 {
   auth_finished (0, AUTH_REJECT);
 }
