@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-  2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -51,7 +51,7 @@
 
 #include <config.h>
 
-#if defined(KRB5) && defined(FORWARD)
+#if defined KRB5 && defined FORWARD
 # include <stdio.h>
 # include <pwd.h>
 # include <netdb.h>
