@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-  Foundation, Inc.
+  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free
+  Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -67,9 +67,7 @@
  * or implied warranty.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #ifdef	KRB4
 # include <sys/types.h>
@@ -85,8 +83,8 @@
 # include "misc.h"
 
 int kerberos4_cksum (unsigned char *, int);
-int krb_mk_req (KTEXT, char *, char *, char *, u_long);
-int krb_rd_req (KTEXT, char *, char *, u_long, AUTH_DAT *, char *);
+int krb_mk_req (KTEXT, char *, char *, char *, unsigned long);
+int krb_rd_req (KTEXT, char *, char *, unsigned long, AUTH_DAT *, char *);
 int krb_kntoln (AUTH_DAT *, char *);
 int krb_get_cred (char *, char *, char *, CREDENTIALS *);
 int krb_get_lrealm (char *, int);
