@@ -1,6 +1,7 @@
 /*
   Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-  2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation,
+  Inc.
 
   This file is part of GNU Inetutils.
 
@@ -46,9 +47,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
 
@@ -67,7 +66,7 @@ auth_encrypt_init (char *local, char *remote, char *name, int server)
   LocalHostName = local;
   (void) name;
   (void) server;		/* shutup gcc */
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
   auth_init (name, server);
 #endif
 #ifdef	ENCRYPTION
