@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-  2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -58,15 +58,12 @@
  * echoing.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
-#if defined(RSA_ENCPWD) || defined(KRB4_ENCPWD)
+#if defined RSA_ENCPWD || defined KRB4_ENCPWD
 
 # include <stdio.h>
 # include <sys/ioctl.h>
-# include <signal.h>
 # include <setjmp.h>
 
 static jmp_buf env;
