@@ -1,6 +1,6 @@
 /*
-  Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free
-  Software Foundation, Inc.
+  Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+  Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -20,9 +20,7 @@
 /* Written by Simon Josefsson and Nicolas Pouvesle, based on
    kerberos5.c from GNU Inetutils. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #ifdef SHISHI
 # include <stdlib.h>
