@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
+  Copyright (C) 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -16,16 +16,14 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/file.h>
 #include <sys/time.h>
-#include <signal.h>
 
+#include <netinet/in.h>
 /*#include <netinet/ip_icmp.h> -- deliberately not including this */
 #include <arpa/inet.h>
 #include <netdb.h>
@@ -65,7 +63,7 @@ ping_init (int type, int ident)
   fd = socket (AF_INET, SOCK_RAW, proto->p_proto);
   if (fd < 0)
     {
-      if (errno == EPERM)
+      if (errno == EPERM || errno == EACCES)
 	fprintf (stderr, "ping: ping must run as root\n");
       return NULL;
     }
@@ -85,7 +83,7 @@ ping_init (int type, int ident)
   p->ping_count = 0;
   p->ping_interval = PING_DEFAULT_INTERVAL;
   p->ping_datalen = sizeof (icmphdr_t);
-  /* Make sure we use only 16 bits in this field, id for icmp is a u_short.  */
+  /* Make sure we use only 16 bits in this field, id for icmp is a unsigned short.  */
   p->ping_ident = ident & 0xFFFF;
   p->ping_cktab_size = PING_CKTABSIZE;
   gettimeofday (&p->ping_start_time, NULL);
