@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-  2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -17,9 +17,7 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/param.h>
 #include <sys/socket.h>
@@ -57,8 +55,8 @@ extern int ping_router (char *hostname);
 
 PING *ping;
 bool is_root = false;
-u_char *data_buffer;
-u_char *patptr;
+unsigned char *data_buffer;
+unsigned char *patptr;
 int pattern_len = 16;
 int socket_type;
 size_t count = DEFAULT_PING_COUNT;
@@ -67,6 +65,7 @@ size_t data_length = PING_DATALEN;
 unsigned options;
 unsigned long preload = 0;
 int timeout = -1;
+int linger = MAXWAIT;
 int (*ping_type) (char *hostname) = ping_echo;
 
 int (*decode_type (const char *arg)) (char *hostname);
@@ -114,6 +113,7 @@ static struct argp_option argp_options[] = {
    "network", GRP+1},
   {"verbose", 'v', NULL, 0, "verbose output", GRP+1},
   {"timeout", 'w', "N", 0, "stop after N seconds", GRP+1},
+  {"linger", 'W', "N", 0, "number of seconds to wait for response", GRP+1},
 #undef GRP
 #define GRP 20
   {NULL, 0, NULL, 0, "Options valid for --echo requests:", GRP},
@@ -133,7 +133,7 @@ static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
 {
   char *endptr;
-  u_char pattern[16];
+  static unsigned char pattern[16];
   double v;
 
   switch (key)
@@ -181,6 +181,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
       timeout = ping_cvt_number (arg, INT_MAX, 0);
       break;
 
+    case 'W':
+      linger = ping_cvt_number (arg, INT_MAX, 0);
+      break;
+
     case 'R':
       options |= OPT_RROUTE;
       break;
@@ -239,7 +243,7 @@ main (int argc, char **argv)
   int status = 0;
 
   set_program_name (argv[0]);
-  
+
   if (getuid () == 0)
     is_root = true;
 
@@ -250,7 +254,7 @@ main (int argc, char **argv)
   ping = ping_init (ICMP_ECHO, getpid ());
   if (ping == NULL)
     /* ping_init() prints our error message.  */
-    exit (1);
+    exit (EXIT_FAILURE);
 
   ping_set_sockopt (ping, SO_BROADCAST, (char *) &one, sizeof (one));
 
@@ -304,7 +308,7 @@ int (*decode_type (const char *arg)) (char *hostname)
 
 int volatile stop = 0;
 
-RETSIGTYPE
+void
 sig_int (int signal)
 {
   stop = 1;
@@ -404,7 +408,7 @@ ping_run (PING * ping, int (*finish) ())
 	    {
 	      finishing = 1;
 
-	      intvl.tv_sec = MAXWAIT;
+	      intvl.tv_sec = linger;
 	    }
 	  gettimeofday (&last, NULL);
 	}
@@ -437,7 +441,7 @@ send_echo (PING * ping)
 }
 
 int
-ping_finish ()
+ping_finish (void)
 {
   fflush (stdout);
   printf ("--- %s ping statistics ---\n", ping->ping_hostname);
