@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+  Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
   Foundation, Inc.
 
   This file is part of GNU Inetutils.
@@ -17,12 +17,15 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
+
+#ifdef __sun__
+#  define _XPG4_2	1	/* OpenSolaris: msg_control */
+#endif /* __sun__ */
 
 #include <sys/socket.h>
 #include <sys/time.h>
+#include <sys/uio.h>
 #include <signal.h>
 
 #include <netinet/in.h>
@@ -46,7 +49,7 @@
 static PING *ping;
 bool is_root = false;
 unsigned char *data_buffer;
-u_char *patptr;
+unsigned char *patptr;
 int one = 1;
 int pattern_len = 16;
 size_t data_length = PING_DATALEN;
@@ -99,7 +102,7 @@ static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
 {
   char *endptr;
-  u_char pattern[16];
+  static unsigned char pattern[16];
 
   switch (key)
     {
@@ -113,7 +116,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
     case 'f':
       if (!is_root)
-        error (EXIT_FAILURE, errno, NULL);
+        error (EXIT_FAILURE, 0, "flooding needs root privilege");
 
       options |= OPT_FLOOD;
       setbuf (stdout, (char *) NULL);
@@ -126,11 +129,11 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
     case 'l':
       if (!is_root)
-        error (EXIT_FAILURE, errno, NULL);
+        error (EXIT_FAILURE, 0, "preloading needs root privilege");
 
       preload = strtoul (arg, &endptr, 0);
       if (*endptr || preload > INT_MAX)
-        error (EXIT_FAILURE, errno, NULL);
+        error (EXIT_FAILURE, 0, "preload size too large");
 
       break;
 
@@ -178,7 +181,7 @@ main (int argc, char **argv)
   int status = 0;
 
   set_program_name (argv[0]);
-  
+
   if (getuid () == 0)
     is_root = true;
 
@@ -189,7 +192,7 @@ main (int argc, char **argv)
   ping = ping_init (0, getpid ());
   if (ping == NULL)
     /* ping_init() prints our error message.  */
-    exit (1);
+    exit (EXIT_FAILURE);
 
   setsockopt (ping->ping_fd, SOL_SOCKET, SO_BROADCAST, (char *) &one, sizeof (one));
 
@@ -259,7 +262,7 @@ ipaddr2str (struct sockaddr_in6 *from)
 
 static volatile int stop = 0;
 
-static RETSIGTYPE
+static void
 sig_int (int signal)
 {
   stop = 1;
@@ -394,7 +397,7 @@ send_echo (PING * ping)
 }
 
 static int
-ping_finish ()
+ping_finish (void)
 {
   fflush (stdout);
   printf ("--- %s ping statistics ---\n", ping->ping_hostname);
@@ -678,7 +681,7 @@ print_icmp_error (struct sockaddr_in6 *from, struct icmp6_hdr *icmp6, int len)
 }
 
 static int
-echo_finish ()
+echo_finish (void)
 {
   ping_finish ();
   if (ping->ping_num_recv && PING_TIMING (data_length))
@@ -706,8 +709,8 @@ ping_init (int type, int ident)
   fd = socket (PF_INET6, SOCK_RAW, IPPROTO_ICMPV6);
   if (fd < 0)
     {
-      if (errno == EPERM)
-        error (EXIT_FAILURE, errno, NULL);
+      if (errno == EPERM || errno == EACCES)
+        error (EXIT_FAILURE, errno, "raw socket");
 
       return NULL;
     }
@@ -749,7 +752,7 @@ ping_init (int type, int ident)
   p->ping_count = DEFAULT_PING_COUNT;
   p->ping_interval = PING_DEFAULT_INTERVAL;
   p->ping_datalen = sizeof (struct icmp6_hdr);
-  /* Make sure we use only 16 bits in this field, id for icmp is a u_short.  */
+  /* Make sure we use only 16 bits in this field, id for icmp is a unsigned short.  */
   p->ping_ident = ident & 0xFFFF;
   p->ping_cktab_size = PING_CKTABSIZE;
   gettimeofday (&p->ping_start_time, NULL);
