@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -18,15 +18,12 @@
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
 /* NOTE: most of existing routers simply discard ICMP_ADDRESS requests. */
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/file.h>
 #include <sys/time.h>
-#include <signal.h>
 
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -56,7 +53,7 @@ static void print_address (int dupflag, void *closure,
 static void print_address (int dupflag, void *closure,
 			   struct sockaddr_in *dest, struct sockaddr_in *from,
 			   struct ip *ip, icmphdr_t * icmp, int datalen);
-static int address_finish ();
+static int address_finish (void);
 
 int
 ping_address (char *hostname)
@@ -114,7 +111,7 @@ print_address (int dupflag, void *closure,
 
 
 int
-address_finish ()
+address_finish (void)
 {
   return 0;
 }
