@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -17,15 +17,12 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/file.h>
 #include <sys/time.h>
-#include <signal.h>
 
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -97,7 +94,7 @@ ping_echo (char *hostname)
       rspace[IPOPT_OFFSET] = IPOPT_MINOFF;
       if (setsockopt (ping->ping_fd, IPPROTO_IP,
 		      IP_OPTIONS, rspace, sizeof (rspace)) < 0)
-        error (EXIT_FAILURE, errno, NULL);
+        error (EXIT_FAILURE, errno, "setsockopt");
 #else
       error (EXIT_FAILURE, 0, "record route not available in this "
              "implementation.");
@@ -285,10 +282,10 @@ static void
 print_ip_header (struct ip *ip)
 {
   int hlen;
-  u_char *cp;
+  unsigned char *cp;
 
   hlen = ip->ip_hl << 2;
-  cp = (u_char *) ip + 20;	/* point to options */
+  cp = (unsigned char *) ip + 20;	/* point to options */
 
   printf
     ("Vr HL TOS  Len   ID Flg  off TTL Pro  cks      Src      Dst Data\n");
@@ -309,13 +306,13 @@ void
 print_ip_data (icmphdr_t * icmp, void *data)
 {
   int hlen;
-  u_char *cp;
+  unsigned char *cp;
   struct ip *ip = &icmp->icmp_ip;
 
   print_ip_header (ip);
 
   hlen = ip->ip_hl << 2;
-  cp = (u_char *) ip + hlen;
+  cp = (unsigned char *) ip + hlen;
 
   if (ip->ip_p == 6)
     printf ("TCP: from port %u, to port %u (decimal)\n",
@@ -395,12 +392,12 @@ print_icmp_header (struct sockaddr_in *from,
 void
 print_ip_opt (struct ip *ip, int hlen)
 {
-  u_char *cp;
+  unsigned char *cp;
   int i, j, l;
   static int old_rrlen;
   static char old_rr[MAX_IPOPTLEN];
 
-  cp = (u_char *) (ip + 1);
+  cp = (unsigned char *) (ip + 1);
 
   for (; hlen > (int) sizeof (struct ip); --hlen, ++cp)
     switch (*cp)
@@ -452,7 +449,7 @@ print_ip_opt (struct ip *ip, int hlen)
 	if (i <= 0)
 	  continue;
 	if (i == old_rrlen
-	    && cp == (u_char *) (ip + 1) + 2
+	    && cp == (unsigned char *) (ip + 1) + 2
 	    && !memcmp ((char *) cp, old_rr, i) && !(options & OPT_FLOOD))
 	  {
 	    printf ("\t (same route)");
@@ -515,7 +512,7 @@ print_ip_opt (struct ip *ip, int hlen)
 }
 
 int
-echo_finish ()
+echo_finish (void)
 {
   ping_finish ();
   if (ping->ping_num_recv && PING_TIMING (data_length))
