@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -21,7 +21,7 @@
 
 extern unsigned options;
 extern PING *ping;
-extern u_char *data_buffer;
+extern unsigned char *data_buffer;
 extern size_t data_length;
 
 extern int ping_run (PING * ping, int (*finish) ());
