@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -17,15 +17,12 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/file.h>
 #include <sys/time.h>
-#include <signal.h>
 
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -53,7 +50,7 @@ static void print_timestamp (int dupflag, void *closure,
 			     struct sockaddr_in *dest,
 			     struct sockaddr_in *from, struct ip *ip,
 			     icmphdr_t * icmp, int datalen);
-static int timestamp_finish ();
+static int timestamp_finish (void);
 
 int
 ping_timestamp (char *hostname)
@@ -109,7 +106,7 @@ print_timestamp (int dupflag, void *closure,
 }
 
 int
-timestamp_finish ()
+timestamp_finish (void)
 {
   return ping_finish ();
 }
