@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-  2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+  2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
   Foundation, Inc.
 
   This file is part of GNU Inetutils.
@@ -98,9 +98,7 @@
  * Comment lines are indicated by a `#' in column 1.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -138,6 +136,7 @@
 #include "libinetutils.h"
 #include "argcv.h"
 #include "version-etc.h"
+#include "unused-parameter.h"
 
 #ifndef EAI_ADDRFAMILY
 # define EAI_ADDRFAMILY 1
@@ -165,6 +164,8 @@ char **config_files;
 
 static bool env_option = false;	       /* Set environment variables */
 static bool resolve_option = false;    /* Resolve IP addresses */
+static bool pidfile_option = true;     /* Record the PID in a file */
+static const char *pid_file = PATH_INETDPID;
 
 const char args_doc[] = "[CONF-FILE [CONF-DIR]]...";
 const char doc[] = "Internet super-server.";
@@ -187,6 +188,9 @@ static struct argp_option argp_options[] = {
    "turn on debugging, run in foreground mode", GRP+1},
   {"environment", OPT_ENVIRON, NULL, 0,
    "pass local and remote socket information in environment variables", GRP+1},
+  { "pidfile", 'p', "PIDFILE", OPTION_ARG_OPTIONAL,
+    "override pidfile (default: \"" PATH_INETDPID "\")",
+    GRP+1 },
   {"rate", 'R', "NUMBER", 0,
    "maximum invocation rate (per minute)", GRP+1},
   {"resolve", OPT_RESOLVE, NULL, 0,
@@ -213,6 +217,13 @@ parse_opt (int key, char *arg, struct argp_state *state)
       env_option = true;
       break;
 
+    case 'p':
+      if (arg && strlen (arg))
+	pid_file = arg;
+      else
+	pidfile_option = false;
+      break;
+
     case 'R':
       number = strtol (arg, &p, 0);
       if (number < 1 || *p)
@@ -256,6 +267,7 @@ struct servtab
   sa_family_t se_family;	/* address family of the socket */
   char se_v4mapped;		/* 1 = accept v4mapped connection, 0 = don't */
   struct sockaddr_storage se_ctrladdr;	/* bound address */
+  socklen_t se_addrlen;		/* exact address length in use */
   unsigned se_refcnt;
   int se_count;			/* number started since se_time */
   struct timeval se_time;	/* start of se_count */
@@ -328,7 +340,7 @@ bi_lookup (const struct servtab *sep)
 
 /* Signal handling */
 
-#if defined(HAVE_SIGACTION)
+#if defined HAVE_SIGACTION
 # define SIGSTATUS sigset_t
 # define sigstatus_empty(s) sigemptyset(&s)
 # define inetd_pause(s) sigsuspend(&s)
@@ -339,9 +351,9 @@ bi_lookup (const struct servtab *sep)
 #endif
 
 void
-signal_set_handler (int signo, RETSIGTYPE (*handler) ())
+signal_set_handler (int signo, void (*handler) ())
 {
-#if defined(HAVE_SIGACTION)
+#if defined HAVE_SIGACTION
   struct sigaction sa;
   memset ((char *) &sa, 0, sizeof (sa));
   sigemptyset (&sa.sa_mask);
@@ -450,7 +462,7 @@ run_service (int ctrl, struct servtab *sep)
 }
 
 void
-reapchild (int signo ARG_UNUSED)
+reapchild (int signo _GL_UNUSED_PARAMETER)
 {
   int status;
   pid_t pid;
@@ -524,12 +536,15 @@ void
 print_service (const char *action, struct servtab *sep)
 {
   fprintf (stderr,
-	   "%s:%d: %s: %s:%s proto=%s, wait=%d, max=%u, user=%s builtin=%lx server=%s\n",
+	   "%s:%d: %s: %s:%s proto=%s, wait=%d, max=%u, user=%s builtin=%s server=%s\n",
 	   sep->se_file, sep->se_line,
 	   action,
-	   sep->se_node ? sep->se_node : "*", sep->se_service, sep->se_proto,
+	   ISMUX (sep) ? (ISMUXPLUS (sep) ? "tcpmuxplus" : "tcpmux")
+		      : (sep->se_node ? sep->se_node : "*"),
+	   sep->se_service, sep->se_proto,
 	   sep->se_wait, sep->se_max, sep->se_user,
-	   (long) sep->se_bi, sep->se_server);
+	   sep->se_bi ? sep->se_bi->bi_service : "no",
+	   sep->se_server);
 }
 
 
@@ -549,7 +564,7 @@ setup (struct servtab *sep)
       if (errno == EAFNOSUPPORT && sep->se_family == AF_INET6
 	  && sep->se_v4mapped)
 	{
-	  /* Fall back to IPv6 silently.  */
+	  /* Fall back to IPv4 silently.  */
 	  sep->se_family = AF_INET;
 	  goto tryagain;
 	}
@@ -565,8 +580,8 @@ setup (struct servtab *sep)
   if (sep->se_family == AF_INET6)
     {
       /* Reverse the value of SEP->se_v4mapped, since otherwise if
-	 using `tcp' as a protocol type all connections will be mapped
-	 to IPv6, and with `tcp6' they get mapped IPv4 mapped to
+	 using `tcp6' as a protocol type, all connections will be
+	 mapped to IPv6, and with `tcp6only', IPv4 gets mapped to
 	 IPv6.  */
       int val = sep->se_v4mapped ? 0 : 1;
       if (setsockopt (sep->se_fd, IPPROTO_IPV6, IPV6_V6ONLY,
@@ -587,14 +602,14 @@ setup (struct servtab *sep)
     syslog (LOG_ERR, "setsockopt (SO_REUSEADDR): %m");
 
   err = bind (sep->se_fd, (struct sockaddr *) &sep->se_ctrladdr,
-	      sizeof (sep->se_ctrladdr));
+	      sep->se_addrlen);
   if (err < 0)
     {
       /* If we can't bind with AF_INET6 try again with AF_INET.  */
       if ((errno == EADDRNOTAVAIL || errno == EAFNOSUPPORT)
 	  && sep->se_family == AF_INET6 && sep->se_v4mapped)
 	{
-	  /* Fall back to IPv6 silently.  */
+	  /* Fall back to IPv4 silently.  */
 	  sep->se_family = AF_INET;
 	  close (sep->se_fd);
 	  goto tryagain;
@@ -633,7 +648,7 @@ servent_setup (struct servtab *sep)
 }
 
 void
-retry (int signo ARG_UNUSED)
+retry (int signo _GL_UNUSED_PARAMETER)
 {
   struct servtab *sep;
 
@@ -735,17 +750,44 @@ enter (struct servtab *cp)
   return sep;
 }
 
+#define IPV4_NUMCHARS ".0123456789"
+#define IPV6_NUMCHARS ".:0123456789abcdefABCDEF"
+
 int
 inetd_getaddrinfo (struct servtab *sep, int proto, struct addrinfo **result)
 {
   struct addrinfo hints;
+#ifdef IPV6
+  bool numeric_address = false;
+
+  /* In case a numerical address is supplied, which does not
+     apply to the indicated domain, a non-local resolver
+     will wait in vain until time out occurs, thus blocking.
+     Avoid this by falling back to numerical host resolving
+     when address string seems to be numerical.  */
+
+  /* Purely numeric address?  Separate criteria for IPv4 and IPv6,
+     since IPv6 allows hexadecimal coding and IPv4 mapping!  */
+  if (sep->se_node
+      && (strspn (sep->se_node, IPV4_NUMCHARS) == strlen (sep->se_node)
+	  || (strchr (sep->se_node, ':')
+	      && strspn (sep->se_node, IPV6_NUMCHARS)) ) )
+    numeric_address = true;
+  else
+    if (debug && sep->se_node)
+      fprintf (stderr, "Resolving address: %s\n", sep->se_node);
+#endif /* IPV6 */
 
   memset (&hints, 0, sizeof (hints));
 
   hints.ai_flags = AI_PASSIVE;
 #ifdef AI_V4MAPPED
-  if (sep->se_v4mapped)
+  if (sep->se_v4mapped && (sep->se_family != AF_INET))
     hints.ai_flags |= AI_V4MAPPED;
+#endif
+#ifdef IPV6
+  if (numeric_address)
+    hints.ai_flags |= AI_NUMERICHOST;
 #endif
   hints.ai_family = sep->se_family;
   hints.ai_socktype = sep->se_socktype;
@@ -781,7 +823,7 @@ expand_enter (struct servtab *sep)
   if (err == EAI_ADDRFAMILY
       && sep->se_family == AF_INET6 && sep->se_v4mapped)
     {
-      /* Fall back to IPv6 silently.  */
+      /* Fall back to IPv4 silently.  */
       sep->se_family = AF_INET;
       err = inetd_getaddrinfo (sep, proto->p_proto, &result);
     }
@@ -802,7 +844,9 @@ expand_enter (struct servtab *sep)
 
   for (rp = result; rp != NULL; rp = rp->ai_next)
     {
+      memset (&sep->se_ctrladdr, 0, sizeof (sep->se_ctrladdr));
       memcpy (&sep->se_ctrladdr, rp->ai_addr, rp->ai_addrlen);
+      sep->se_addrlen = rp->ai_addrlen;
       cp = enter (sep);
       servent_setup (cp);
     }
@@ -940,7 +984,7 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
       sep->se_line = *line;
 
       node = argv[INETD_SERVICE];
-      service = strchr (node, ':');
+      service = strrchr (node, ':');
       if (!service)
         {
 	  if (global_serv_node)
@@ -1004,9 +1048,8 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
       sep->se_proto = newstr (argv[INETD_PROTOCOL]);
 
 #ifdef IPV6
-      /* We default to IPv6, in setup() we'll fall back to IPv4 if
-         it doesn't work.  */
-      sep->se_family = AF_INET6;
+      /* We default to IPv4. */
+      sep->se_family = AF_INET;
       sep->se_v4mapped = 1;
 
       if ((strncmp (sep->se_proto, "tcp", 3) == 0)
@@ -1016,6 +1059,9 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
 	    {
 	      sep->se_family = AF_INET6;
 	      sep->se_v4mapped = 0;
+	      /* Check for tcp6only and udp6only.  */
+	      if (strcmp (&sep->se_proto[3], "6only") == 0)
+	        sep->se_v4mapped = 0;
 	    }
 	  else if (sep->se_proto[3] == '4')
 	    {
@@ -1110,6 +1156,18 @@ getconfigent (FILE *fconfig, const char *file, size_t *line)
 	  sep->se_argv[i] = argv[INETD_SERVER_ARGS + i];
 	  argv[INETD_SERVER_ARGS + i] = 0;
 	}
+
+      /* If no arguments are provided, use server name as argv[0].  */
+      if (sep->se_argc == 1)
+	{
+	  const char *argv0 = strrchr (sep->se_server, '/');
+	  if (argv0)
+	    argv0++;
+	  else
+	    argv0 = sep->se_server;
+          sep->se_argv[0] = newstr (argv0);
+	}
+
       sep->se_argv[i] = NULL;
       break;
     }
@@ -1152,7 +1210,11 @@ nextconfig (const char *file)
 	}
       else
 	expand_enter (sep);
-      freeconfig (sep);
+
+      if (serv_node)
+	free (sep->se_node);
+      else
+	freeconfig (sep);
     }
   endconfig (fconfig);
   /*
@@ -1179,7 +1241,7 @@ nextconfig (const char *file)
 }
 
 void
-fix_tcpmux ()
+fix_tcpmux (void)
 {
   struct servtab *sep;
   int need_tcpmux = 0;
@@ -1208,9 +1270,9 @@ fix_tcpmux ()
       struct servtab serv;
       memset (&serv, 0, sizeof (serv));
 
+      serv.se_file = "fix_tcpmux";
       serv.se_service = newstr ("tcpmux");
       serv.se_socktype = SOCK_STREAM;
-      serv.se_proto = newstr ("tcp");
       serv.se_checked = 1;
       serv.se_user = newstr ("root");
       serv.se_bi = bi_lookup (&serv);
@@ -1219,8 +1281,8 @@ fix_tcpmux ()
 	  /* Should not happen */
 	  freeconfig (&serv);
 	  if (debug)
-	    fprintf (stderr, "INETERNAL ERROR: could not found tcpmux built-in");
-	  syslog (LOG_ERR, "INETERNAL ERROR: could not found tcpmux built-in");
+	    fprintf (stderr, "INTERNAL ERROR: could not find tcpmux built-in");
+	  syslog (LOG_ERR, "INTERNAL ERROR: could not find tcpmux built-in");
 	  return;
 	}
       serv.se_wait = serv.se_bi->bi_wait;
@@ -1228,15 +1290,17 @@ fix_tcpmux ()
       serv.se_fd = -1;
       serv.se_type = NORM_TYPE;
 #ifdef IPV6
+      serv.se_proto = newstr ("tcp6");
       serv.se_family = AF_INET6;
       serv.se_v4mapped = 1;
 #else
+      serv.se_proto = newstr ("tcp");
       serv.se_family = AF_INET;
 #endif
       if (debug)
 	fprintf (stderr, "inserting default tcpmux entry\n");
       syslog (LOG_INFO, "inserting default tcpmux entry");
-      enter (&serv);
+      expand_enter (&serv);
     }
 }
 
@@ -1361,7 +1425,7 @@ echo_stream (int s, struct servtab *sep)
   while ((i = read (s, buffer, sizeof buffer)) > 0
 	 && write (s, buffer, i) > 0)
     ;
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 /* Echo service -- echo data back */
@@ -1399,7 +1463,7 @@ discard_stream (int s, struct servtab *sep)
       if (ret == 0 || errno != EINTR)
 	break;
     }
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 void
@@ -1459,7 +1523,7 @@ chargen_stream (int s, struct servtab *sep)
       if (write (s, text, sizeof text) != sizeof text)
 	break;
     }
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 /* Character generator */
@@ -1519,7 +1583,7 @@ machtime (void)
 	fprintf (stderr, "Unable to get time of day\n");
       return 0L;
     }
-#define OFFSET ((u_long)25567 * 24*60*60)
+#define OFFSET ((unsigned long)25567 * 24*60*60)
   return (htonl ((long) (tv.tv_sec + OFFSET)));
 #undef OFFSET
 }
@@ -1671,20 +1735,26 @@ tcpmux (int s, struct servtab *sep)
 	}
     }
   strwrite (s, "-Service not available\r\n");
-  exit (1);
+  exit (EXIT_FAILURE);
 }
 
 /* Set TCP environment variables, modelled after djb's ucspi-tcp tools:
    http://cr.yp.to/ucspi-tcp/environment.html
-   FIXME: This needs support for IPv6
 */
 void
-prepenv (int ctrl, struct sockaddr_in sa_client)
+prepenv (int ctrl, struct sockaddr *sa_client, socklen_t sa_len)
 {
   char str[16];
-  char *ip;
-  struct hostent *host;
+  /* IP is used both for numeric addresses and for symbolic ones.
+   * Being statically allocated, and only for logging purposes,
+   * a full MAXPATHLEN is exaggerated, so a compromise is made.  */
+  char ip[4 * INET6_ADDRSTRLEN];
+  int ret;
+#ifdef IPV6
+  struct sockaddr_storage sa_server;
+#else
   struct sockaddr_in sa_server;
+#endif
   socklen_t len = sizeof (sa_server);
 
   setenv ("PROTO", "TCP", 1);
@@ -1699,47 +1769,58 @@ prepenv (int ctrl, struct sockaddr_in sa_client)
     syslog (LOG_WARNING, "getsockname(): %m");
   else
     {
-      ip = inet_ntoa (sa_server.sin_addr);
-      if (ip)
+      ret = getnameinfo ((struct sockaddr *) &sa_server, len,
+			  ip, sizeof (ip), str, sizeof (str),
+			  NI_NUMERICHOST | NI_NUMERICSERV);
+      if (ret == 0)
 	{
 	  if (setenv ("TCPLOCALIP", ip, 1) < 0)
 	    syslog (LOG_WARNING, "setenv (TCPLOCALIP): %m");
-	}
+	  else if (debug)
+	    fprintf (stderr, "Assigned TCPLOCALIP = %s\n", ip);
 
-      snprintf (str, sizeof (str), "%d", ntohs (sa_server.sin_port));
-      setenv ("TCPLOCALPORT", str, 1);
+	  if (setenv ("TCPLOCALPORT", str, 1) < 0)
+	    syslog (LOG_WARNING, "setenv (TCPLOCALPORT): %m");
+	}
+      else
+	syslog (LOG_WARNING, "getnameinfo: %s", gai_strerror (ret));
 
       if (resolve_option)
 	{
-	  if ((host = gethostbyaddr ((char *) &sa_server.sin_addr,
-				     sizeof (sa_server.sin_addr),
-				     AF_INET)) == NULL)
-	    syslog (LOG_WARNING, "gethostbyaddr: %m");
-	  else if (setenv ("TCPLOCALHOST", host->h_name, 1) < 0)
+	  ret = getnameinfo ((struct sockaddr *) &sa_server, len,
+			      ip, sizeof (ip), NULL, 0, 0);
+	  if (ret != 0)
+	    syslog (LOG_WARNING, "getnameinfo: %s", gai_strerror (ret));
+	  else if (setenv ("TCPLOCALHOST", ip, 1) < 0)
 	    syslog (LOG_WARNING, "setenv(TCPLOCALHOST): %m");
 	}
     }
 
-  ip = inet_ntoa (sa_client.sin_addr);
-  if (ip)
+  ret = getnameinfo (sa_client, sa_len, ip, sizeof (ip), str, sizeof (str),
+		      NI_NUMERICHOST | NI_NUMERICSERV);
+  if (ret == 0)
     {
       if (setenv ("TCPREMOTEIP", ip, 1) < 0)
 	syslog (LOG_WARNING, "setenv(TCPREMOTEIP): %m");
-    }
+      else if (debug)
+	fprintf (stderr, "Assigned TCPREMOTEIP = %s\n", ip);
 
-  snprintf (str, sizeof (str), "%d", ntohs (sa_client.sin_port));
-  if (setenv ("TCPREMOTEPORT", str, 1) < 0)
-    syslog (LOG_WARNING, "setenv(TCPREMOTEPORT): %m");
+      if (setenv ("TCPREMOTEPORT", str, 1) < 0)
+	syslog (LOG_WARNING, "setenv(TCPREMOTEPORT): %m");
 
-  if (resolve_option)
-    {
-      if ((host = gethostbyaddr ((char *) &sa_client.sin_addr,
-				 sizeof (sa_client.sin_addr),
-				 AF_INET)) == NULL)
-	syslog (LOG_WARNING, "gethostbyaddr: %m");
-      else if (setenv ("TCPREMOTEHOST", host->h_name, 1) < 0)
-	syslog (LOG_WARNING, "setenv(TCPREMOTEHOST): %m");
+      if (resolve_option)
+	{
+	  ret = getnameinfo (sa_client, sa_len, ip, sizeof (ip), NULL, 0, 0);
+	  if (ret != 0)
+	    syslog (LOG_WARNING, "getnameinfo: %s", gai_strerror (ret));
+	  else if (setenv ("TCPREMOTEHOST", ip, 1) < 0)
+	    syslog (LOG_WARNING, "setenv(TCPREMOTEHOST): %m");
+	  else if (debug)
+	    fprintf (stderr, "Assigned TCPREMOTEHOST = %s\n", ip);
+	}
     }
+  else
+    syslog (LOG_WARNING, "getnameinfo: %s", gai_strerror (ret));
 }
 
 
@@ -1791,15 +1872,18 @@ main (int argc, char *argv[], char *envp[])
 
   openlog ("inetd", LOG_PID | LOG_NOWAIT, LOG_DAEMON);
 
+  if (pidfile_option)
   {
-    FILE *fp = fopen (PATH_INETDPID, "w");
+    FILE *fp = fopen (pid_file, "w");
     if (fp != NULL)
       {
+	if (debug)
+	  fprintf(stderr, "Using pid-file at \"%s\".\n", pid_file);
 	fprintf (fp, "%d\n", getpid ());
 	fclose (fp);
       }
     else
-      syslog (LOG_CRIT, "can't open %s: %s\n", PATH_INETDPID,
+      syslog (LOG_CRIT, "can't open %s: %s\n", pid_file,
 	      strerror (errno));
   }
 
@@ -1850,7 +1934,11 @@ main (int argc, char *argv[], char *envp[])
 	      fprintf (stderr, "someone wants %s\n", sep->se_service);
 	    if (!sep->se_wait && sep->se_socktype == SOCK_STREAM)
 	      {
+#ifdef IPV6
+		struct sockaddr_storage sa_client;
+#else
 		struct sockaddr_in sa_client;
+#endif
 		socklen_t len = sizeof (sa_client);
 
 		ctrl = accept (sep->se_fd, (struct sockaddr *) &sa_client,
@@ -1865,7 +1953,7 @@ main (int argc, char *argv[], char *envp[])
 		    continue;
 		  }
 		if (env_option)
-		  prepenv (ctrl, sa_client);
+		  prepenv (ctrl, (struct sockaddr *) &sa_client, len);
 	      }
 	    else
 	      ctrl = sep->se_fd;
