@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -47,9 +47,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -228,7 +226,7 @@ static struct argp argp = {
 };
 
 struct passwd *pwd;
-u_short port;
+unsigned short port;
 uid_t userid;
 int errs, rem;
 
@@ -282,11 +280,11 @@ main (int argc, char *argv[])
   sp = getservbyname (shell = "shell", "tcp");
 #endif
   if (sp == NULL)
-    error (1, 0, "%s/tcp: unknown service", shell);
+    error (EXIT_FAILURE, 0, "%s/tcp: unknown service", shell);
   port = sp->s_port;
 
   if ((pwd = getpwuid (userid = getuid ())) == NULL)
-    error (1, 0, "unknown user %d", (int) userid);
+    error (EXIT_FAILURE, 0, "unknown user %d", (int) userid);
 
   rem = STDIN_FILENO;		/* XXX */
 
@@ -306,11 +304,17 @@ main (int argc, char *argv[])
     }
 
   if (argc < 2)
-    error (1, 0, "not enough arguments");
+    error (EXIT_FAILURE, 0, "not enough arguments");
 
   if (argc > 2)
     targetshouldbedirectory = 1;
 
+#ifndef KERBEROS
+  /* We must be setuid root.  */
+  if (geteuid ())
+    error (EXIT_FAILURE, 0, "must be setuid root.");
+#endif
+
   /* Command to be executed on remote system using "rsh". */
 #ifdef	KERBEROS
   rc = asprintf (&command, "rcp%s%s%s%s", iamrecursive ? " -r" : "",
@@ -363,7 +367,7 @@ toremote (char *targ, int argc, char *argv[])
       if (*tuser == '\0')
 	tuser = NULL;
       else if (!okname (tuser))
-	exit (1);
+	exit (EXIT_FAILURE);
     }
   else
     {
@@ -424,15 +428,16 @@ toremote (char *targ, int argc, char *argv[])
 		rem = rcmd (&host, port, pwd->pw_name,
 			    tuser ? tuser : pwd->pw_name, bp, 0);
 	      if (rem < 0)
-		exit (1);
-#if defined (IP_TOS) && defined (IPPROTO_IP) && defined (IPTOS_THROUGHPUT)
+		exit (EXIT_FAILURE);
+#if defined IP_TOS && defined IPPROTO_IP && defined IPTOS_THROUGHPUT
 	      tos = IPTOS_THROUGHPUT;
 	      if (setsockopt (rem, IPPROTO_IP, IP_TOS,
 			      (char *) &tos, sizeof (int)) < 0)
-		error (0, errno, "TOS (ignored)");
+		if (errno != ENOPROTOOPT)
+		  error (0, errno, "TOS (ignored)");
 #endif
 	      if (response () < 0)
-		exit (1);
+		exit (EXIT_FAILURE);
 	      free (bp);
 	      setuid (userid);
 	    }
@@ -494,11 +499,12 @@ tolocal (int argc, char *argv[])
 	  continue;
 	}
       seteuid (userid);
-#if defined (IP_TOS) && defined (IPPROTO_IP) && defined (IPTOS_THROUGHPUT)
+#if defined IP_TOS && defined IPPROTO_IP && defined IPTOS_THROUGHPUT
       tos = IPTOS_THROUGHPUT;
       if (setsockopt (rem, IPPROTO_IP, IP_TOS, (char *) &tos, sizeof (int)) <
 	  0)
-	error (0, errno, "TOS (ignored)");
+	if (errno != ENOPROTOOPT)
+	  error (0, errno, "TOS (ignored)");
 #endif
       sink (1, argv + argc - 1);
       seteuid (0);
@@ -582,7 +588,7 @@ source (int argc, char *argv[])
 #define RCP_MODEMASK	(S_ISUID|S_ISGID|S_ISVTX|S_IRWXU|S_IRWXG|S_IRWXO)
       snprintf (buf, sizeof buf,
 		(sizeof (stb.st_size) > sizeof (long)
-		 ? "C%04o %qd %s\n"
+		 ? "C%04o %lld %s\n"
 		 : "C%04o %ld %s\n"),
 		stb.st_mode & RCP_MODEMASK, stb.st_size, last);
       write (rem, buf, strlen (buf));
@@ -728,7 +734,7 @@ sink (int argc, char *argv[])
   if (argc != 1)
     {
       run_err ("ambiguous target");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   targ = *argv;
   if (targetshouldbedirectory)
@@ -757,7 +763,7 @@ sink (int argc, char *argv[])
 	  if (iamremote == 0)
 	    write (STDERR_FILENO, buf + 1, strlen (buf + 1));
 	  if (buf[0] == '\02')
-	    exit (1);
+	    exit (EXIT_FAILURE);
 	  ++errs;
 	  continue;
 	}
@@ -803,7 +809,7 @@ sink (int argc, char *argv[])
 	  if (first)
 	    {
 	      run_err ("%s", cp);
-	      exit (1);
+	      exit (EXIT_FAILURE);
 	    }
 	  SCREWUP ("expected control record");
 	}
@@ -908,7 +914,7 @@ sink (int argc, char *argv[])
 	      if (j <= 0)
 		{
 		  run_err ("%s", j ? strerror (errno) : "dropped connection");
-		  exit (1);
+		  exit (EXIT_FAILURE);
 		}
 	      amt -= j;
 	      cp += j;
@@ -990,7 +996,7 @@ sink (int argc, char *argv[])
     }
 screwup:
   run_err ("protocol error: %s", why);
-  exit (1);
+  exit (EXIT_FAILURE);
 }
 
 #ifdef KERBEROS
@@ -1017,7 +1023,7 @@ again:
 	{
 	  use_kerberos = 0;
 	  if ((sp = getservbyname ("shell", "tcp")) == NULL)
-	    error (1, 0, "unknown service shell/tcp");
+	    error (EXIT_FAILURE, 0, "unknown service shell/tcp");
 	  if (errno == ECONNREFUSED)
 	    oldw ("remote host doesn't support Kerberos");
 	  else if (errno == ENOENT)
@@ -1030,7 +1036,7 @@ again:
     {
 # ifdef CRYPT
       if (doencrypt)
-	error (1, 0, "the -x option requires Kerberos authentication");
+	error (EXIT_FAILURE, 0, "the -x option requires Kerberos authentication");
 # endif
       rem = rcmd (host, port, locuser, user, bp, 0);
     }
@@ -1039,7 +1045,7 @@ again:
 #endif /* KERBEROS */
 
 int
-response ()
+response (void)
 {
   char ch, *cp, resp, rbuf[BUFSIZ];
 
@@ -1069,7 +1075,7 @@ response ()
       ++errs;
       if (resp == 1)
 	return -1;
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
@@ -1142,7 +1148,7 @@ verifydir (char *cp)
       errno = ENOTDIR;
     }
   run_err ("%s: %s", cp, strerror (errno));
-  exit (1);
+  exit (EXIT_FAILURE);
 }
 
 int
@@ -1171,7 +1177,7 @@ bad:
 int
 susystem (char *s, int userid)
 {
-  sig_t istat, qstat;
+  sighandler_t istat, qstat;
   int status;
   pid_t pid;
 
@@ -1224,10 +1230,10 @@ allocbuf (BUF * bp, int fd, int blksize)
   return (bp);
 }
 
-RETSIGTYPE
+void
 lostconn (int signo)
 {
   if (!iamremote)
     error (0, 0, "lost connection");
-  exit (1);
+  exit (EXIT_FAILURE);
 }
