@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -51,9 +51,7 @@
  * rlogin - remote login
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -104,12 +102,13 @@
 
 #include <progname.h>
 #include "libinetutils.h"
+#include "unused-parameter.h"
 
 #ifdef SHISHI
 # define REALM_SZ 1040
 #endif
 
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
 int use_kerberos = 1, doencrypt;
 # ifdef KERBEROS
 #  include <kerberosIV/des.h>
@@ -145,7 +144,7 @@ int wlen;
 # define TIOCPKT_DOSTOP          0x20
 #endif /*TIOCPKT*/
 /* The server sends us a TIOCPKT_WINDOW notification when it starts up.
-   The value for this (0x80) can not overlap the kernel defined TIOCPKT_xxx
+   The value for this (0x80) cannot overlap the kernel defined TIOCPKT_xxx
    values.  */
 #ifndef TIOCPKT_WINDOW
 # define TIOCPKT_WINDOW	0x80
@@ -170,7 +169,7 @@ int dflag = 0;
 int noescape;
 char * host = NULL;
 char * user = NULL;
-u_char escapechar = '~';
+unsigned char escapechar = '~';
 
 #ifdef OLDSUN
 
@@ -188,32 +187,32 @@ int get_window_size (int, struct winsize *);
 #endif
 struct winsize winsize;
 
-RETSIGTYPE catch_child (int);
-RETSIGTYPE copytochild (int);
+void catch_child (int);
+void copytochild (int);
 void doit (sigset_t *);
 void done (int);
 void echo (char);
 u_int getescape (char *);
-RETSIGTYPE lostpeer (int);
+void lostpeer (int);
 void mode (int);
-RETSIGTYPE oob (int);
+void oob (int);
 int reader (sigset_t *);
 void sendwindow (void);
 void setsignal (int);
 int speed (int);
 unsigned int speed_translate (unsigned int);
-RETSIGTYPE sigwinch (int);
+void sigwinch (int);
 void stop (char);
 void writer (void);
-RETSIGTYPE writeroob (int);
+void writeroob (int);
 
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
 void warning (const char *, ...);
 #endif
 
-extern sig_t setsig (int, sig_t);
+extern sighandler_t setsig (int, sighandler_t);
 
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
 # define OPTIONS	"8EKde:k:l:xhV"
 #else
 # define OPTIONS	"8EKde:l:hV"
@@ -232,7 +231,7 @@ static struct argp_option argp_options[] = {
   {"no-escape", 'E', NULL, 0, "stops any character from being recognized as "
    "an escape character", GRP+1},
   {"user", 'l', "USER", 0, "run as USER on the remote system", GRP+1},
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
 # ifdef ENCRYPTION
   {"encrypt", 'x', NULL, 0, "turns on DES encryption for all data passed via "
    "the rlogin session", GRP+1},
@@ -276,7 +275,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       user = arg;
       break;
 
-#if defined (KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
 # ifdef ENCRYPTION
     case 'x':
       doencrypt = 1;
@@ -316,7 +315,7 @@ main (int argc, char *argv[])
 {
   struct passwd *pw;
   struct servent *sp;
-  sigset_t smask;
+  sigset_t smask, osmask;
   uid_t uid;
   int index;
   int term_speed;
@@ -324,9 +323,9 @@ main (int argc, char *argv[])
 
   set_program_name (argv[0]);
 
-  /* Traditionnaly, if a symbolic link was made to the rlogin binary
-     rlogin --> hostname
-     hostname will be use as the name of the server to login too.  */
+  /* Traditionally, if a symbolic link was made to the rlogin binary
+         hostname --> rlogin
+     then hostname will be used as the name of the server to access.  */
   {
     char *p = strrchr (argv[0], '/');
     if (p)
@@ -349,11 +348,11 @@ main (int argc, char *argv[])
 
   /* We must be uid root to access rcmd().  */
   if (geteuid ())
-    error (1, 0, "must be setuid root.\n");
+    error (EXIT_FAILURE, 0, "must be setuid root.\n");
 
   /* Get the name of the user invoking us: the client-user-name.  */
   if (!(pw = getpwuid (uid = getuid ())))
-    error (1, 0, "unknown user id.");
+    error (EXIT_FAILURE, 0, "unknown user id.");
 
   /* Accept user1@host format, though "-l user2" overrides user1 */
   {
@@ -370,7 +369,7 @@ main (int argc, char *argv[])
   }
 
   sp = NULL;
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
   if (use_kerberos)
     {
       sp = getservbyname ((doencrypt ? "eklogin" : "klogin"), "tcp");
@@ -387,11 +386,11 @@ main (int argc, char *argv[])
   if (sp == NULL)
     sp = getservbyname ("login", "tcp");
   if (sp == NULL)
-    error (1, 0, "login/tcp: unknown service.");
+    error (EXIT_FAILURE, 0, "login/tcp: unknown service.");
 
   /* Get the name of the terminal from the environment.  Also get the
-     terminal's spee.  Both the name and the spee are passed to the server
-     as the "cmd" argument of the rcmd() function.  This is something like
+     terminal's speed.  The name and the speed are passed to the server
+     as the argument "cmd" of the rcmd() function.  This is something like
      "vt100/9600".  */
   term_speed = speed (0);
   if (term_speed == SPEED_NOTATTY)
@@ -410,13 +409,14 @@ main (int argc, char *argv[])
 
   setsig (SIGPIPE, lostpeer);
 
-  /* Block SIGURG and SIGUSR1 signals.  This will be handled by the
+  /* Block SIGURG and SIGUSR1 signals.  These will be handled by the
      parent and the child after the fork.  */
-  /* Will use SIGUSR1 for window size hack, so hold it off.  */
+  /* Will be using SIGUSR1 for window size hack, so hold it off.  */
   sigemptyset (&smask);
+  sigemptyset (&osmask);
   sigaddset (&smask, SIGURG);
   sigaddset (&smask, SIGUSR1);
-  sigprocmask (SIG_SETMASK, &smask, &smask);
+  sigprocmask (SIG_SETMASK, &smask, &osmask);
 
   /*
    * We set SIGURG and SIGUSR1 below so that an
@@ -427,18 +427,18 @@ main (int argc, char *argv[])
   setsig (SIGURG, copytochild);
   setsig (SIGUSR1, writeroob);
 
-#if defined (KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
 try_connect:
   if (use_kerberos)
     {
       struct hostent *hp;
 
-      /* Fully qualify hostname (needed for krb_realmofhost).  */
+      /* Fully qualified hostname (needed for krb_realmofhost).  */
       hp = gethostbyname (host);
       if (hp != NULL && !(host = strdup (hp->h_name)))
-	error (1, errno, "strdup");
+	error (EXIT_FAILURE, errno, "strdup");
 
-# if defined (KERBEROS)
+# if defined KERBEROS
       rem = KSUCCESS;
       errno = 0;
       if (dest_realm == NULL)
@@ -450,7 +450,7 @@ try_connect:
 
 # ifdef ENCRYPTION
       if (doencrypt)
-#  if defined(SHISHI)
+#  if defined SHISHI
 	{
 	  int i;
 
@@ -514,7 +514,7 @@ try_connect:
 # endif	/* CRYPT */
 
 	rem = krcmd (
-# if defined (SHISHI)
+# if defined SHISHI
 		      &handle, &host, sp->s_port, &user, term, 0, dest_realm);
 # else
 		      &host, sp->s_port, user, term, 0, dest_realm);
@@ -524,7 +524,7 @@ try_connect:
 	  use_kerberos = 0;
 	  sp = getservbyname ("login", "tcp");
 	  if (sp == NULL)
-	    error (1, 0, "unknown service login/tcp.");
+	    error (EXIT_FAILURE, 0, "unknown service login/tcp.");
 	  if (errno == ECONNREFUSED)
 	    warning ("remote host doesn't support Kerberos");
 	  if (errno == ENOENT)
@@ -537,7 +537,7 @@ try_connect:
     {
 # ifdef ENCRYPTION
       if (doencrypt)
-	error (1, 0, "the -x flag requires Kerberos authentication.");
+	error (EXIT_FAILURE, 0, "the -x flag requires Kerberos authentication.");
 # endif	/* CRYPT */
       if (!user)
 	user = pw->pw_name;
@@ -553,7 +553,7 @@ try_connect:
 #endif /* KERBEROS */
 
   if (rem < 0)
-    exit (1);
+    exit (EXIT_FAILURE);
 
   {
     int one = 1;
@@ -562,7 +562,7 @@ try_connect:
       error (0, errno, "setsockopt DEBUG (ignored)");
   }
 
-#if defined (IP_TOS) && defined (IPPROTO_IP) && defined (IPTOS_LOWDELAY)
+#if defined IP_TOS && defined IPPROTO_IP && defined IPTOS_LOWDELAY
   {
     int one = IPTOS_LOWDELAY;
     if (setsockopt (rem, IPPROTO_IP, IP_TOS, (char *) &one, sizeof (int)) < 0)
@@ -571,22 +571,23 @@ try_connect:
 #endif
 
   /* Now change to the real user ID.  We have to be set-user-ID root
-     to get the privileged port that rcmd () uses,  however we now want to
-     run as the real user who invoked us.  */
+     to get the privileged port that rcmd () uses.  We now want, however,
+     to run as the real user who invoked us.  */
   seteuid (uid);
   setuid (uid);
 
-  doit (&smask);
+  doit (&osmask);
 
   return 0;
 }
 
-/* Some systems, like QNX/Neutrino , The constant B0, B50,.. maps straigth to
-   the actual speed, 0, 50, ..., where on other system like GNU/Linux
-   it maps to a const 0, 1, ... i.e the value are encoded.
-   cfgetispeed(), according to posix should return a constant value reprensenting the Baud.
-   So to be portable we have to the conversion ourselves.  */
-/* Some values are not not define by POSIX.  */
+/* On some systems, like QNX/Neutrino, the constants B0, B50, etcetera,
+ * map straight to the actual speed (0, 50, etcetera), whereas on other
+ * systems like GNU/Linux, they map to ordered constants (0, 1, etcetera),
+ * i.e, the values are only order encoded.  cfgetispeed() should according
+ * to POSIX return a constant value representing the numerical speed.
+ * To be portable we need to do the conversion ourselves.  */
+/* Some values are not defined by POSIX.  */
 #ifndef B7200
 # define B7200   B4800
 #endif
@@ -665,8 +666,8 @@ speed_translate (unsigned int sym)
   return 0;
 }
 
-/* Returns the terminal speed for the file descriptor FD, or
-   SPEED_NOTATTY if FD is not associated with a terminal.  */
+/* Returns the terminal speed for the file descriptor FD,
+   or SPEED_NOTATTY if FD is not associated with a terminal.  */
 int
 speed (int fd)
 {
@@ -712,17 +713,17 @@ doit (sigset_t * smask)
       mode (1);
       if (reader (smask) == 0)
 	{
-	  /* If the reader () return 0, the socket to the server returned an
-	     EOF, meaning the client logged out of the remote system.
+	  /* If the reader returns zero, the socket to the server returned
+	     an EOF, meaning the client logged out of the remote system.
 	     This is the normal termination.  */
-          error (0, 0, "connection closed");
+          error (0, 0, "Connection to %s closed normally.\r", host);
           /* EXIT_SUCCESS is usually zero. So error might not exit.  */
           exit (EXIT_SUCCESS);
 	}
-      /* If the reader () returns nonzero, the socket to the server
-         returned an error.  Somethingg went wrong.  */
+      /* If the reader returns non-zero, the socket to the server
+         returned an error.  Something went wrong.  */
       sleep (1);
-      error (EXIT_FAILURE, 0, "\007connection closed");
+      error (EXIT_FAILURE, 0, "\007Connection to %s closed with error.\r", host);
     }
 
   /*
@@ -734,18 +735,18 @@ doit (sigset_t * smask)
    * signals to the child. We can now unblock SIGURG and SIGUSR1
    * that were set above.
    */
-  /* Reenables SIGURG and SIUSR1.  */
+  /* Reenable SIGURG and SIGUSR1.  */
   sigprocmask (SIG_SETMASK, smask, (sigset_t *) 0);
 
   setsig (SIGCHLD, catch_child);
 
   writer ();
 
-  /* If the write returns, it means the user entered "~." on the terminal.
+  /* If the writer returns, it means the user entered "~." on the terminal.
      In this case we terminate and the server will eventually get an EOF
      on its end of the network connection.  This should cause the server to
      log you out on the remote system.  */
-  error (0, 0, "closed connection");
+  error (0, 0, "Connection to %s aborted.\r", host);
 
 #ifdef SHISHI
   if (use_kerberos)
@@ -769,28 +770,28 @@ doit (sigset_t * smask)
 
 /* Enable a signal handler, unless the signal is already being ignored.
    This function is called before the fork (), for SIGHUP and SIGQUIT.  */
-/* Trap a signal, unless it is being ignored. */
 void
 setsignal (int sig)
 {
-  sig_t handler;
-  sigset_t sigs;
+  sighandler_t handler;
+  sigset_t sigs, osigs;
 
   sigemptyset (&sigs);
+  sigemptyset (&osigs);
   sigaddset (&sigs, sig);
-  sigprocmask (SIG_BLOCK, &sigs, &sigs);
+  sigprocmask (SIG_BLOCK, &sigs, &osigs);
 
   handler = setsig (sig, exit);
   if (handler == SIG_IGN)
     setsig (sig, handler);
 
-  sigprocmask (SIG_SETMASK, &sigs, (sigset_t *) 0);
+  sigprocmask (SIG_SETMASK, &osigs, (sigset_t *) 0);
 }
 
 /* This function is called by the parent:
    (1) at the end (user terminates the client end);
-   (2) SIGCLD signal - the sigcld_parent () function.
-   (3) SIGPPE signal - the connection has dropped.
+   (2) SIGCHLD signal - the catch_child () function.
+   (3) SIGPIPE signal - the connection was lost.
 
    We send the child a SIGKILL signal, which it can't ignore, then
    wait for it to terminate.  */
@@ -800,13 +801,13 @@ done (int status)
   pid_t w;
   int wstatus;
 
-  mode (0);
+  mode (0);	/* FIXME: Calls tcgetattr/tcsetattr in signal handler.  */
   if (child > 0)
     {
       /* make sure catch_child does not snap it up */
       setsig (SIGCHLD, SIG_DFL);
       if (kill (child, SIGKILL) >= 0)
-	while ((w = wait (&wstatus)) > 0 && w != child)
+	while ((w = waitpid (-1, &wstatus, WNOHANG)) > 0 && w != child)
 	  continue;
     }
   exit (status);
@@ -817,9 +818,11 @@ int dosigwinch;
 /*
  * This is called when the reader process gets the out-of-band (urgent)
  * request to turn on the window-changing protocol.
+ *
+ * FIXME: Race condition due to sendwindow() in signal handler?
  */
-RETSIGTYPE
-writeroob (int signo ARG_UNUSED)
+void
+writeroob (int signo _GL_UNUSED_PARAMETER)
 {
   if (dosigwinch == 0)
     {
@@ -829,8 +832,10 @@ writeroob (int signo ARG_UNUSED)
   dosigwinch = 1;
 }
 
-RETSIGTYPE
-catch_child (int signo ARG_UNUSED)
+/* FIXME: System dependent race condition due to done().
+ */
+void
+catch_child (int signo _GL_UNUSED_PARAMETER)
 {
   int status;
   pid_t pid;
@@ -841,6 +846,8 @@ catch_child (int signo ARG_UNUSED)
       if (pid == 0)
 	return;
       /* if the child (reader) dies, just quit */
+      if (pid < 0 && errno == EINTR)
+	continue;
       if (pid < 0 || (pid == child && !WIFSTOPPED (status)))
 	done (WEXITSTATUS (status) | WTERMSIG (status));
     }
@@ -853,7 +860,7 @@ catch_child (int signo ARG_UNUSED)
  * ~<delayed-suspend char>	suspend rlogin process, but leave reader alone.
  */
 void
-writer ()
+writer (void)
 {
   register int bol, local, n;
   char c;
@@ -992,8 +999,8 @@ stop (char cmdc)
   sigwinch (0);			/* check for size changes */
 }
 
-RETSIGTYPE
-sigwinch (int signo ARG_UNUSED)
+void
+sigwinch (int signo _GL_UNUSED_PARAMETER)
 {
   struct winsize ws;
 
@@ -1007,9 +1014,11 @@ sigwinch (int signo ARG_UNUSED)
 
 /*
  * Send the window size to the server via the magic escape
+ *
+ * FIXME: Used in signal handler. Race condition?
  */
 void
-sendwindow ()
+sendwindow (void)
 {
   struct winsize *wp;
   char obuf[4 + sizeof (struct winsize)];
@@ -1049,8 +1058,8 @@ pid_t ppid;
 int rcvcnt, rcvstate;
 char rcvbuf[8 * 1024];
 
-RETSIGTYPE
-oob (int signo ARG_UNUSED)
+void
+oob (int signo _GL_UNUSED_PARAMETER)
 {
   char mark;
   struct termios tt;
@@ -1158,7 +1167,7 @@ reader (sigset_t * smask)
   int n, remaining;
   char *bufp;
 
-#if BSD >= 43 || defined(SUNOS4)
+#if BSD >= 43 || defined SUNOS4
   pid = getpid ();		/* modern systems use positives for pid */
 #else
   pid = -getpid ();		/* old broken systems use negatives */
@@ -1259,22 +1268,24 @@ mode (int f)
     }
 }
 
-RETSIGTYPE
-lostpeer (int signo ARG_UNUSED)
+/* FIXME: Race condition due to done() in signal handler?
+ */
+void
+lostpeer (int signo _GL_UNUSED_PARAMETER)
 {
   setsig (SIGPIPE, SIG_IGN);
-  error (0, 0, "\007connection closed.");
+  error (0, 0, "\007Connection to %s lost.\r", host);
   done (1);
 }
 
 /* copy SIGURGs to the child process. */
-RETSIGTYPE
-copytochild (int signo ARG_UNUSED)
+void
+copytochild (int signo _GL_UNUSED_PARAMETER)
 {
   kill (child, SIGURG);
 }
 
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
 void
 warning (const char *fmt, ...)
 {
