@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-  2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
+  2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
   Foundation, Inc.
 
   This file is part of GNU Inetutils.
@@ -48,7 +48,7 @@
  */
 
 /*
- * remote shell server:
+ * remote shell server exchange protocol:
  *	[port]\0
  *	remuser\0
  *	locuser\0
@@ -56,9 +56,7 @@
  *	data
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <alloca.h>
 
@@ -71,6 +69,12 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#ifdef HAVE_NETINET_IN_SYSTM_H
+# include <netinet/in_systm.h>	/* n_time */
+#endif
+#ifdef HAVE_NETINET_IP_H
+# include <netinet/ip.h>	/* IPOPT_* */
+#endif
 
 #include <unistd.h>
 #include <errno.h>
@@ -104,7 +108,7 @@ char *getstr (const char *);
 int local_domain (const char *);
 const char *topdomain (const char *);
 
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
 # ifdef KERBEROS
 #  include <kerberosIV/des.h>
 #  include <kerberosIV/krb.h>
@@ -130,8 +134,10 @@ int doencrypt, use_kerberos, vacuous;
 static struct argp_option options[] = {
   { "verify-hostname", 'a', NULL, 0,
     "ask hostname for verification" },
+#ifdef HAVE___CHECK_RHOSTS_FILE
   { "no-rhosts", 'l', NULL, 0,
     "ignore .rhosts file" },
+#endif
   { "no-keepalive", 'n', NULL, 0,
     "do not set SO_KEEPALIVE" },
   { "log-sessions", 'L', NULL, 0,
@@ -147,9 +153,11 @@ static struct argp_option options[] = {
   { NULL }
 };
 
+#ifdef HAVE___CHECK_RHOSTS_FILE
 extern int __check_rhosts_file;	/* hook in rcmd(3) */
+#endif
 
-#ifdef __GLIBC__
+#if defined __GLIBC__ && defined WITH_IRUSEROK
 extern int iruserok (uint32_t raddr, int superuser,
                      const char *ruser, const char *luser);
 #endif
@@ -163,15 +171,17 @@ parse_opt (int key, char *arg, struct argp_state *state)
       check_all = 1;
       break;
 
+#ifdef HAVE___CHECK_RHOSTS_FILE
     case 'l':
       __check_rhosts_file = 0;	/* don't check .rhosts file */
       break;
+#endif
 
     case 'n':
       keepalive = 0;	/* don't enable SO_KEEPALIVE */
       break;
 
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
     case 'k':
       use_kerberos = 1;
       break;
@@ -224,20 +234,20 @@ main (int argc, char *argv[])
   if (argc > 0)
     {
       syslog (LOG_ERR, "%d extra arguments", argc);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
   if (use_kerberos && vacuous)
     {
       syslog (LOG_ERR, "only one of -k and -v allowed");
-      exit (2);
+      exit (EXIT_FAILURE);
     }
 # ifdef ENCRYPTION
   if (doencrypt && !use_kerberos)
     {
       syslog (LOG_ERR, "-k is required for -x");
-      exit (2);
+      exit (EXIT_FAILURE);
     }
 # endif
 #endif
@@ -286,10 +296,12 @@ extern char **environ;
 void
 doit (int sockfd, struct sockaddr_in *fromp)
 {
+#ifdef HAVE___RCMD_ERRSTR
   extern char *__rcmd_errstr;	/* syslog hook from libc/net/rcmd.c. */
+#endif
   struct hostent *hp;
   struct passwd *pwd;
-  u_short port;
+  unsigned short port;
   fd_set ready, readfrom;
   int cc, nfd, pv[2], pid, s = sockfd;
   int one = 1;
@@ -335,11 +347,11 @@ doit (int sockfd, struct sockaddr_in *fromp)
     {
       syslog (LOG_ERR, "malformed \"from\" address (af %d)\n",
 	      fromp->sin_family);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 #ifdef IP_OPTIONS
   {
-    u_char optbuf[BUFSIZ / 3], *cp;
+    unsigned char optbuf[BUFSIZ / 3], *cp;
     char lbuf[BUFSIZ], *lp;
     socklen_t optsize = sizeof (optbuf);
     int ipproto;
@@ -353,30 +365,60 @@ doit (int sockfd, struct sockaddr_in *fromp)
 		     &optsize) && optsize != 0)
       {
 	lp = lbuf;
-	/* The clent has set IP options.  This isn't allowd.
-	 * Use syslog() to record the fact.
+	/* The client has set IP options.  This isn't allowed.
+	 * Use syslog() to record the fact.  Only the option
+	 * types are printed, not their contents.
 	 */
-	for (cp = optbuf; optsize > 0; cp++, optsize--, lp += 3)
-	  sprintf (lp, " %2.2x", *cp);
+	for (cp = optbuf; optsize > 0; )
+	  {
+	    sprintf (lp, " %2.2x", *cp);
+	    lp += 3;
+
+	    if (*cp == IPOPT_SSRR || *cp == IPOPT_LSRR)
+	      {
+		/* Already the TCP handshake suffices for
+		 * a man-in-the-middle attack vector.
+		 */
+		syslog (LOG_NOTICE,
+			"Discarding connection from %s with set source routing",
+			inet_ntoa (fromp->sin_addr));
+		exit (EXIT_FAILURE);
+	      }
+	    if (*cp == IPOPT_EOL)
+	      break;
+	    if (*cp == IPOPT_NOP)
+	      cp++, optsize--;
+	    else
+	      {
+		/* Options using a length octet, see RFC 791.  */
+		int inc = cp[1];
+
+		optsize -= inc;
+		cp += inc;
+	      }
+	  }
+
+	/* At this point presumably harmless options are present.
+	 * Make a report about them, erase them, and continue.  */
 	syslog (LOG_NOTICE,
-		"Connection received from %s using IP options (ignored):%s",
+		"Connection received from %s using IP options (erased):%s",
 		inet_ntoa (fromp->sin_addr), lbuf);
 
-	/* Turn off the options.  If this doesn't work, we quit */
+	/* Turn off the options.  If this doesn't work, we quit.  */
 	if (setsockopt (sockfd, ipproto, IP_OPTIONS,
 			(char *) NULL, optsize) != 0)
 	  {
 	    syslog (LOG_ERR, "setsockopt IP_OPTIONS NULL: %m");
-	    exit (1);
+	    exit (EXIT_FAILURE);
 	  }
       }
   }
 #endif
 
   /* Need host byte ordered port# to compare */
-  fromp->sin_port = ntohs ((u_short) fromp->sin_port);
+  fromp->sin_port = ntohs ((unsigned short) fromp->sin_port);
   /* Verify that the client's address was bound to a reserved port */
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
   if (!use_kerberos)
 #endif
     if (fromp->sin_port >= IPPORT_RESERVED
@@ -385,7 +427,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	syslog (LOG_NOTICE | LOG_AUTH,
 		"Connection from %s on illegal port %u",
 		inet_ntoa (fromp->sin_addr), fromp->sin_port);
-	exit (1);
+	exit (EXIT_FAILURE);
       }
 
   /* Read the ASCII string specifying the secondary port# from
@@ -403,7 +445,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	  if (cc < 0)
 	    syslog (LOG_NOTICE, "read: %m");
 	  shutdown (sockfd, 2);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
       /* null byte terminates the string */
       if (c == 0)
@@ -414,12 +456,12 @@ doit (int sockfd, struct sockaddr_in *fromp)
   alarm (0);
   if (port != 0)
     {
-      /* If the secondary port# is nonzero, the we have to
+      /* If the secondary port# is non-zero, then we have to
        * connect to that port (which the client has already
        * created and is listening on).  The secondary port#
-       * that the client tells us to connect to has to also be
+       * that the client tells us to connect to has also to be
        * a reserved port#.  Also, our end of this secondary
-       * connection has to also have a reserved TCP port bond
+       * connection has also to have a reserved TCP port bound
        * to it, plus.
        */
       int lport = IPPORT_RESERVED - 1;
@@ -427,36 +469,34 @@ doit (int sockfd, struct sockaddr_in *fromp)
       if (s < 0)
 	{
 	  syslog (LOG_ERR, "can't get stderr port: %m");
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
       if (!use_kerberos)
 #endif
 	if (port >= IPPORT_RESERVED || port < IPPORT_RESERVED / 2)
 	  {
 	    syslog (LOG_ERR, "2nd port not reserved\n");
-	    exit (1);
+	    exit (EXIT_FAILURE);
 	  }
-      /* Use the fromp structure taht we already have.
+      /* Use the fromp structure that we already have available.
        * The 32-bit Internet address is obviously that of the
-       * client's, just change the port# to the one specified
-       * by the clent as the secondary port.
+       * client; just change the port# to the one specified
+       * as secondary port by the client.
        */
-      syslog (LOG_ERR, "2nd port not reserved %d\n", port);
-
       fromp->sin_port = htons (port);
       if (connect (s, (struct sockaddr *) fromp, sizeof (*fromp)) < 0)
 	{
 	  syslog (LOG_INFO, "connect second port %d: %m", port);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
 
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
   if (vacuous)
     {
       rshd_error ("rshd: remote host requires Kerberos authentication\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 #endif
 
@@ -483,7 +523,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
        * address corresponds to the name.
        */
       hostname = strdup (hp->h_name);
-#if defined(KERBEROS) || defined(SHISHI)
+#if defined KERBEROS || defined SHISHI
       if (!use_kerberos)
 #endif
 	if (check_all || local_domain (hp->h_name))
@@ -548,7 +588,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	    {
 	      syslog (LOG_ERR, "getsockname: %m");
 	      rshd_error ("rlogind: getsockname: %s", strerror (errno));
-	      exit (1);
+	      exit (EXIT_FAILURE);
 	    }
 	  authopts = KOPT_DO_MUTUAL;
 	  rc = krb_recvauth (authopts, 0, ticket,
@@ -566,7 +606,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	{
 	  rshd_error ("Kerberos authentication failure: %s\n",
 		      krb_err_txt[rc]);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else
@@ -582,7 +622,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	{
 	  rshd_error ("Kerberos authentication failure: %s\n",
 		      err_msg ? err_msg : shishi_strerror (rc));
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
   else
@@ -667,7 +707,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
     remuser = getstr ("remuser");
     rc = read (STDIN_FILENO, &error, sizeof (int));
     if ((rc != sizeof (int)) && rc)
-      exit (1);
+      exit (EXIT_FAILURE);
 
     /* verify checksum */
 
@@ -675,7 +715,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
        if (getsockname (STDIN_FILENO, (struct sockaddr *)&sock, &socklen) < 0)
        {
        syslog (LOG_ERR, "Can't get sock name");
-       exit (1);
+       exit (EXIT_FAILURE);
        }
      */
     snprintf (cksumdata, 100, "544:%s%s", /*ntohs(sock.sin_port), */ cmdbuf,
@@ -692,7 +732,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	/* *err_msg = "checksum verify failed"; */
 	syslog (LOG_ERR, "checksum verify failed: %s", shishi_error (h));
 	free (compcksum);
-	exit (1);
+	exit (EXIT_FAILURE);
       }
 
     rc = shishi_authorized_p (h, shishi_ap_tkt (ap), locuser);
@@ -700,7 +740,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
       {
 	syslog (LOG_ERR, "User is not authorized to log in as: %s", locuser);
 	shishi_ap_done (ap);
-	exit (1);
+	exit (EXIT_FAILURE);
       }
 
     free (compcksum);
@@ -734,7 +774,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	      syslog (LOG_INFO | LOG_AUTH, "Kerberos rsh denied to %s.%s@%s",
 		      kdata->pname, kdata->pinst, kdata->prealm);
 	      rshd_error ("Permission denied.\n");
-	      exit (1);
+	      exit (EXIT_FAILURE);
 	    }
 	}
     }
@@ -749,21 +789,31 @@ doit (int sockfd, struct sockaddr_in *fromp)
 				   syslog (LOG_INFO|LOG_AUTH, "Kerberos rsh denied to %s.%s@%s",
 				   kdata->pname, kdata->pinst, kdata->prealm);
 				   rshd_error ("Permission denied.\n");
-				   exit (1);
+				   exit (EXIT_FAILURE);
 				   }
 				   } */
     }
   else
 #endif
+#ifdef WITH_IRUSEROK
     if (errorstr || (pwd->pw_passwd != 0 && *pwd->pw_passwd != '\0'
                      && (iruserok (fromp->sin_addr.s_addr, pwd->pw_uid == 0,
                                    remuser, locuser)) < 0))
+#elif defined WITH_RUSEROK
+    if (errorstr || (pwd->pw_passwd != 0 && *pwd->pw_passwd != '\0'
+                     && (ruserok (inet_ntoa (fromp->sin_addr),
+				  pwd->pw_uid == 0, remuser, locuser)) < 0))
+#else /* !WITH_IRUSEROK && !WITH_RUSEROK */
+#error Unable to use mandatory iruserok/ruserok.  This should not happen.
+#endif /* !WITH_IRUSEROK && !WITH_RUSEROK */
     {
+#ifdef HAVE___RCMD_ERRSTR
       if (__rcmd_errstr)
 	syslog (LOG_INFO | LOG_AUTH,
 		"%s@%s as %s: permission denied (%s). cmd='%.80s'",
 		remuser, hostname, locuser, __rcmd_errstr, cmdbuf);
       else
+#endif
 	syslog (LOG_INFO | LOG_AUTH,
 		"%s@%s as %s: permission denied. cmd='%.80s'",
 		remuser, hostname, locuser, cmdbuf);
@@ -771,14 +821,14 @@ doit (int sockfd, struct sockaddr_in *fromp)
       if (errorstr == NULL)
 	errorstr = "Permission denied.\n";
       rshd_error (errorstr, errorhost);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   /* If the locuser isn't root, then check if logins are disabled. */
   if (pwd->pw_uid && !access (PATH_NOLOGIN, F_OK))
     {
       rshd_error ("Logins currently disabled.\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   /* Now write the null byte back to the client telling it
@@ -791,7 +841,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
   if (write (STDERR_FILENO, "\0", 1) < 0)
     {
       rshd_error ("Lost connection.\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   sent_null = 1;
 
@@ -806,21 +856,21 @@ doit (int sockfd, struct sockaddr_in *fromp)
       if (pipe (pv) < 0)
 	{
 	  rshd_error ("Can't make pipe.\n");
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
 #ifdef ENCRYPTION
-# if defined(KERBEROS) || defined(SHISHI)
+# if defined KERBEROS || defined SHISHI
       if (doencrypt)
 	{
 	  if (pipe (pv1) < 0)
 	    {
 	      rshd_error ("Can't make 2nd pipe.\n");
-	      exit (1);
+	      exit (EXIT_FAILURE);
 	    }
 	  if (pipe (pv2) < 0)
 	    {
 	      rshd_error ("Can't make 3rd pipe.\n");
-	      exit (1);
+	      exit (EXIT_FAILURE);
 	    }
 	}
 # endif
@@ -829,7 +879,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
       if (pid == -1)
 	{
 	  rshd_error ("Can't fork; try again.\n");
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
       if (pid)
 	{
@@ -839,7 +889,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	   *         signal.
 	   */
 #ifdef ENCRYPTION
-# if defined(KERBEROS)
+# if defined KERBEROS
 	  if (doencrypt)
 	    {
 	      static char msg[] = SECURE_MESSAGE;
@@ -875,7 +925,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	  else
 	    nfd = s;
 #ifdef ENCRYPTION
-# if defined(KERBEROS) || defined(SHISHI)
+# if defined KERBEROS || defined SHISHI
 	  if (doencrypt)
 	    {
 	      FD_ZERO (&writeto);
@@ -895,7 +945,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	    {
 	      ready = readfrom;
 #ifdef ENCRYPTION
-# if defined(KERBEROS) || defined(SHISHI)
+# if defined KERBEROS || defined SHISHI
 	      if (doencrypt)
 		{
 #  ifdef SHISHI
@@ -960,7 +1010,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 		    }
 		}
 #ifdef ENCRYPTION
-# if defined(KERBEROS) || defined(SHISHI)
+# if defined KERBEROS || defined SHISHI
 	      if (doencrypt && FD_ISSET (pv1[0], &ready))
 		{
 		  errno = 0;
@@ -1000,7 +1050,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	    }
 	  while (FD_ISSET (s, &readfrom) ||
 #ifdef ENCRYPTION
-# if defined(KERBEROS) || defined(SHISHI)
+# if defined KERBEROS || defined SHISHI
 		 (doencrypt && FD_ISSET (pv1[0], &readfrom)) ||
 # endif
 #endif
@@ -1009,7 +1059,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 	   * terminates.  The socket will terninate whe the
 	   * client process terminates.
 	   */
-	  exit (0);
+	  exit (EXIT_SUCCESS);
 	}
 
       /* Child process. Become a process group leader, so that
@@ -1022,7 +1072,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
       close (s);		/* control process handles this fd */
       close (pv[0]);		/* close read end of pipe */
 #ifdef ENCRYPTION
-# if defined(KERBEROS) || defined(SHISHI)
+# if defined KERBEROS || defined SHISHI
       if (doencrypt)
 	{
 	  close (pv1[0]);
@@ -1035,7 +1085,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
 # endif
 #endif
 
-#if defined(SHISHI)
+#if defined SHISHI
       if (use_kerberos)
 	{
 	  int i;
@@ -1121,7 +1171,7 @@ doit (int sockfd, struct sockaddr_in *fromp)
   else
 #endif
     execl (pwd->pw_shell, cp, "-c", cmdbuf, NULL);
-  error (1, errno, "cannot execute %s", pwd->pw_shell);
+  error (EXIT_FAILURE, errno, "cannot execute %s", pwd->pw_shell);
 }
 
 /*
@@ -1159,7 +1209,7 @@ getstr (const char *err)
   if (!buf)
     {
       rshd_error ("Out of space reading %s\n", err);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   do
@@ -1172,7 +1222,7 @@ getstr (const char *err)
 	    rshd_error ("EOF reading %s\n", err);
 	  else
 	    perror (err);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
 
       end += rd;
@@ -1185,7 +1235,7 @@ getstr (const char *err)
 	  if (!buf)
 	    {
 	      rshd_error ("Out of space reading %s\n", err);
-	      exit (1);
+	      exit (EXIT_FAILURE);
 	    }
 	  end = buf + end_offs;
 	}
