@@ -1,6 +1,6 @@
 /* syslogd - log system messages
   Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-  2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -68,9 +68,7 @@
  * more extensive changes by Eric Allman (again)
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #define IOVCNT          6	/* size of the iovec array */
 #define MAXLINE		1024	/* Maximum line length.  */
@@ -91,9 +89,7 @@
 #include <time.h>
 #include <sys/resource.h>
 #include <poll.h>
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
+#include <sys/types.h>
 
 #include <netinet/in.h>
 #include <netdb.h>
@@ -115,13 +111,18 @@
 #define SYSLOG_NAMES
 #include <syslog.h>
 #ifndef HAVE_SYSLOG_INTERNAL
-# include <syslog-int.h>
+# include "logprio.h"
+#endif
+
+#ifndef LOG_MAKEPRI
+#  define LOG_MAKEPRI(fac, p)	(((fac) << 3) | (p))
 #endif
 
 #include <error.h>
 #include <progname.h>
 #include <libinetutils.h>
 #include <readutmp.h>
+#include "unused-parameter.h"
 
 /* A mask of all facilities mentioned explicitly in the configuration file
  *
@@ -165,7 +166,7 @@ struct filed
   short f_type;			/* Entry type, see below.  */
   short f_file;			/* File descriptor.  */
   time_t f_time;		/* Time this was last written.  */
-  u_char f_pmask[LOG_NFACILITIES + 1];	/* Priority mask.  */
+  unsigned char f_pmask[LOG_NFACILITIES + 1];	/* Priority mask.  */
   union
   {
     struct
@@ -176,7 +177,8 @@ struct filed
     struct
     {
       char *f_hname;
-      struct sockaddr_in f_addr;
+      struct sockaddr_storage f_addr;
+      socklen_t f_addrlen;
     } f_forw;			/* Forwarding address.  */
     char *f_fname;		/* Name use for Files|Pipes|TTYs.  */
   } f_un;
@@ -243,7 +245,7 @@ int repeatinterval[] = { 30, 60 };	/* Number of seconds before flush.  */
 extern int waitdaemon (int nochdir, int noclose, int maxwait);
 
 void cfline (const char *, struct filed *);
-const char *cvthname (struct sockaddr_in *);
+const char *cvthname (struct sockaddr *, socklen_t);
 int decode (const char *, CODE *);
 void die (int);
 void doexit (int);
@@ -263,13 +265,22 @@ static void dbg_printf (const char *, ...);
 void trigger_restart (int);
 static void add_funix (const char *path);
 static int create_unix_socket (const char *path);
-static int create_inet_socket (void);
+static void create_inet_socket (int af, int fd46[2]);
 
 char *LocalHostName;		/* Our hostname.  */
 char *LocalDomain;		/* Our local domain name.  */
-int finet = -1;			/* Internet datagram socket fd.  */
+char *BindAddress = NULL;	/* Binding address for INET listeners.
+				 * The default is a wildcard address.  */
+char addrstr[INET6_ADDRSTRLEN];	/* Common address presentation.  */
+char addrname[NI_MAXHOST];	/* Common name lookup.  */
+int usefamily = AF_INET;	/* Address family for INET services.
+				 * Each of the values `AF_INET' and `AF_INET6'
+				 * produces a single-stacked server.  */
+int finet[2] = {-1, -1};	/* Internet datagram socket fd.  */
+#define IU_FD_IP4	0	/* Indices for the address families.  */
+#define IU_FD_IP6	1
 int fklog = -1;			/* Kernel log device fd.  */
-int LogPort;			/* Port number for INET connections.  */
+char *LogPortText = "syslog";	/* Service/port for INET connections.  */
 int Initialized;		/* True when we are initialized. */
 int MarkInterval = 20 * 60;	/* Interval between marks in seconds.  */
 int MarkSeq;			/* Mark sequence number.  */
@@ -296,7 +307,8 @@ enum {
   OPT_NO_FORWARD = 256,
   OPT_NO_KLOG,
   OPT_NO_UNIXAF,
-  OPT_PIDFILE
+  OPT_PIDFILE,
+  OPT_IPANY
 };
 
 static struct argp_option argp_options[] = {
@@ -311,6 +323,10 @@ static struct argp_option argp_options[] = {
   {"hop", 'h', NULL, 0, "forward messages from remote hosts", GRP+1},
   {"inet", 'r', NULL, 0, "receive remote messages via internet domain socket",
    GRP+1},
+  {"ipv4", '4', NULL, 0, "restrict to IPv4 transport (default)", GRP+1},
+  {"ipv6", '6', NULL, 0, "restrict to IPv6 transport", GRP+1},
+  {"ipany", OPT_IPANY, NULL, 0, "allow transport with IPv4 and IPv6", GRP+1},
+  {"bind", 'b', "ADDR", 0, "bind listener to this address/name", GRP+1},
   {"mark", 'm', "INTVL", 0, "specify timestamp interval in logs (0 for no "
    "timestamps)", GRP+1},
   {"no-detach", 'n', NULL, 0, "do not enter daemon mode", GRP+1},
@@ -367,6 +383,22 @@ parse_opt (int key, char *arg, struct argp_state *state)
       AcceptRemote = 1;
       break;
 
+    case '4':
+      usefamily = AF_INET;
+      break;
+
+    case '6':
+      usefamily = AF_INET6;
+      break;
+
+    case OPT_IPANY:
+      usefamily = AF_UNSPEC;
+      break;
+
+    case 'b':
+      BindAddress = arg;
+      break;
+
     case 'm':
       v = strtol (arg, &endptr, 10);
       if (*endptr)
@@ -452,7 +484,7 @@ main (int argc, char *argv[])
       signal (SIGTERM, doexit);
       ppid = waitdaemon (0, 0, 30);
       if (ppid < 0)
-        error (1, errno, "could not become daemon");
+        error (EXIT_FAILURE, errno, "could not become daemon");
     }
   else
     {
@@ -463,7 +495,7 @@ main (int argc, char *argv[])
   /* Get our hostname.  */
   LocalHostName = localhost ();
   if (LocalHostName == NULL)
-    error (2, errno, "can't get local host name");
+    error (EXIT_FAILURE, errno, "can't get local host name");
 
   /* Get the domainname.  */
   p = strchr (LocalHostName, '.');
@@ -474,21 +506,27 @@ main (int argc, char *argv[])
     }
   else
     {
-      struct hostent *host_ent;
+      struct addrinfo hints, *rp;
+      int err;
 
-      /* Try to resolve the domainname by calling DNS. */
-      host_ent = gethostbyname (LocalHostName);
-      if (host_ent)
+      memset (&hints, 0, sizeof (hints));
+      hints.ai_family = AF_UNSPEC;	/* Family is irrelevant.  */
+      hints.ai_flags = AI_CANONNAME;
+
+      /* Try to resolve the domainname by calling DNS.  */
+      err = getaddrinfo (LocalHostName, NULL, &hints, &rp);
+      if (err == 0)
 	{
 	  /* Override what we had */
 	  free (LocalHostName);
-	  LocalHostName = strdup (host_ent->h_name);
+	  LocalHostName = strdup (rp->ai_canonname);
 	  p = strchr (LocalHostName, '.');
 	  if (p != NULL)
 	    {
 	      *p++ = '\0';
 	      LocalDomain = p;
 	    }
+	  freeaddrinfo (rp);
 	}
       if (LocalDomain == NULL)
 	LocalDomain = strdup ("");
@@ -504,10 +542,10 @@ main (int argc, char *argv[])
   signal (SIGUSR1, Debug ? dbg_toggle : SIG_IGN);
   alarm (TIMERINTVL);
 
-  /* We add  2 = 1(klog) + 1(inet), even if they may be not use.  */
-  fdarray = (struct pollfd *) malloc ((nfunix + 2) * sizeof (*fdarray));
+  /* We add  3 = 1(klog) + 2(inet,inet6), even if they may stay unused.  */
+  fdarray = (struct pollfd *) malloc ((nfunix + 3) * sizeof (*fdarray));
   if (fdarray == NULL)
-    error (2, errno, "can't allocate fd table");
+    error (EXIT_FAILURE, errno, "can't allocate fd table");
 
   /* read configuration file */
   init (0);
@@ -548,18 +586,27 @@ main (int argc, char *argv[])
 	}
     }
 
-  /* Initialize inet socket and add them to the list.  */
-  if (AcceptRemote || (!NoForward))
+  /* Initialize inet socket and add it to the list.  */
+  if (AcceptRemote)
     {
-      finet = create_inet_socket ();
-      if (finet >= 0)
+      create_inet_socket (usefamily, finet);
+      if (finet[IU_FD_IP4] >= 0)
 	{
-	  fdarray[nfds].fd = finet;
+	  /* IPv4 socket is present.  */
+	  fdarray[nfds].fd = finet[IU_FD_IP4];
 	  fdarray[nfds].events = POLLIN | POLLPRI;
 	  nfds++;
-	  dbg_printf ("Opened syslog UDP port.\n");
+	  dbg_printf ("Opened syslog UDP/IPv4 port.\n");
 	}
-      else
+      if (finet[IU_FD_IP6] >= 0)
+	{
+	  /* IPv6 socket is present.  */
+	  fdarray[nfds].fd = finet[IU_FD_IP6];
+	  fdarray[nfds].events = POLLIN | POLLPRI;
+	  nfds++;
+	  dbg_printf ("Opened syslog UDP/IPv6 port.\n");
+	}
+      if (finet[IU_FD_IP4] < 0 && finet[IU_FD_IP6] < 0)
 	dbg_printf ("Can't open UDP port: %s\n", strerror (errno));
     }
 
@@ -696,9 +743,10 @@ main (int argc, char *argv[])
 		      }
 		  }
 	      }
-	    else if (fdarray[i].fd == finet)
+	    else if (fdarray[i].fd == finet[IU_FD_IP4]
+		     || fdarray[i].fd == finet[IU_FD_IP6])
 	      {
-		struct sockaddr_in frominet;
+		struct sockaddr_storage frominet;
 		/*dbg_printf ("inet message\n"); */
 		len = sizeof (frominet);
 		memset (line, '\0', sizeof (line));
@@ -707,7 +755,7 @@ main (int argc, char *argv[])
 		if (result > 0)
 		  {
 		    line[result] = '\0';
-		    printline (cvthname (&frominet), line);
+		    printline (cvthname ((struct sockaddr *) &frominet, len), line);
 		  }
 		else if (result < 0 && errno != EINTR)
 		  logerror ("recvfrom inet");
@@ -749,7 +797,7 @@ add_funix (const char *name)
 {
   funix = realloc (funix, (nfunix + 1) * sizeof (*funix));
   if (funix == NULL)
-    error (1, errno, "cannot allocate space for unix sockets");
+    error (EXIT_FAILURE, errno, "cannot allocate space for unix sockets");
 
   funix[nfunix].name = name;
   funix[nfunix].fd = -1;
@@ -766,11 +814,18 @@ create_unix_socket (const char *path)
   if (path[0] == '\0')
     return -1;
 
+  if (strlen (path) >= sizeof (sunx.sun_path))
+    {
+      snprintf (line, sizeof (line), "UNIX socket name too long: %s", path);
+      logerror (line);
+      return -1;
+    }
+
   unlink (path);
 
   memset (&sunx, 0, sizeof (sunx));
   sunx.sun_family = AF_UNIX;
-  strncpy (sunx.sun_path, path, sizeof (sunx.sun_path));
+  strncpy (sunx.sun_path, path, sizeof (sunx.sun_path) - 1);
   fd = socket (AF_UNIX, SOCK_DGRAM, 0);
   if (fd < 0 || bind (fd, (struct sockaddr *) &sunx, SUN_LEN (&sunx)) < 0
       || chmod (path, 0666) < 0)
@@ -779,33 +834,63 @@ create_unix_socket (const char *path)
       logerror (line);
       dbg_printf ("cannot create %s: %s\n", path, strerror (errno));
       close (fd);
+      fd = -1;
     }
   return fd;
 }
 
-static int
-create_inet_socket ()
+static void
+create_inet_socket (int af, int fd46[2])
 {
-  int fd;
-  struct sockaddr_in sin;
+  int err, fd = -1;
+  struct addrinfo hints, *rp, *ai;
 
-  fd = socket (AF_INET, SOCK_DGRAM, 0);
-  if (fd < 0)
+  /* Invalidate old descriptors.  */
+  fd46[IU_FD_IP4] = fd46[IU_FD_IP6] = -1;
+
+  memset (&hints, 0, sizeof (hints));
+  hints.ai_family = af;
+  hints.ai_socktype = SOCK_DGRAM;
+  hints.ai_flags = AI_PASSIVE;
+
+  err = getaddrinfo (BindAddress, LogPortText, &hints, &rp);
+  if (err)
     {
-      logerror ("unknown protocol, suspending inet service");
-      return fd;
+      logerror ("lookup error, suspending inet service");
+      return;
     }
 
-  memset (&sin, 0, sizeof (sin));
-  sin.sin_family = AF_INET;
-  sin.sin_port = LogPort;
-  if (bind (fd, (struct sockaddr *) &sin, sizeof (sin)) < 0)
+  for (ai = rp; ai; ai = ai->ai_next)
     {
-      logerror ("bind, suspending inet");
-      close (fd);
-      return -1;
+      fd = socket (ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+      if (fd < 0)
+	continue;
+      if (ai->ai_family == AF_INET6)
+	{
+	  int yes = 1;
+	  /* Avoid dual stacked sockets.  Better to use distinct sockets.  */
+	  (void) setsockopt (fd, IPPROTO_IPV6, IPV6_V6ONLY, &yes, sizeof (yes));
+	}
+      if (bind (fd, ai->ai_addr, ai->ai_addrlen) < 0)
+	{
+	  close (fd);
+	  fd = -1;
+	  continue;
+	}
+      /* Register any success.  */
+      if (ai->ai_family == AF_INET && fd46[IU_FD_IP4] < 0)
+	fd46[IU_FD_IP4] = fd;
+      else if (ai->ai_family == AF_INET6 && fd46[IU_FD_IP6] < 0)
+	fd46[IU_FD_IP6] = fd;
     }
-  return fd;
+  freeaddrinfo (rp);
+
+  if (fd46[IU_FD_IP4] < 0 && fd46[IU_FD_IP6] < 0)
+    {
+      logerror ("inet service, failed lookup.");
+      return;
+    }
+  return;
 }
 
 char **
@@ -843,7 +928,7 @@ crunch_list (char **oldlist, char *list)
   /* allocate enough space */
   oldlist = (char **) realloc (oldlist, (i + count + 1) * sizeof (*oldlist));
   if (oldlist == NULL)
-    error (1, errno, "can't allocate memory");
+    error (EXIT_FAILURE, errno, "can't allocate memory");
 
   /*
      We now can assume that the first and last
@@ -856,7 +941,7 @@ crunch_list (char **oldlist, char *list)
     {
       oldlist[count] = (char *) malloc ((q - p + 1) * sizeof (char));
       if (oldlist[count] == NULL)
-        error (1, errno, "can't allocate memory");
+        error (EXIT_FAILURE, errno, "can't allocate memory");
 
       strncpy (oldlist[count], p, q - p);
       oldlist[count][q - p] = '\0';
@@ -865,7 +950,7 @@ crunch_list (char **oldlist, char *list)
   /* take the last one */
   oldlist[count] = (char *) malloc ((strlen (p) + 1) * sizeof (char));
   if (oldlist[count] == NULL)
-    error (1, errno, "can't allocate memory");
+    error (EXIT_FAILURE, errno, "can't allocate memory");
 
   strcpy (oldlist[count], p);
 
@@ -1124,7 +1209,6 @@ fprintlog (struct filed *f, const char *from, int flags, const char *msg)
   int l;
   char line[MAXLINE + 1], repbuf[80], greetings[200];
   time_t fwd_suspend;
-  struct hostent *hp;
 
   v = iov;
   /* Be paranoid.  */
@@ -1150,9 +1234,12 @@ fprintlog (struct filed *f, const char *from, int flags, const char *msg)
       v->iov_len = 1;
       v++;
     }
-  v->iov_base = f->f_prevhost;
-  v->iov_len = strlen (v->iov_base);
-  v++;
+  if (f->f_prevhost)
+    {
+      v->iov_base = f->f_prevhost;
+      v->iov_len = strlen (v->iov_base);
+      v++;
+    }
   v->iov_base = (char *) " ";
   v->iov_len = 1;
   v++;
@@ -1206,20 +1293,32 @@ fprintlog (struct filed *f, const char *from, int flags, const char *msg)
       fwd_suspend = time ((time_t *) 0) - f->f_time;
       if (fwd_suspend >= INET_SUSPEND_TIME)
 	{
-	  hp = gethostbyname (f->f_un.f_forw.f_hname);
-	  if (hp == NULL)
+	  struct addrinfo hints, *rp;
+	  int err;
+
+	  memset (&hints, 0, sizeof (hints));
+	  hints.ai_family = usefamily;
+#ifdef AI_ADDRCONFIG
+	  hints.ai_flags = AI_ADDRCONFIG;
+#endif
+	  err = getaddrinfo (f->f_un.f_forw.f_hname, LogPortText, &hints, &rp);
+	  if (err)
 	    {
-	      extern int h_errno;
-	      dbg_printf ("Failure: %s\n", hstrerror (h_errno));
+	      dbg_printf ("Failure: %s\n", gai_strerror (err));
 	      dbg_printf ("Retries: %d\n", f->f_prevcount);
 	      if (--f->f_prevcount < 0)
-		f->f_type = F_UNUSED;
+		{
+		  f->f_type = F_UNUSED;
+		  free (f->f_un.f_forw.f_hname);
+		  f->f_un.f_forw.f_hname = NULL;
+		}
 	    }
 	  else
 	    {
 	      dbg_printf ("%s found, resuming.\n", f->f_un.f_forw.f_hname);
-	      memcpy (&f->f_un.f_forw.f_addr.sin_addr,
-		      hp->h_addr, hp->h_length);
+	      f->f_un.f_forw.f_addrlen = rp->ai_addrlen;
+	      memcpy (&f->f_un.f_forw.f_addr, rp->ai_addr, rp->ai_addrlen);
+	      freeaddrinfo (rp);
 	      f->f_prevcount = 0;
 	      f->f_type = F_FORW;
 	      goto f_forw;
@@ -1237,10 +1336,55 @@ fprintlog (struct filed *f, const char *from, int flags, const char *msg)
 	dbg_printf ("Not forwarding remote message.\n");
       else if (NoForward)
 	dbg_printf ("Not forwarding because forwarding is disabled.\n");
-      else if (finet < 0)
-	dbg_printf ("Not forwarding because of invalid inet fd.\n");
       else
 	{
+	  int temp_finet, *pfinet;	/* PFINET points to active fd.  */
+
+	  if (f->f_un.f_forw.f_addr.ss_family == AF_INET)
+	    pfinet = &finet[IU_FD_IP4];
+	  else	/* AF_INET6 */
+	    pfinet = &finet[IU_FD_IP6];
+
+	  temp_finet = *pfinet;
+
+	  if (temp_finet < 0)
+	    {
+	      int err;
+	      struct addrinfo hints, *rp;
+
+	      /* Forwarding needs a temporary socket.
+	       * The source port is fixed!  */
+	      memset (&hints, 0, sizeof (hints));
+	      hints.ai_family = f->f_un.f_forw.f_addr.ss_family;
+	      hints.ai_socktype = SOCK_DGRAM;
+	      hints.ai_flags = AI_PASSIVE;
+
+	      err = getaddrinfo (NULL, LogPortText, &hints, &rp);
+	      if (err)
+		{
+		  dbg_printf ("Not forwarding due to lookup failure: %s.\n",
+			      gai_strerror(err));
+		  break;
+		}
+	      temp_finet = socket (rp->ai_family, rp->ai_socktype,
+				   rp->ai_protocol);
+	      if (temp_finet < 0)
+		{
+		  dbg_printf ("Not forwarding due to socket failure.\n");
+		  freeaddrinfo (rp);
+		  break;
+		}
+
+	      err = bind (temp_finet, rp->ai_addr, rp->ai_addrlen);
+	      freeaddrinfo (rp);
+	      if (err)
+		{
+		  dbg_printf ("Not forwarding due to bind error: %s.\n",
+			      strerror (errno));
+		  break;
+		}
+	    } /* Creation of temporary outgoing socket since "finet < 0" */
+
 	  f->f_time = now;
 	  snprintf (line, sizeof (line), "<%d>%.15s %s",
 		    f->f_prevpri, (char *) iov[0].iov_base,
@@ -1248,9 +1392,9 @@ fprintlog (struct filed *f, const char *from, int flags, const char *msg)
 	  l = strlen (line);
 	  if (l > MAXLINE)
 	    l = MAXLINE;
-	  if (sendto (finet, line, l, 0,
+	  if (sendto (temp_finet, line, l, 0,
 		      (struct sockaddr *) &f->f_un.f_forw.f_addr,
-		      sizeof (f->f_un.f_forw.f_addr)) != l)
+		      f->f_un.f_forw.f_addrlen) != l)
 	    {
 	      int e = errno;
 	      dbg_printf ("INET sendto error: %d = %s.\n", e, strerror (e));
@@ -1258,6 +1402,9 @@ fprintlog (struct filed *f, const char *from, int flags, const char *msg)
 	      errno = e;
 	      logerror ("sendto");
 	    }
+
+	  if (*pfinet < 0)
+	    close (temp_finet);	/* Only temporary socket may be closed.  */
 	}
       break;
 
@@ -1303,6 +1450,8 @@ fprintlog (struct filed *f, const char *from, int flags, const char *msg)
 		{
 		  f->f_type = F_UNUSED;
 		  logerror (f->f_un.f_fname);
+		  free (f->f_un.f_fname);
+		  f->f_un.f_fname = NULL;
 		}
 	      else
 		goto again;
@@ -1312,6 +1461,8 @@ fprintlog (struct filed *f, const char *from, int flags, const char *msg)
 	      f->f_type = F_UNUSED;
 	      errno = e;
 	      logerror (f->f_un.f_fname);
+	      free (f->f_un.f_fname);
+	      f->f_un.f_fname = NULL;
 	    }
 	}
       else if ((flags & SYNC_FILE) && !(f->f_flags & OMIT_SYNC))
@@ -1387,28 +1538,31 @@ wallmsg (struct filed *f, struct iovec *iov)
 
 /* Return a printable representation of a host address.  */
 const char *
-cvthname (struct sockaddr_in *f)
+cvthname (struct sockaddr *f, socklen_t len)
 {
-  struct hostent *hp;
+  int err;
   char *p;
 
-  dbg_printf ("cvthname(%s)\n", inet_ntoa (f->sin_addr));
-
-  if (f->sin_family != AF_INET)
+  err = getnameinfo (f, len, addrstr, sizeof (addrstr),
+		     NULL, 0, NI_NUMERICHOST);
+  if (err)
     {
-      dbg_printf ("Malformed from address.\n");
+      dbg_printf ("Malformed from address: %s.\n",
+		  gai_strerror (err));
       return "???";
     }
-  hp = gethostbyaddr ((char *) &f->sin_addr,
-		      sizeof (struct in_addr), f->sin_family);
-  if (hp == 0)
+
+  dbg_printf ("cvthname(%s)\n", addrstr);
+
+  err = getnameinfo (f, len, addrname, sizeof (addrname),
+		     NULL, 0, NI_NAMEREQD);
+  if (err)
     {
-      dbg_printf ("Host name for your address (%s) unknown.\n",
-		  inet_ntoa (f->sin_addr));
-      return inet_ntoa (f->sin_addr);
+      dbg_printf ("Host name for your address (%s) unknown.\n", addrstr);
+      return addrstr;
     }
 
-  p = strchr (hp->h_name, '.');
+  p = strchr (addrname, '.');
   if (p != NULL)
     {
       if (strcasecmp (p + 1, LocalDomain) == 0)
@@ -1425,7 +1579,7 @@ cvthname (struct sockaddr_in *f)
 		  if (strcasecmp (p + 1, StripDomains[count]) == 0)
 		    {
 		      *p = '\0';
-		      return hp->h_name;
+		      return addrname;
 		    }
 		  count++;
 		}
@@ -1435,21 +1589,21 @@ cvthname (struct sockaddr_in *f)
 	      count = 0;
 	      while (LocalHosts[count])
 		{
-		  if (strcasecmp (hp->h_name, LocalHosts[count]) == 0)
+		  if (strcasecmp (addrname, LocalHosts[count]) == 0)
 		    {
 		      *p = '\0';
-		      return hp->h_name;
+		      return addrname;
 		    }
 		  count++;
 		}
 	    }
 	}
     }
-  return hp->h_name;
+  return addrname;
 }
 
 void
-domark (int signo ARG_UNUSED)
+domark (int signo _GL_UNUSED_PARAMETER)
 {
   struct filed *f;
 
@@ -1493,13 +1647,13 @@ logerror (const char *type)
   logmsg (LOG_SYSLOG | LOG_ERR, buf, LocalHostName, ADDDATE);
 }
 
-RETSIGTYPE
-doexit (int signo ARG_UNUSED)
+void
+doexit (int signo _GL_UNUSED_PARAMETER)
 {
   _exit (0);
 }
 
-RETSIGTYPE
+void
 die (int signo)
 {
   struct filed *f;
@@ -1535,15 +1689,17 @@ die (int signo)
 	  unlink (funix[i].name);
       }
 
-  if (finet >= 0)
-    close (finet);
+  if (finet[IU_FD_IP4] >= 0)
+    close (finet[IU_FD_IP4]);
+  if (finet[IU_FD_IP6] >= 0)
+    close (finet[IU_FD_IP6]);
 
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 /* INIT -- Initialize syslogd from configuration table.  */
-RETSIGTYPE
-init (int signo ARG_UNUSED)
+void
+init (int signo _GL_UNUSED_PARAMETER)
 {
   FILE *cf;
   struct filed *f, *next, **nextp;
@@ -1555,24 +1711,15 @@ init (int signo ARG_UNUSED)
   char *cbuf;
   char *cline;
   int cont_line = 0;
-  struct servent *sp;
 
   dbg_printf ("init\n");
-  sp = getservbyname ("syslog", "udp");
-  if (sp == NULL)
-    {
-      errno = 0;
-      logerror ("network logging disabled (syslog/udp service unknown).");
-      logerror
-	("see syslogd(8) for details of whether and how to enable it.");
-      return;
-    }
-  LogPort = sp->s_port;
 
   /* Close all open log files.  */
   Initialized = 0;
   for (f = Files; f != NULL; f = next)
     {
+      int j;
+
       /* Flush any pending output.  */
       if (f->f_prevcount)
 	fprintlog (f, LocalHostName, 0, (char *) NULL);
@@ -1583,9 +1730,21 @@ init (int signo ARG_UNUSED)
 	case F_TTY:
 	case F_CONSOLE:
 	case F_PIPE:
+	  free (f->f_un.f_fname);
 	  close (f->f_file);
 	  break;
+	case F_FORW:
+	case F_FORW_SUSP:
+	case F_FORW_UNKN:
+	  free (f->f_un.f_forw.f_hname);
+	  break;
+	case F_USERS:
+	  for (j = 0; j < f->f_un.f_user.f_nusers; ++j)
+	    free (f->f_un.f_user.f_unames[j]);
+	  free (f->f_un.f_user.f_unames);
+	  break;
 	}
+      free (f->f_prevhost);
       next = f->f_next;
       free (f);
     }
@@ -1756,8 +1915,8 @@ init (int signo ARG_UNUSED)
 void
 cfline (const char *line, struct filed *f)
 {
-  struct hostent *hp;
-  int i, pri, negate_pri, excl_pri;
+  struct addrinfo hints, *rp;
+  int i, pri, negate_pri, excl_pri, err;
   unsigned int pri_set, pri_clear;
   char *bp;
   const char *p, *q;
@@ -1789,7 +1948,7 @@ cfline (const char *line, struct filed *f)
       *bp = '\0';
 
       /* Skip cruft.  */
-      while (strchr (",;", *q))
+      while (*q && strchr (",;", *q))
 	q++;
 
       bp = buf;
@@ -1891,24 +2050,59 @@ cfline (const char *line, struct filed *f)
       p++;
     }
 
+  if (!strlen(p))
+    {
+      /* Invalidate an entry with empty action field.  */
+      f->f_type = F_UNUSED;
+      logerror ("empty action field");
+      return;
+    }
+
   switch (*p)
     {
     case '@':
       f->f_un.f_forw.f_hname = strdup (++p);
-      hp = gethostbyname (p);
-      if (hp == NULL)
+      memset (&hints, 0, sizeof (hints));
+      hints.ai_family = usefamily;
+      hints.ai_socktype = SOCK_DGRAM;
+#ifdef AI_ADDRCONFIG
+      hints.ai_flags = AI_ADDRCONFIG;
+#endif
+
+      f->f_un.f_forw.f_addrlen = 0;	/* Invalidate address.  */
+      memset (&f->f_un.f_forw.f_addr, 0, sizeof (f->f_un.f_forw.f_addr));
+
+      err = getaddrinfo (p, LogPortText, &hints, &rp);
+      if (err)
 	{
-	  f->f_type = F_FORW_UNKN;
-	  f->f_prevcount = INET_RETRY_MAX;
+	  switch (err)
+	    {
+	      case EAI_AGAIN:	/* Known kinds of temporary error.  */
+	      case EAI_MEMORY:
+		f->f_type = F_FORW_UNKN;
+		f->f_prevcount = INET_RETRY_MAX;
+		break;
+
+	      case EAI_NONAME:	/* The most probable causes for failure.  */
+#if defined EAI_NODATA && (EAI_NODATA != EAI_NONAME)	/* FreeBSD complains.  */
+	      case EAI_NODATA:
+#endif
+#ifdef EAI_ADDRFAMILY
+	      case EAI_ADDRFAMILY:
+#endif
+	      default:		/* Catch system exceptions.  */
+		f->f_type = F_UNUSED;
+	    }
+
 	  f->f_time = time ((time_t *) 0);
 	}
       else
-	f->f_type = F_FORW;
-      memset (&f->f_un.f_forw.f_addr, 0, sizeof (f->f_un.f_forw.f_addr));
-      f->f_un.f_forw.f_addr.sin_family = AF_INET;
-      f->f_un.f_forw.f_addr.sin_port = LogPort;
-      if (f->f_type == F_FORW)
-	memcpy (&f->f_un.f_forw.f_addr.sin_addr, hp->h_addr, hp->h_length);
+	{
+	  f->f_type = F_FORW;
+	  f->f_un.f_forw.f_addrlen = rp->ai_addrlen;
+	  memcpy (&f->f_un.f_forw.f_addr, rp->ai_addr, rp->ai_addrlen);
+	  freeaddrinfo (rp);
+	}
       break;
 
     case '|':
@@ -1917,6 +2111,8 @@ cfline (const char *line, struct filed *f)
 	{
 	  f->f_type = F_UNUSED;
 	  logerror (p);
+	  free (f->f_un.f_fname);
+	  f->f_un.f_fname = NULL;
 	  break;
 	}
       if (strcmp (p, ctty) == 0)
@@ -1933,6 +2129,8 @@ cfline (const char *line, struct filed *f)
 	{
 	  f->f_type = F_UNUSED;
 	  logerror (p);
+	  free (f->f_un.f_fname);
+	  f->f_un.f_fname = NULL;
 	  break;
 	}
       if (strcmp (p, ctty) == 0)
@@ -1989,8 +2187,8 @@ decode (const char *name, CODE * codetab)
   return -1;
 }
 
-RETSIGTYPE
-dbg_toggle (int signo ARG_UNUSED)
+void
+dbg_toggle (int signo _GL_UNUSED_PARAMETER)
 {
   int dbg_save = dbg_output;
 
@@ -2021,8 +2219,8 @@ dbg_printf (const char *fmt, ...)
    not being doing this during a signal handler.  Instead we simply
    set a flag variable which will tell the main loop to go through a
    restart.  */
-RETSIGTYPE
-trigger_restart (int signo ARG_UNUSED)
+void
+trigger_restart (int signo _GL_UNUSED_PARAMETER)
 {
   restart = 1;
 }
