@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -54,9 +54,7 @@
  * <guyton@rand-unix>.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/param.h>
 #include <sys/ioctl.h>
@@ -100,10 +98,13 @@ static int peer;
 static int rexmtval = TIMEOUT;
 static int maxtimeout = 5 * TIMEOUT;
 
+/* Some systems define PKTSIZE in <arpa/tftp.h>.  */
+#ifndef PKTSIZE
 #define PKTSIZE	SEGSIZE+4
+#endif
 static char buf[PKTSIZE];
 static char ackbuf[PKTSIZE];
-static struct sockaddr_in from;
+static struct sockaddr_storage from;
 static socklen_t fromlen;
 
 void tftp (struct tftphdr *, int);
@@ -126,7 +127,7 @@ static int logging;
 
 static const char *errtomsg (int);
 static void nak (int);
-static const char *verifyhost (struct sockaddr_in *);
+static const char *verifyhost (struct sockaddr_storage *, socklen_t);
 
 
 
@@ -174,7 +175,7 @@ main (int argc, char *argv[])
   int index;
   register struct tftphdr *tp;
   int on, n;
-  struct sockaddr_in sin;
+  struct sockaddr_storage sin;
 
   set_program_name (argv[0]);
   iu_argp_init ("tftpd", default_program_authors);
@@ -202,14 +203,14 @@ main (int argc, char *argv[])
   if (ioctl (0, FIONBIO, &on) < 0)
     {
       syslog (LOG_ERR, "ioctl(FIONBIO): %m\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   fromlen = sizeof (from);
   n = recvfrom (0, buf, sizeof (buf), 0, (struct sockaddr *) &from, &fromlen);
   if (n < 0)
     {
       syslog (LOG_ERR, "recvfrom: %m\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   /*
    * Now that we have read the message out of the UDP
@@ -263,40 +264,44 @@ main (int argc, char *argv[])
     if (pid < 0)
       {
 	syslog (LOG_ERR, "fork: %m\n");
-	exit (1);
+	exit (EXIT_FAILURE);
       }
     else if (pid != 0)
       {
-	exit (0);
+	exit (EXIT_SUCCESS);
       }
   }
-  from.sin_family = AF_INET;
+
   alarm (0);
   close (0);
   close (1);
-  peer = socket (AF_INET, SOCK_DGRAM, 0);
+
+  /* The peer's address 'from' is valid at this point.
+   * 'from.ss_family' contains the correct address
+   * family for any callback connection, and 'fromlen'
+   * is the length of the corresponding address structure.  */
+  peer = socket (from.ss_family, SOCK_DGRAM, 0);
   if (peer < 0)
     {
       syslog (LOG_ERR, "socket: %m\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   memset (&sin, 0, sizeof (sin));
-  sin.sin_family = AF_INET;
-  if (bind (peer, (struct sockaddr *) &sin, sizeof (sin)) < 0)
+  sin.ss_family = from.ss_family;
+#if HAVE_STRUCT_SOCKADDR_SA_LEN
+  sin.ss_len = from.ss_len;
+#endif
+  if (bind (peer, (struct sockaddr *) &sin, fromlen) < 0)
     {
       syslog (LOG_ERR, "bind: %m\n");
-      exit (1);
-    }
-  if (connect (peer, (struct sockaddr *) &from, sizeof (from)) < 0)
-    {
-      syslog (LOG_ERR, "connect: %m\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
+
   tp = (struct tftphdr *) buf;
   tp->th_opcode = ntohs (tp->th_opcode);
   if (tp->th_opcode == RRQ || tp->th_opcode == WRQ)
     tftp (tp, n);
-  exit (1);
+  exit (EXIT_FAILURE);
 }
 
 struct formats;
@@ -329,7 +334,11 @@ tftp (struct tftphdr *tp, int size)
   register struct formats *pf;
   char *filename, *mode;
 
+#if HAVE_STRUCT_TFTPHDR_TH_U
   filename = cp = tp->th_stuff;
+#else
+  filename = cp = (char *) &(tp->th_stuff);
+#endif
 again:
   while (cp < buf + size)
     {
@@ -340,7 +349,7 @@ again:
   if (*cp != '\0')
     {
       nak (EBADOP);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   if (first)
     {
@@ -357,13 +366,27 @@ again:
   if (pf->f_mode == 0)
     {
       nak (EBADOP);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   ecode = (*pf->f_validate) (&filename, tp->th_opcode);
   if (logging)
     {
-      syslog (LOG_INFO, "%s: %s request for %s: %s",
-	      verifyhost (&from),
+      char *family;
+
+      switch (from.ss_family)
+	{
+	case AF_INET:
+	  family = "IPv4";
+	  break;
+	case AF_INET6:
+	  /* Should mapped IPv4 addresses be reported?  */
+	  family = "IPv6";
+	  break;
+	default:
+	  family = "?";
+	}
+      syslog (LOG_INFO, "%s (%s): %s request for %s: %s",
+	      verifyhost (&from, fromlen), family,
 	      tp->th_opcode == WRQ ? "write" : "read",
 	      filename, errtomsg (ecode));
     }
@@ -374,15 +397,15 @@ again:
        * bootfile pathname from a diskless Sun.
        */
       if (suppress_naks && *filename != '/' && ecode == ENOTFOUND)
-	exit (0);
+	exit (EXIT_SUCCESS);
       nak (ecode);
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   if (tp->th_opcode == WRQ)
     (*pf->f_recv) (pf);
   else
     (*pf->f_send) (pf);
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 
@@ -507,7 +530,7 @@ timer (int sig)
 
   timeout += rexmtval;
   if (timeout >= maxtimeout)
-    exit (1);
+    exit (EXIT_FAILURE);
   siglongjmp (timeoutbuf, 1);
 }
 
@@ -517,7 +540,7 @@ timer (int sig)
 void
 send_file (struct formats *pf)
 {
-  struct tftphdr *dp, *r_init ();
+  struct tftphdr *dp, *r_init (void);
   register struct tftphdr *ap;	/* ack packet */
   register int size, n;
   volatile int block;
@@ -534,13 +557,14 @@ send_file (struct formats *pf)
 	  nak (errno + 100);
 	  goto abort;
 	}
-      dp->th_opcode = htons ((u_short) DATA);
-      dp->th_block = htons ((u_short) block);
+      dp->th_opcode = htons ((unsigned short) DATA);
+      dp->th_block = htons ((unsigned short) block);
       timeout = 0;
       sigsetjmp (timeoutbuf, SIGALRM);
 
     send_data:
-      if (send (peer, (const char *) dp, size + 4, 0) != size + 4)
+      if (sendto (peer, (const char *) dp, size + 4, 0,
+		 (struct sockaddr *) &from, fromlen) != size + 4)
 	{
 	  syslog (LOG_ERR, "tftpd: write: %m\n");
 	  goto abort;
@@ -556,19 +580,19 @@ send_file (struct formats *pf)
 	      syslog (LOG_ERR, "tftpd: read: %m\n");
 	      goto abort;
 	    }
-	  ap->th_opcode = ntohs ((u_short) ap->th_opcode);
-	  ap->th_block = ntohs ((u_short) ap->th_block);
+	  ap->th_opcode = ntohs ((unsigned short) ap->th_opcode);
+	  ap->th_block = ntohs ((unsigned short) ap->th_block);
 
 	  if (ap->th_opcode == ERROR)
 	    goto abort;
 
 	  if (ap->th_opcode == ACK)
 	    {
-	      if ((u_short) ap->th_block == (u_short) block)
+	      if ((unsigned short) ap->th_block == (unsigned short) block)
 		break;
 	      /* Re-synchronize with the other side */
 	      synchnet (peer);
-	      if ((u_short) ap->th_block == (u_short) (block - 1))
+	      if ((unsigned short) ap->th_block == (unsigned short) (block - 1))
 		goto send_data;
 	    }
 
@@ -583,7 +607,7 @@ abort:
 void
 justquit (int sig)
 {
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 
@@ -593,7 +617,7 @@ justquit (int sig)
 void
 recvfile (struct formats *pf)
 {
-  struct tftphdr *dp, *w_init ();
+  struct tftphdr *dp, *w_init (void);
   register struct tftphdr *ap;	/* ack buffer */
   register int n, size;
   volatile int block;
@@ -605,12 +629,12 @@ recvfile (struct formats *pf)
   do
     {
       timeout = 0;
-      ap->th_opcode = htons ((u_short) ACK);
-      ap->th_block = htons ((u_short) block);
+      ap->th_opcode = htons ((unsigned short) ACK);
+      ap->th_block = htons ((unsigned short) block);
       block++;
       sigsetjmp (timeoutbuf, SIGALRM);
     send_ack:
-      if (send (peer, ackbuf, 4, 0) != 4)
+      if (sendto (peer, ackbuf, 4, 0, (struct sockaddr *) &from, fromlen) != 4)
 	{
 	  syslog (LOG_ERR, "tftpd: write: %m\n");
 	  goto abort;
@@ -626,8 +650,8 @@ recvfile (struct formats *pf)
 	      syslog (LOG_ERR, "tftpd: read: %m\n");
 	      goto abort;
 	    }
-	  dp->th_opcode = ntohs ((u_short) dp->th_opcode);
-	  dp->th_block = ntohs ((u_short) dp->th_block);
+	  dp->th_opcode = ntohs ((unsigned short) dp->th_opcode);
+	  dp->th_block = ntohs ((unsigned short) dp->th_block);
 	  if (dp->th_opcode == ERROR)
 	    goto abort;
 	  if (dp->th_opcode == DATA)
@@ -657,9 +681,9 @@ recvfile (struct formats *pf)
   write_behind (file, pf->f_convert);
   fclose (file);		/* close data file */
 
-  ap->th_opcode = htons ((u_short) ACK);	/* send the "final" ack */
-  ap->th_block = htons ((u_short) (block));
-  send (peer, ackbuf, 4, 0);
+  ap->th_opcode = htons ((unsigned short) ACK);	/* send the "final" ack */
+  ap->th_block = htons ((unsigned short) (block));
+  sendto (peer, ackbuf, 4, 0, (struct sockaddr *) &from, fromlen);
 
   signal (SIGALRM, justquit);	/* just quit on timeout */
   alarm (rexmtval);
@@ -669,7 +693,7 @@ recvfile (struct formats *pf)
       dp->th_opcode == DATA &&	/* and got a data block */
       block == dp->th_block)
     {				/* then my last ack was lost */
-      send (peer, ackbuf, 4, 0);	/* resend final ack */
+      sendto (peer, ackbuf, 4, 0, (struct sockaddr *) &from, fromlen);	/* resend final ack */
     }
 abort:
   return;
@@ -720,8 +744,8 @@ nak (int error)
   register struct errmsg *pe;
 
   tp = (struct tftphdr *) buf;
-  tp->th_opcode = htons ((u_short) ERROR);
-  tp->th_code = htons ((u_short) error);
+  tp->th_opcode = htons ((unsigned short) ERROR);
+  tp->th_code = htons ((unsigned short) error);
   for (pe = errmsgs; pe->e_code >= 0; pe++)
     if (pe->e_code == error)
       break;
@@ -734,21 +758,25 @@ nak (int error)
   length = strlen (pe->e_msg);
   tp->th_msg[length] = '\0';
   length += 5;
-  if (send (peer, buf, length, 0) != length)
+  if (sendto (peer, buf, length, 0, (struct sockaddr *) &from, fromlen) != length)
     syslog (LOG_ERR, "nak: %m\n");
 }
 
 static const char *
-verifyhost (struct sockaddr_in *fromp)
+verifyhost (struct sockaddr_storage *fromp, socklen_t frlen)
 {
-  struct hostent *hp;
+  int rc;
+  static char host[NI_MAXHOST];
 
-  hp = gethostbyaddr ((char *) &fromp->sin_addr, sizeof (fromp->sin_addr),
-		      fromp->sin_family);
-  if (hp)
-    return hp->h_name;
+  rc = getnameinfo ((struct sockaddr *) fromp, frlen,
+		    host, sizeof (host), NULL, 0, 0);
+  if (rc == 0)
+    return host;
   else
-    return inet_ntoa (fromp->sin_addr);
+    {
+      syslog (LOG_ERR, "getnameinfo: %s\n", gai_strerror(rc));
+      return "0.0.0.0";
+    }
 }
 
 static const char usage_str[] =
