@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -53,15 +53,10 @@
  * the progress
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/socket.h>
-#ifdef HAVE_OSOCKADDR_H
-# include <osockaddr.h>
-#endif
 #include <protocols/talkd.h>
 #include <netinet/in.h>
 #include "talk.h"
@@ -81,7 +76,7 @@ struct sockaddr_in my_addr = { AF_INET };
 struct in_addr my_machine_addr;
 struct in_addr his_machine_addr;
 
-u_short daemon_port;		/* port number of the talk daemon */
+unsigned short daemon_port;		/* port number of the talk daemon */
 
 int ctl_sockt;
 int sockt;
@@ -90,7 +85,7 @@ int invitation_waiting = 0;
 CTL_MSG msg;
 
 int
-open_sockt ()
+open_sockt (void)
 {
   socklen_t length;
 
@@ -110,7 +105,7 @@ open_sockt ()
 
 /* open the ctl socket */
 int
-open_ctl ()
+open_ctl (void)
 {
   socklen_t length;
 
