@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -47,9 +47,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -57,9 +55,6 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#ifdef HAVE_OSOCKADDR_H
-# include <osockaddr.h>
-#endif
 #include <protocols/talkd.h>
 #include <netdb.h>
 #include <stdio.h>
@@ -81,7 +76,7 @@ get_addrs (char *my_machine_name, char *his_machine_name)
       herror ((char *) NULL);
       exit (-1);
     }
-  bcopy (hp->h_addr, (char *) &my_machine_addr, hp->h_length);
+  memmove (&my_machine_addr, hp->h_addr, hp->h_length);
   /*
    * If the callee is on-machine, just copy the
    * network address, otherwise do a lookup...
@@ -95,7 +90,7 @@ get_addrs (char *my_machine_name, char *his_machine_name)
 	  herror ((char *) NULL);
 	  exit (-1);
 	}
-      bcopy (hp->h_addr, (char *) &his_machine_addr, hp->h_length);
+      memmove (&his_machine_addr, hp->h_addr, hp->h_length);
     }
   else
     his_machine_addr = my_machine_addr;
