@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -56,22 +56,15 @@
  * as well as the signal handling routines.
  */
 
-#ifdef HAVE_TERMIOS_H
-# include <termios.h>
-#else
-# include <sys/ioctl.h>
-# ifdef HAVE_SYS_IOCTL_COMPAT_H
-#  include <sys/ioctl_compat.h>
-# endif
-#endif
-
+#include <termios.h>
 #include <unistd.h>
 #include <signal.h>
 #include <error.h>
 #include "talk.h"
+#include "unused-parameter.h"
 
-static RETSIGTYPE
-sig_sent (int sig ARG_UNUSED)
+static void
+sig_sent (int sig _GL_UNUSED_PARAMETER)
 {
 
   message ("Connection closing. Exiting");
@@ -83,7 +76,7 @@ sig_sent (int sig ARG_UNUSED)
  * and build the various windows.
  */
 int
-init_display ()
+init_display (void)
 {
 #ifdef HAVE_SIGACTION
   struct sigaction siga;
@@ -94,7 +87,7 @@ init_display ()
 #endif
 
   if (initscr () == NULL)
-    error (1, 0, "Terminal type unset or lacking necessary features.");
+    error (EXIT_FAILURE, 0, "Terminal type unset or lacking necessary features.");
 
 #ifdef HAVE_SIGACTION
   sigaction (SIGTSTP, (struct sigaction *) 0, &siga);
@@ -146,7 +139,7 @@ init_display ()
  * connection are the three edit characters.
  */
 int
-set_edit_chars ()
+set_edit_chars (void)
 {
   int cc;
   char buf[3];
@@ -155,7 +148,7 @@ set_edit_chars ()
   struct termios tty;
   cc_t disable = (cc_t) - 1, erase, werase, kill;
 
-# if !defined (_POSIX_VDISABLE) && defined (HAVE_FPATHCONF) && defined (_PC_VDISABLE)
+# if !defined _POSIX_VDISABLE && defined HAVE_FPATHCONF && defined _PC_VDISABLE
   disable = fpathconf (0, _PC_VDISABLE);
 # endif
 
@@ -214,7 +207,7 @@ set_edit_chars ()
  * All done talking...hang up the phone and reset terminal thingy's
  */
 int
-quit ()
+quit (void)
 {
 
   if (curses_initialized)
@@ -226,5 +219,5 @@ quit ()
     }
   if (invitation_waiting)
     send_delete ();
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
