@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -47,9 +47,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <string.h>
 
@@ -59,9 +57,6 @@
 #include <time.h>
 #include <signal.h>
 #include <netinet/in.h>
-#ifdef HAVE_OSOCKADDR_H
-# include <osockaddr.h>
-#endif
 #include <protocols/talkd.h>
 #include <errno.h>
 #include <unistd.h>
@@ -75,7 +70,7 @@ static char *answers[] = {
   "Target machine is too confused to talk to us",	/* FAILED */
   "Target machine does not recognize us",	/* MACHINE_UNKNOWN */
   "Your party is refusing messages",	/* PERMISSION_REFUSED */
-  "Target machine can not handle remote talk",	/* UNKNOWN_REQUEST */
+  "Target machine cannot handle remote talk",	/* UNKNOWN_REQUEST */
   "Target machine indicates protocol mismatch",	/* BADVERSION */
   "Target machine indicates protocol botch (addr)",	/* BADADDR */
   "Target machine indicates protocol botch (ctl_addr)",	/* BADCTLADDR */
@@ -96,7 +91,7 @@ jmp_buf invitebuf;
  * Transmit the invitation and process the response
  */
 int
-announce_invite ()
+announce_invite (void)
 {
   CTL_RESPONSE response;
 
@@ -119,7 +114,7 @@ announce_invite ()
 /*
  * Routine called on interupt to re-invite the callee
  */
-RETSIGTYPE
+void
 re_invite (int sig)
 {
 
@@ -132,7 +127,7 @@ re_invite (int sig)
 }
 
 int
-invite_remote ()
+invite_remote (void)
 {
   int new_sockt;
   struct itimerval itimer;
@@ -190,7 +185,7 @@ invite_remote ()
  * Tell the daemon to remove your invitation
  */
 int
-send_delete ()
+send_delete (void)
 {
 
   msg.type = DELETE;
