@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -53,9 +53,7 @@
  * ctl.c
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/types.h>
 #include <sys/ioctl.h>
@@ -77,7 +75,7 @@
  * The routine to do the actual talking
  */
 int
-talk ()
+talk (void)
 {
   fd_set read_template, read_set;
   int stdin_fd = fileno (stdin);
