@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -52,9 +52,7 @@
  * if we are slow connecting.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/time.h>
 #include <time.h>
@@ -74,7 +72,7 @@ disp_msg (int sig)
 }
 
 int
-start_msgs ()
+start_msgs (void)
 {
   struct itimerval itimer;
 
@@ -88,7 +86,7 @@ start_msgs ()
 }
 
 int
-end_msgs ()
+end_msgs (void)
 {
   struct itimerval itimer;
 
