@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -47,9 +47,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <stdlib.h>
 
 #include "talk.h"
@@ -107,7 +105,7 @@ main (int argc, char *argv[])
       printf ("Standard input must be a tty, not a pipe or a file\n");
       exit (-1);
     }
-  
+
   get_names (argc, argv);
   init_display ();
   open_ctl ();
