@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -52,9 +52,9 @@ extern struct sockaddr_in ctl_addr;
 extern struct sockaddr_in my_addr;
 extern struct in_addr my_machine_addr;
 extern struct in_addr his_machine_addr;
-extern u_short daemon_port;
+extern unsigned short daemon_port;
 extern int ctl_sockt;
 extern CTL_MSG msg;
 
 /* ctl_transact.c */
-int ctl_transact (struct in_addr, CTL_MSG, int, CTL_RESPONSE *); 
+int ctl_transact (struct in_addr, CTL_MSG, int, CTL_RESPONSE *);
