@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010 Free Software Foundation, Inc.
+  2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -120,7 +120,7 @@ netdef_parse (char *str)
   if (!netdef)
     {
       syslog (LOG_ERR, "out of memory");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
   netdef->next = NULL;
@@ -218,7 +218,7 @@ read_acl (char *config_file)
       if (!acl)
 	{
 	  syslog (LOG_CRIT, "out of memory");
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
       acl->next = NULL;
       acl->action = action;
