@@ -1,6 +1,7 @@
 /*
   Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-  2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation,
+  Inc.
 
   This file is part of GNU Inetutils.
 
@@ -17,13 +18,13 @@
   You should have received a copy of the GNU General Public License
   along with this program.  If not, see `http://www.gnu.org/licenses/'. */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 #include <argp.h>
 #include <intalkd.h>
 #include <signal.h>
 #include <libinetutils.h>
+#include <progname.h>
+#include "unused-parameter.h"
 
 #ifndef LOG_FACILITY
 # define LOG_FACILITY LOG_DAEMON
@@ -97,6 +98,7 @@ static struct argp argp = {argp_options, parse_opt, args_doc, doc};
 int
 main (int argc, char *argv[])
 {
+  set_program_name (argv[0]);
   /* Parse command line */
   iu_argp_init ("talkd", program_authors);
   argp_parse (&argp, argc, argv, 0, NULL, NULL);
@@ -108,24 +110,24 @@ main (int argc, char *argv[])
 }
 
 void
-talkd_init ()
+talkd_init (void)
 {
   openlog ("talkd", LOG_PID, LOG_FACILITY);
   hostname = localhost ();
   if (!hostname)
     {
       syslog (LOG_ERR, "can't determine my hostname: %m");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 }
 
 time_t last_msg_time;
 
 static void
-alarm_handler (int err ARG_UNUSED)
+alarm_handler (int err _GL_UNUSED_PARAMETER)
 {
   if ((time (NULL) - last_msg_time) >= max_idle_time)
-    exit (0);
+    exit (EXIT_SUCCESS);
   alarm (timeout);
 }
 
