@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -47,16 +47,12 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #ifdef HAVE_SYS_PARAM_H
 # include <sys/param.h>
 #endif
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
+#include <sys/types.h>
 #include <sys/wait.h>
 #include <sys/file.h>
 
@@ -73,9 +69,6 @@
 #include <errno.h>
 
 #include <stdlib.h>
-#ifdef HAVE_MALLOC_H
-# include <malloc.h>
-#endif
 
 #include <arpa/inet.h>
 #include <arpa/telnet.h>
@@ -93,11 +86,11 @@
 #include "xalloc.h"
 #include "xvasprintf.h"
 
-#if !defined(CRAY) && !defined(sysV88)
+#if !defined CRAY && !defined sysV88
 # ifdef HAVE_NETINET_IN_SYSTM_H
 #  include <netinet/in_systm.h>
 # endif
-# if (defined(vax) || defined(tahoe) || defined(hp300)) && !defined(ultrix)
+# if (defined vax || defined tahoe || defined hp300) && !defined ultrix
 #  include <machine/endian.h>
 # endif	/* vax */
 #endif /* !defined(CRAY) && !defined(sysV88) */
@@ -424,7 +417,7 @@ sendcmd (int argc, char **argv)
 }
 
 static int
-send_esc ()
+send_esc (void)
 {
   NETADD (escape);
   return 1;
@@ -434,7 +427,9 @@ int
 send_tncmd (void (*func) (), char *cmd, char *name)
 {
   char **cpp;
+#if !HAVE_DECL_TELOPTS
   extern char *telopts[];
+#endif
   register int val = 0;
 
   if (isprefix (name, "help") || isprefix (name, "?"))
@@ -469,7 +464,11 @@ send_tncmd (void (*func) (), char *cmd, char *name)
     }
   if (cpp)
     {
+#if HAVE_DECL_TELOPTS
+      val = cpp - (char **) telopts;
+#else /* !HAVE_DECL_TELOPTS */
       val = cpp - telopts;
+#endif
     }
   else
     {
@@ -528,7 +527,7 @@ send_wontcmd (char *name)
 }
 
 static int
-send_help ()
+send_help (void)
 {
   struct sendlist *s;		/* pointer to current command */
   for (s = Sendlist; s->name; s++)
@@ -545,14 +544,14 @@ send_help ()
  */
 
 static int
-lclchars ()
+lclchars (void)
 {
   donelclchars = 1;
   return 1;
 }
 
 static int
-togdebug ()
+togdebug (void)
 {
 #ifndef NOT43
   if (net > 0 && (SetSockOpt (net, SOL_SOCKET, SO_DEBUG, debug)) < 0)
@@ -573,7 +572,7 @@ togdebug ()
 
 
 static int
-togcrlf ()
+togcrlf (void)
 {
   if (crlf)
     {
@@ -713,7 +712,7 @@ togxbinary (int val)
 
 
 static int togglehelp (void);
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
 extern int auth_togdebug (int);
 #endif
 #ifdef	ENCRYPTION
@@ -743,7 +742,7 @@ static struct togglelist Togglelist[] = {
    0,
    &autosynch,
    "send interrupt characters in urgent mode"},
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
   {"autologin",
    "automatic sending of login and/or authentication info",
    0,
@@ -813,7 +812,7 @@ static struct togglelist Togglelist[] = {
    &localchars,
    "recognize certain control characters"},
   {" ", "", 0},			/* empty line */
-#if defined(unix) && defined(TN3270)
+#if defined unix && defined TN3270
   {"apitrace",
    "(debugging) toggle tracing of API transactions",
    0,
@@ -860,7 +859,7 @@ static struct togglelist Togglelist[] = {
 };
 
 static int
-togglehelp ()
+togglehelp (void)
 {
   struct togglelist *c;
 
@@ -997,7 +996,7 @@ static struct setlist Setlist[] = {
   {0}
 };
 
-#if defined(CRAY) && !defined(__STDC__)
+#if defined CRAY && !defined __STDC__
 /* Work around compiler bug in pcc 4.1.5 */
 void
 _setlist_init ()
@@ -1227,7 +1226,7 @@ unsetcmd (int argc, char *argv[])
 extern int kludgelinemode;
 
 static int
-dokludgemode ()
+dokludgemode (void)
 {
   kludgelinemode = 1;
   send_wont (TELOPT_LINEMODE, 1);
@@ -1238,7 +1237,7 @@ dokludgemode ()
 #endif
 
 static int
-dolinemode ()
+dolinemode (void)
 {
 #ifdef	KLUDGELINEMODE
   if (kludgelinemode)
@@ -1250,7 +1249,7 @@ dolinemode ()
 }
 
 static int
-docharmode ()
+docharmode (void)
 {
 #ifdef	KLUDGELINEMODE
   if (kludgelinemode)
@@ -1263,8 +1262,7 @@ docharmode ()
 }
 
 static int
-dolmmode (bit, on)
-     int bit, on;
+dolmmode (int bit, int on)
 {
   unsigned char c;
   extern int linemode;
@@ -1305,7 +1303,7 @@ struct modelist
   int arg1;
 };
 
-extern int modehelp ();
+extern int modehelp (void);
 
 static struct modelist ModeList[] = {
   {"character", "Disable LINEMODE option", docharmode, 1},
@@ -1342,7 +1340,7 @@ static struct modelist ModeList[] = {
 
 
 int
-modehelp ()
+modehelp (void)
 {
   struct modelist *mt;
 
@@ -1507,7 +1505,7 @@ setescape (int argc, char *argv[])
 }
 
 static int
-togcrmod ()
+togcrmod (void)
 {
   crmod = !crmod;
   printf ("Deprecated usage - please use 'toggle crmod' in the future.\n");
@@ -1517,7 +1515,7 @@ togcrmod ()
 }
 
 int
-suspend ()
+suspend (void)
 {
 #ifdef	SIGTSTP
   setcommandmode ();
@@ -1546,7 +1544,7 @@ suspend ()
   return 1;
 }
 
-#if !defined(TN3270)
+#if !defined TN3270
 int
 shell (int argc, char *argv[])
 {
@@ -1568,7 +1566,7 @@ shell (int argc, char *argv[])
 	 */
 	register char *shellp, *shellname;
 # ifndef strrchr
-	extern char *strrchr ();
+	extern char *strrchr (const char *, int);
 # endif
 
 	shellp = getenv ("SHELL");
@@ -1616,12 +1614,12 @@ bye (int argc, char *argv[])
       NetClose (net);
       connected = 0;
       resettermname = 1;
-#if defined(AUTHENTICATION) || defined(ENCRYPTION)
+#if defined AUTHENTICATION || defined ENCRYPTION
       auth_encrypt_connect (connected);
 #endif /* defined(AUTHENTICATION) || defined(ENCRYPTION) */
       /* reset options */
       tninit ();
-#if defined(TN3270)
+#if defined TN3270
       SetIn3270 ();		/* Get out of 3270 mode */
 #endif /* defined(TN3270) */
     }
@@ -1633,7 +1631,7 @@ bye (int argc, char *argv[])
 }
 
 int
-quit ()
+quit (void)
 {
   call (bye, "bye", "fromquit", 0);
   Exit (0);
@@ -1641,7 +1639,7 @@ quit ()
 }
 
 int
-logout ()
+logout (void)
 {
   send_do (TELOPT_LOGOUT, 1);
   netflush ();
@@ -1661,7 +1659,7 @@ struct slclist
   int arg;
 };
 
-static void slc_help ();
+static void slc_help (void);
 
 struct slclist SlcList[] = {
   {"export", "Use local special character definitions",
@@ -1676,7 +1674,7 @@ struct slclist SlcList[] = {
 };
 
 static void
-slc_help ()
+slc_help (void)
 {
   struct slclist *c;
 
@@ -1693,17 +1691,14 @@ slc_help ()
 }
 
 static struct slclist *
-getslc (name)
-     char *name;
+getslc (char *name)
 {
   return (struct slclist *)
     genget (name, (char **) SlcList, sizeof (struct slclist));
 }
 
 static int
-slccmd (argc, argv)
-     int argc;
-     char *argv[];
+slccmd (int argc, char **argv)
 {
   struct slclist *c;
 
@@ -1748,7 +1743,7 @@ extern void
 env_undefine (const char *),
 env_export (const char *),
 env_unexport (const char *), env_send (const char *),
-#if defined(OLD_ENVIRON) && defined(ENV_HACK)
+#if defined OLD_ENVIRON && defined ENV_HACK
   env_varval (const char *),
 #endif
   env_list (void);
@@ -1766,7 +1761,7 @@ struct envlist EnvList[] = {
   {"send", "Send an environment variable", env_send, 1},
   {"list", "List the current environment variables",
    env_list, 0},
-#if defined(OLD_ENVIRON) && defined(ENV_HACK)
+#if defined OLD_ENVIRON && defined ENV_HACK
   {"varval", "Reverse VAR and VALUE (auto, right, wrong, status)",
    env_varval, 1},
 #endif
@@ -1776,7 +1771,7 @@ struct envlist EnvList[] = {
 };
 
 static void
-env_help ()
+env_help (void)
 {
   struct envlist *c;
 
@@ -1861,13 +1856,13 @@ env_find (const char *var)
 }
 
 void
-env_init ()
+env_init (void)
 {
   extern char **environ;
   register char **epp, *cp;
   register struct env_lst *ep;
 #ifndef strchr
-  extern char *strchr ();
+  extern char *strchr (const char *, int);
 #endif
 
   for (epp = environ; *epp; epp++)
@@ -1998,7 +1993,7 @@ env_send (const char *var)
 }
 
 void
-env_list ()
+env_list (void)
 {
   register struct env_lst *ep;
 
@@ -2038,7 +2033,7 @@ env_getvalue (const char *var)
   return (NULL);
 }
 
-#if defined(OLD_ENVIRON) && defined(ENV_HACK)
+#if defined OLD_ENVIRON && defined ENV_HACK
 void
 env_varval (const char *what)
 {
@@ -2085,7 +2080,7 @@ env_varval (const char *what)
 }
 #endif
 
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
 /*
  * The AUTHENTICATE command.
  */
@@ -2297,7 +2292,7 @@ encrypt_cmd (int argc, char *argv[])
 }
 #endif /* ENCRYPTION */
 
-#if defined(unix) && defined(TN3270)
+#if defined unix && defined TN3270
 static void
 filestuff (int fd)
 {
@@ -2375,7 +2370,7 @@ status (int argc, char *argv[])
     {
       printf ("No connection.\n");
     }
-#if !defined(TN3270)
+#if !defined TN3270
   printf ("Escape character is '%s'.\n", control (escape));
   fflush (stdout);
 #else /* !defined(TN3270) */
@@ -2383,7 +2378,7 @@ status (int argc, char *argv[])
     {
       printf ("Escape character is '%s'.\n", control (escape));
     }
-# if defined(unix)
+# if defined unix
   if ((argc >= 2) && !strcmp (argv[1], "everything"))
     {
       printf ("SIGIO received %d time%s.\n",
@@ -2626,7 +2621,7 @@ tn (int argc, char *argv[])
 	}
 
       connected++;
-# if defined(AUTHENTICATION) || defined(ENCRYPTION)
+# if defined AUTHENTICATION || defined ENCRYPTION
       auth_encrypt_connect (connected);
 # endif	/* defined(AUTHENTICATION) || defined(ENCRYPTION) */
     }
@@ -2678,7 +2673,7 @@ tn (int argc, char *argv[])
 	  perror ("telnet: socket");
 	  return 0;
 	}
-# if defined(IPPROTO_IP) && defined(IP_TOS)
+# if defined IPPROTO_IP && defined IP_TOS
       {
 #  ifdef IPTOS_LOWDELAY
 	const int tos = IPTOS_LOWDELAY;
@@ -2717,7 +2712,7 @@ tn (int argc, char *argv[])
 	  return 0;
 	}
       connected++;
-# if defined(AUTHENTICATION) || defined(ENCRYPTION)
+# if defined AUTHENTICATION || defined ENCRYPTION
       auth_encrypt_connect (connected);
 # endif	/* defined(AUTHENTICATION) || defined(ENCRYPTION) */
     }
@@ -2766,18 +2761,18 @@ static char
   togglestring[] = "toggle operating parameters ('toggle ?' for more)",
   slchelp[] = "change state of special characters ('slc ?' for more)",
   displayhelp[] = "display operating parameters",
-#if defined(TN3270) && defined(unix)
+#if defined TN3270 && defined unix
   transcomhelp[] = "specify Unix command for transparent mode pipe",
 #endif
   /* defined(TN3270) && defined(unix) */
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
   authhelp[] = "turn on (off) authentication ('auth ?' for more)",
 #endif
 #ifdef	ENCRYPTION
   encrypthelp[] = "turn on (off) encryption ('encrypt ?' for more)",
 #endif
   /* ENCRYPTION */
-#if defined(unix)
+#if defined unix
   zhelp[] = "suspend telnet",
 #endif
   /* defined(unix) */
@@ -2785,7 +2780,7 @@ static char
   envhelp[] = "change environment variables ('environ ?' for more)",
   modestring[] = "try to enter line or character mode ('mode ?' for more)";
 
-static int help ();
+static int help (int argc, char **argv);
 
 static Command cmdtab[] = {
   {"close", closehelp, bye, 1},
@@ -2800,19 +2795,19 @@ static Command cmdtab[] = {
   {"status", statushelp, status, 0},
   {"toggle", togglestring, toggle, 0},
   {"slc", slchelp, slccmd, 0},
-#if defined(TN3270) && defined(unix)
+#if defined TN3270 && defined unix
   {"transcom", transcomhelp, settranscom, 0},
 #endif /* defined(TN3270) && defined(unix) */
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
   {"auth", authhelp, auth_cmd, 0},
 #endif
 #ifdef	ENCRYPTION
   {"encrypt", encrypthelp, encrypt_cmd, 0},
 #endif /* ENCRYPTION */
-#if defined(unix)
+#if defined unix
   {"z", zhelp, suspend, 0},
 #endif /* defined(unix) */
-#if defined(TN3270)
+#if defined TN3270
   {"!", shellhelp, shell, 1},
 #else
   {"!", shellhelp, shell, 0},
@@ -2873,7 +2868,7 @@ command (int top, char *tbuf, int cnt)
   if (!top)
     {
       putchar ('\n');
-#if defined(unix)
+#if defined unix
     }
   else
     {
@@ -2946,7 +2941,7 @@ command (int top, char *tbuf, int cnt)
 	{
 	  longjmp (toplevel, 1);
 	}
-#if defined(TN3270)
+#if defined TN3270
       if (shell_active == 0)
 	{
 	  setconnmode (0);
