@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -53,31 +53,25 @@
 # define BSD 43
 #endif
 
-#ifdef HAVE_TERMIOS_H
-# define USE_TERMIO
-#else /* !HAVE_TERMIOS_H */
-# ifdef HAVE_TERMIO_H
-#  define USE_TERMIO
-#  define SYSV_TERMIO
-# endif
-#endif /* HAVE_TERMIOS_H */
+
+#define USE_TERMIO
 
 /*
  * ucb stdio.h defines BSD as something wierd
  */
-#if defined(sun) && defined(__svr4__)
+#if defined sun && defined __svr4__
 # define BSD 43
 #endif
 
 #ifndef USE_TERMIO
-# if BSD > 43 || defined(SYSV_TERMIO)
+# if BSD > 43 || defined SYSV_TERMIO
 #  define USE_TERMIO
 # endif
 #endif
 
 #include <stdio.h>
 #include <setjmp.h>
-#if defined(CRAY) && !defined(NO_BSD_SETJMP)
+#if defined CRAY && !defined NO_BSD_SETJMP
 # include <bsdsetjmp.h>
 #endif
 #include <sys/ioctl.h>
@@ -96,15 +90,15 @@
 #  endif
 # endif
 #endif
-#if defined(NO_CC_T) || !defined(USE_TERMIO)
-# if !defined(USE_TERMIO)
+#if defined NO_CC_T || !defined USE_TERMIO
+# if !defined USE_TERMIO
 typedef char cc_t;
 # else
 typedef unsigned char cc_t;
 # endif
 #endif
 
-#if defined (USE_TERMIO) && !defined (SYSV_TERMIO)
+#if defined USE_TERMIO && !defined SYSV_TERMIO
 # define termio termios
 #endif
 
@@ -156,7 +150,7 @@ extern int autologin,		/* Autologin enabled */
   dontlecho,			/* do we suppress local echoing right now? */
   crmod, netdata,		/* Print out network data flow */
   prettydump,			/* Print "netdata" output in user readable format */
-#if defined(TN3270)
+#if defined TN3270
   cursesdata,			/* Print out curses data flow */
   apitrace,			/* Trace API transactions */
 #endif
@@ -393,7 +387,7 @@ extern cc_t termSuspChar;
 # else
 #  define termSuspChar		new_tc.c_cc[VSUSP]
 # endif
-# if defined(VFLUSHO) && !defined(VDISCARD)
+# if defined VFLUSHO && !defined VDISCARD
 #  define VDISCARD VFLUSHO
 # endif
 # ifndef VDISCARD
@@ -442,7 +436,7 @@ extern cc_t termAytChar;
 #  define termAytChar		new_tc.c_cc[VSTATUS]
 # endif
 
-# if !defined(CRAY) || defined(__STDC__)
+# if !defined CRAY || defined __STDC__
 #  define termEofCharp		&termEofChar
 #  define termEraseCharp	&termEraseChar
 #  define termIntCharp		&termIntChar
@@ -484,7 +478,7 @@ extern cc_t termAytChar;
 extern Ring netoring, netiring, ttyoring, ttyiring;
 
 /* Tn3270 section */
-#if defined(TN3270)
+#if defined TN3270
 
 extern int HaveInput,		/* Whether an asynchronous I/O indication came in */
   noasynchtty,			/* Don't do signals on I/O (SIGURG, SIGIO) */
