@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -47,9 +47,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <sys/types.h>
 
@@ -77,7 +75,7 @@
  * Initialize variables.
  */
 void
-tninit ()
+tninit (void)
 {
   init_terminal ();
 
@@ -87,7 +85,7 @@ tninit ()
 
   init_sys ();
 
-#if defined(TN3270)
+#if defined TN3270
   init_3270 ();
 #endif
 }
@@ -108,7 +106,7 @@ static struct argp_option argp_options[] = {
 #define GRID 10
   { NULL, 0, NULL, 0,
     "General options:", GRID },
-    
+
   { "ipv4", '4', NULL, 0,
     "use only IPv4", GRID+1 },
   { "ipv6", '6', NULL, 0,
@@ -153,22 +151,22 @@ static struct argp_option argp_options[] = {
     "Authentication and Kerberos options:", GRID },
   { "disable-auth", 'X', "ATYPE", 0,
     "disable type ATYPE authentication", GRID+1 },
-# if defined(KRB4)  
+# if defined KRB4
   { "realm", 'k', "REALM", 0,
     "obtain tickets for the remote host in REALM "
     "instead of the remote host's realm", GRID+1 },
 # endif
-# if defined(KRB5) && defined(FORWARD)  
+# if defined KRB5 && defined FORWARD
   { "fwd-credentials", 'f', NULL, 0,
     "allow the local credentials to be forwarded", GRID+1 },
   { NULL, 'F', NULL, 0,
     "forward a forwardable copy of the local credentials "
     "to the remote system", GRID+1 },
 # endif
-# undef GRID  
+# undef GRID
 #endif
-  
-#if defined(TN3270) && defined(unix)
+
+#if defined TN3270 && defined unix
 # define GRID 40
   { NULL, 0, NULL, 0,
     "TN3270 support:", GRID },
@@ -194,15 +192,15 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case '6':
       family = 6;
       break;
-	  
+
     case '8':
       eight = 3;		/* binary output and input */
       break;
-      
+
     case 'E':
       rlogin = escape = _POSIX_VDISABLE;
       break;
-      
+
     case 'K':
 #ifdef	AUTHENTICATION
       autologin = 0;
@@ -235,13 +233,13 @@ parse_opt (int key, char *arg, struct argp_state *state)
       set_escape_char (arg);
       break;
 
-#if defined(AUTHENTICATION) && defined(KRB5) && defined(FORWARD)
+#if defined AUTHENTICATION && defined KRB5 && defined FORWARD
     case 'f':
       if (forward_flags & OPTS_FORWARD_CREDS)
 	argp_error (state, "Only one of -f and -F allowed.", prompt);
       forward_flags |= OPTS_FORWARD_CREDS;
       break;
-      
+
     case 'F':
       if (forward_flags & OPTS_FORWARD_CREDS)
 	argp_error (state, "Only one of -f and -F allowed");
@@ -249,18 +247,18 @@ parse_opt (int key, char *arg, struct argp_state *state)
       forward_flags |= OPTS_FORWARDABLE_CREDS;
       break;
 #endif
-      
-#if defined(AUTHENTICATION) && defined(KRB4)
+
+#if defined AUTHENTICATION && defined KRB4
     case 'k':
       dest_realm = arg;
       break;
 #endif
-      
+
     case 'l':
       autologin = 1;
       user = arg;
       break;
-      
+
     case 'n':
       SetNetTrace (arg);
       break;
@@ -268,8 +266,8 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case 'r':
       rlogin = '~';
       break;
-      
-#if defined(TN3270) && defined(unix)
+
+#if defined TN3270 && defined unix
     case 't':
       /* FIXME: Buffer!!! */
       transcom = tline;
@@ -279,27 +277,27 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case OPTION_NOASYNCH:
       noasynchtty = noasynchtty = 1;
       break;
-      
+
     case OPTION_NOASYNCTTY:
       noasynchtty = 1;
       break;
-      
+
     case OPTION_NOASYNCNET:
       noasynchnet = 1;
       break;
 #endif
-      
+
 #ifdef	ENCRYPTION
     case 'x':
       encrypt_auto (1);
       decrypt_auto (1);
       break;
 #endif
-      
+
     default:
       return ARGP_ERR_UNKNOWN;
     }
-  
+
   return 0;
 }
 
@@ -320,9 +318,9 @@ main (int argc, char *argv[])
   int index;
 
   set_program_name (argv[0]);
-  
+
   tninit ();			/* Clear out things */
-#if defined(CRAY) && !defined(__STDC__)
+#if defined CRAY && !defined __STDC__
   _setlist_init ();		/* Work around compiler bug */
 #endif
 
@@ -353,7 +351,7 @@ main (int argc, char *argv[])
       char *args[8], **argp = args;
 
       if (argc > 2)
-	error (1, 0, "too many arguments");
+	error (EXIT_FAILURE, 0, "too many arguments");
       *argp++ = prompt;
       if (user)
 	{
