@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -47,9 +47,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <stdlib.h>
 
@@ -75,15 +73,15 @@ unsigned char netobuf[2 * BUFSIZ], netibuf[BUFSIZ];
  */
 
 void
-init_network ()
+init_network (void)
 {
   if (ring_init (&netoring, netobuf, sizeof netobuf) != 1)
     {
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   if (ring_init (&netiring, netibuf, sizeof netibuf) != 1)
     {
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   NetTrace = stdout;
 }
@@ -95,7 +93,7 @@ init_network ()
  */
 
 int
-stilloob ()
+stilloob (void)
 {
   static struct timeval timeout = { 0 };
   fd_set excepts;
@@ -133,7 +131,7 @@ stilloob ()
  */
 
 void
-setneturg ()
+setneturg (void)
 {
   ring_mark (&netoring);
 }
@@ -150,7 +148,7 @@ setneturg ()
 
 
 int
-netflush ()
+netflush (void)
 {
   register int n, n1;
 
