@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -80,7 +80,7 @@
 
 /* Internal macros */
 
-#if !defined(MIN)
+#if !defined MIN
 # define MIN(a,b)	(((a)<(b))? (a):(b))
 #endif /* !defined(MIN) */
 
@@ -101,7 +101,7 @@
  * to ZERO on allocation, we need to make sure, when interpreting the
  * clock, that when the times are EQUAL, then the buffer is FULL.
  */
-static u_long ring_clock = 0;
+static unsigned long ring_clock = 0;
 
 
 #define ring_empty(d) (((d)->consume == (d)->supply) && \
