@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -47,9 +47,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <stdlib.h>
 
@@ -107,15 +105,15 @@ cc_t termAytChar;
  */
 
 void
-init_terminal ()
+init_terminal (void)
 {
   if (ring_init (&ttyoring, ttyobuf, sizeof ttyobuf) != 1)
     {
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   if (ring_init (&ttyiring, ttyibuf, sizeof ttyibuf) != 1)
     {
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   autoflush = TerminalAutoFlush ();
 }
@@ -190,7 +188,7 @@ ttyflush (int drop)
 
 
 int
-getconnmode ()
+getconnmode (void)
 {
   extern int linemode;
   int mode = 0;
@@ -268,7 +266,7 @@ setconnmode (int force)
 
 
 void
-setcommandmode ()
+setcommandmode (void)
 {
   TerminalNewMode (-1);
 }
