@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
-  2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation,
-  Inc.
+  2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -47,9 +47,7 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #include <stdlib.h>
 
@@ -240,7 +238,7 @@ printoption (char *direction, int cmd, int option)
 }
 
 void
-optionstatus ()
+optionstatus (void)
 {
   register int i;
   extern char will_wont_resp[], do_dont_resp[];
@@ -338,7 +336,7 @@ printsub (char direction, unsigned char *pointer, int length)
   register int i;
   extern int want_status_response;
 
-#if defined(AUTHENTICATION) && defined(ENCRYPTION)
+#if defined AUTHENTICATION && defined ENCRYPTION
   char buf[512];
 #endif
 
@@ -483,7 +481,7 @@ printsub (char direction, unsigned char *pointer, int length)
 	    fprintf (NetTrace, " ?%d?", pointer[i]);
 	  break;
 
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
 	case TELOPT_AUTHENTICATION:
 	  fprintf (NetTrace, "AUTHENTICATION");
 	  if (length < 2)
@@ -879,7 +877,7 @@ printsub (char direction, unsigned char *pointer, int length)
 	    env_common:
 	      {
 		register int noquote = 2;
-#if defined(ENV_HACK) && defined(OLD_ENVIRON)
+#if defined ENV_HACK && defined OLD_ENVIRON
 		extern int old_env_var, old_env_value;
 #endif
 		for (i = 2; i < length; i++)
@@ -985,9 +983,9 @@ printsub (char direction, unsigned char *pointer, int length)
  */
 
 void
-EmptyTerminal ()
+EmptyTerminal (void)
 {
-#if defined(unix)
+#if defined unix
   fd_set o;
 
   FD_ZERO (&o);
@@ -995,7 +993,7 @@ EmptyTerminal ()
 
   if (TTYBYTES () == 0)
     {
-#if defined(unix)
+#if defined unix
       FD_SET (tout, &o);
       select (tout + 1, (fd_set *) 0, &o, (fd_set *) 0, (struct timeval *) 0);	/* wait for TTLOWAT */
 #endif /* defined(unix) */
@@ -1005,7 +1003,7 @@ EmptyTerminal ()
       while (TTYBYTES ())
 	{
 	  ttyflush (0);
-#if defined(unix)
+#if defined unix
 	  FD_SET (tout, &o);
 	  select (tout + 1, (fd_set *) 0, &o, (fd_set *) 0, (struct timeval *) 0);	/* wait for TTLOWAT */
 #endif /* defined(unix) */
@@ -1014,10 +1012,10 @@ EmptyTerminal ()
 }
 
 void
-SetForExit ()
+SetForExit (void)
 {
   setconnmode (0);
-#if defined(TN3270)
+#if defined TN3270
   if (In3270)
     {
       Finish3270 ();
@@ -1033,7 +1031,7 @@ SetForExit ()
   setcommandmode ();
   fflush (stdout);
   fflush (stderr);
-#if defined(TN3270)
+#if defined TN3270
   if (In3270)
     {
       StopScreen (1);
