@@ -1,6 +1,6 @@
 /*
-  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-  Free Software Foundation, Inc.
+  Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+  2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -49,7 +49,7 @@ startslave (char *host, int autologin, char *autoname)
   if (autologin < auth_level)
     {
       fatal (net, "Authorization failed");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 #endif
   pid = forkpty (&master, line, NULL, NULL);
@@ -86,14 +86,14 @@ startslave (char *host, int autologin, char *autoname)
 extern char **environ;
 /*
  * scrub_env()
- *  
+ *
  * Remove a few things from the environment that
  * don't need to be there.
- *  
+ *
  * Security fix included in telnet-95.10.23.NE of David Borman <deb@cray.com>.
  */
 static void
-scrub_env ()
+scrub_env (void)
 {
   register char **cpp, **cpp2;
 
@@ -149,5 +149,5 @@ cleanup (int sig)
   chmod (line, 0644);
   chown (line, 0, 0);
   shutdown (net, 2);
-  exit (1);
+  exit (EXIT_FAILURE);
 }
