@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-  Foundation, Inc.
+  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free
+  Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -52,6 +52,15 @@
 #include "telnetd.h"
 #include <stdarg.h>
 
+#if defined HAVE_STREAMSPTY && defined TIOCSIGNAL \
+	&& defined HAVE_STROPTS_H
+# include <stropts.h>	/* I_FLUSH, FLUSHR */
+#endif
+
+#ifndef NTELOPTS	/* OpenSolaris */
+# define NTELOPTS	(1+TELOPT_NEW_ENVIRON)
+#endif
+
 /* Format lines for corresponing commands */
 char doopt[] = { IAC, DO, '%', 'c', 0 };
 char dont[] = { IAC, DONT, '%', 'c', 0 };
@@ -95,7 +104,7 @@ unsigned char *subsave;
 #define TS_DONT		8	/* dont " */
 
 static void
-send_eof ()
+send_eof (void)
 {
   init_termbuf ();
   term_send_eof ();
@@ -108,9 +117,9 @@ send_eof ()
  * just send back "[Yes]".
  */
 static void
-recv_ayt ()
+recv_ayt (void)
 {
-#if defined(SIGINFO) && defined(TCSIG)
+#if defined SIGINFO && defined TCSIG
   if (slctab[SLC_AYT].sptr && *slctab[SLC_AYT].sptr != _POSIX_VDISABLE)
     {
       ioctl (pty, TCSIG, (char *) SIGINFO);
@@ -121,7 +130,7 @@ recv_ayt ()
 }
 
 static void
-send_susp ()
+send_susp (void)
 {
 #ifdef	SIGTSTP
   ptyflush ();			/* half-hearted */
@@ -140,7 +149,7 @@ send_susp ()
  * otherwise, write quit char.
  */
 static void
-send_brk ()
+send_brk (void)
 {
   ptyflush ();			/* half-hearted */
 #ifdef	TCSIG
@@ -158,11 +167,11 @@ send_brk ()
  * otherwise, write intr char.
  */
 static void
-send_intr ()
+send_intr (void)
 {
   ptyflush ();			/* half-hearted */
 
-#if defined(HAVE_STREAMSPTY) && defined(TIOCSIGNAL)
+#if defined HAVE_STREAMSPTY && defined TIOCSIGNAL
   /* Streams PTY style ioctl to post a signal */
   {
     int sig = SIGINT;
@@ -181,7 +190,7 @@ send_intr ()
 }
 
 void
-telrcv ()
+telrcv (void)
 {
   register int c;
   static int state = TS_DATA;
@@ -449,7 +458,7 @@ telrcv ()
 	default:
 	  syslog (LOG_ERR, "telnetd: panic state=%d\n", state);
 	  printf ("telnetd: panic state=%d\n", state);
-	  exit (1);
+	  exit (EXIT_FAILURE);
 	}
     }
 }				/* end of telrcv */
@@ -537,7 +546,7 @@ send_do (int option, int init)
 #ifdef	AUTHENTICATION
 extern void auth_request ();
 #endif
-extern void doclientstat ();
+extern void doclientstat (void);
 #ifdef	ENCRYPTION
 extern void encrypt_send_support ();
 #endif /* ENCRYPTION */
@@ -816,7 +825,7 @@ wontoption (int option)
 	      slctab[SLC_XOFF].defset.flag |= SLC_CANTCHANGE;
 	      break;
 
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
 	    case TELOPT_AUTHENTICATION:
 	      auth_finished (0, AUTH_REJECT);
 	      break;
@@ -871,7 +880,7 @@ wontoption (int option)
 		}
 	      break;
 
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
 	    case TELOPT_AUTHENTICATION:
 	      auth_finished (0, AUTH_REJECT);
 	      break;
@@ -1135,7 +1144,7 @@ int env_ovalue = -1;
  *      Terminal speed
  */
 void
-suboption ()
+suboption (void)
 {
   register int subchar;
 
@@ -1511,7 +1520,7 @@ suboption ()
 	  unsetenv (varp);
 	break;
       }				/* end of case TELOPT_NEW_ENVIRON */
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
     case TELOPT_AUTHENTICATION:
       if (SB_EOF ())
 	break;
@@ -1585,13 +1594,13 @@ suboption ()
 }				/* end of suboption */
 
 void
-doclientstat ()
+doclientstat (void)
 {
   clientstat (TELOPT_LINEMODE, WILL, 0);
 }
 
 void
-send_status ()
+send_status (void)
 {
 #define ADD(c) \
         do { \
