@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-  Foundation, Inc.
+  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free
+  Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -39,9 +39,9 @@ static void print_hostinfo (void);
 
 char *login_invocation =
 #ifdef SOLARIS
-  "/bin/login -h %h %?T{TERM=%T}{-} %?u{%?a{-f }-- %u}"
+  PATH_LOGIN " -h %h %?T{-t %T} %?u{-u %u}"
 #else
-  "/bin/login -p -h %h %?u{-f %u}"
+  PATH_LOGIN " -p -h %h %?u{-f %u}"
 #endif
   ;
 
@@ -77,7 +77,7 @@ int lmodetype;			/* Client support for linemode */
 int flowmode;			/* current flow control state */
 int restartany;			/* restart output on any character state */
 int diagnostic;			/* telnet diagnostic capabilities */
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
 int auth_level;
 int autologin;
 #endif
@@ -96,7 +96,7 @@ static struct argp_option argp_options[] = {
   { "debug", 'D', "LEVEL", OPTION_ARG_OPTIONAL,
     "set debugging level" },
   { "exec-login", 'E', "STRING", 0,
-    "set program to be executed instead of /bin/login" },
+    "set program to be executed instead of " PATH_LOGIN },
   { "no-hostinfo", 'h', NULL, 0,
     "do not print host information before login has been completed" },
   { "linemode", 'l', "MODE", OPTION_ARG_OPTIONAL,
@@ -104,7 +104,7 @@ static struct argp_option argp_options[] = {
   { "no-keepalive", 'n', NULL, 0,
     "disable TCP keep-alives" },
   { "reverse-lookup", 'U', NULL, 0,
-    "refuse  connections from addresses that "
+    "refuse connections from addresses that "
     "cannot be mapped back into a symbolic name" },
 #ifdef  AUTHENTICATION
   { "disable-auth-type", 'X', "TYPE", 0,
@@ -129,7 +129,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case 'E':
       login_invocation = arg;
       break;
-	  
+
     case 'h':
       hostinfo = 0;
       break;
@@ -141,7 +141,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case 'n':
       keepalive = 0;
       break;
-	  
+
     case 'U':
       reverse_lookup = 1;
       break;
@@ -151,7 +151,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       auth_disable_name (arg);
       break;
 #endif
-      
+
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -177,9 +177,9 @@ main (int argc, char **argv)
   set_program_name (argv[0]);
   iu_argp_init ("telnetd", default_program_authors);
   argp_parse (&argp, argc, argv, 0, &index, NULL);
-  
+
   if (argc != index)
-    error (1, 0, "junk arguments in the command line");
+    error (EXIT_FAILURE, 0, "junk arguments in the command line");
 
   openlog ("telnetd", LOG_PID | LOG_ODELAY, LOG_DAEMON);
   telnetd_setup (0);
@@ -296,11 +296,11 @@ telnetd_setup (int fd)
   if (getpeername (fd, (struct sockaddr *) &saddr, &len) < 0)
     {
       syslog (LOG_ERR, "getpeername: %m");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
 
 #ifdef IPV6
-  err = getnameinfo ((struct sockaddr *) &saddr, sizeof (saddr), buf,
+  err = getnameinfo ((struct sockaddr *) &saddr, len, buf,
 		     sizeof (buf), NULL, 0, NI_NUMERICHOST);
   if (err)
     {
@@ -317,7 +317,7 @@ telnetd_setup (int fd)
 
   /* We use a second buffer so we don't have to call getnameinfo again
      if we need the numeric host below.  */
-  err = getnameinfo ((struct sockaddr *) &saddr, sizeof (saddr), buf2,
+  err = getnameinfo ((struct sockaddr *) &saddr, len, buf2,
 		     sizeof (buf2), NULL, 0, NI_NAMEREQD);
 
   if (reverse_lookup)
@@ -362,7 +362,7 @@ telnetd_setup (int fd)
 	{
 	  syslog (LOG_AUTH | LOG_NOTICE,
 		  "None of addresses of %s matched %s", remote_hostname, buf);
-	  exit (0);
+	  exit (EXIT_SUCCESS);
 	}
 
       freeaddrinfo (result);
@@ -409,7 +409,7 @@ telnetd_setup (int fd)
 	  syslog (LOG_AUTH | LOG_NOTICE,
 		  "None of addresses of %s matched %s",
 		  remote_hostname, inet_ntoa (saddr.sin_addr));
-	  exit (0);
+	  exit (EXIT_SUCCESS);
 	}
     }
   else
@@ -436,7 +436,7 @@ telnetd_setup (int fd)
   net = fd;
 
   local_hostname = localhost ();
-#if defined(AUTHENTICATION) || defined(ENCRYPTION)
+#if defined AUTHENTICATION || defined ENCRYPTION
   auth_encrypt_init (remote_hostname, local_hostname, "TELNETD", 1);
 #endif
 
@@ -457,7 +457,7 @@ telnetd_setup (int fd)
   ioctl (pty, FIONBIO, (char *) &true);
   ioctl (net, FIONBIO, (char *) &true);
 
-#if defined(SO_OOBINLINE)
+#if defined SO_OOBINLINE
   setsockopt (net, SOL_SOCKET, SO_OOBINLINE, (char *) &true, sizeof true);
 #endif
 
@@ -472,7 +472,7 @@ telnetd_setup (int fd)
 }
 
 int
-telnetd_run ()
+telnetd_run (void)
 {
   int nfd;
 
@@ -505,7 +505,7 @@ telnetd_run ()
      clients might not respond to it. To work around this, we wait
      for a response to NAWS, which should have been processed after
      DO ECHO (most dumb telnets respond with WONT for a DO that
-     they don't understand).  
+     they don't understand).
      On the other hand, the client might have sent WILL NAWS as
      part of its startup code, in this case it surely should have
      answered our DO ECHO, so the second loop is waiting for
@@ -602,7 +602,7 @@ telnetd_run ()
 	  if (pty_read () < 0)
 	    break;
 	  c = pty_get_char (1);
-#if defined(TIOCPKT_IOCTL)
+#if defined TIOCPKT_IOCTL
 	  if (c & TIOCPKT_IOCTL)
 	    {
 	      pty_get_char (0);
@@ -666,7 +666,7 @@ telnetd_run ()
 }
 
 void
-print_hostinfo ()
+print_hostinfo (void)
 {
   char *im = NULL;
   char *str;
