@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-  Foundation, Inc.
+  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free
+  Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -60,7 +60,7 @@ int def_row = 0, def_col = 0;
 #endif
 static int _terminit = 0;
 
-#if defined(CRAY2) && defined(UNICOS5)
+#if defined CRAY2 && defined UNICOS5
 int newmap = 1;			/* nonzero if \n maps to ^M^J */
 #endif
 
@@ -79,10 +79,10 @@ int newmap = 1;			/* nonzero if \n maps to ^M^J */
  * Linemode support uses the following state flags to keep track of
  * current and desired linemode state.
  *	alwayslinemode : true if -l was specified on the telnetd
- * 	command line.  It means to have linemode on as much as
+ *	command line.  It means to have linemode on as much as
  *	possible.
  *
- * 	lmodetype: signifies whether the client can
+ *	lmodetype: signifies whether the client can
  *	handle real linemode, or if use of kludgeomatic linemode
  *	is preferred.  It will be set to one of the following:
  *		REAL_LINEMODE : use linemode option
@@ -141,12 +141,12 @@ int newmap = 1;			/* nonzero if \n maps to ^M^J */
  *	   is on.
  */
 void
-localstat ()
+localstat (void)
 {
-  void netflush ();
+  void netflush (void);
   int need_will_echo = 0;
 
-#if defined(CRAY2) && defined(UNICOS5)
+#if defined CRAY2 && defined UNICOS5
   /*
    * Keep track of that ol' CR/NL mapping while we're in the
    * neighborhood.
@@ -371,7 +371,7 @@ done:
  * Check for changes to flow control
  */
 void
-flowstat ()
+flowstat (void)
 {
   if (his_state_is_will (TELOPT_LFLOW))
     {
@@ -404,7 +404,7 @@ flowstat ()
 void
 clientstat (register int code, register int parm1, register int parm2)
 {
-  void netflush ();
+  void netflush (void);
 
   /*
    * Get a copy of terminal characteristics.
@@ -427,7 +427,7 @@ clientstat (register int code, register int parm1, register int parm2)
 	  /*
 	   * If using kludge linemode, make sure that
 	   * we can do what the client asks.
-	   * We can not turn off linemode if alwayslinemode
+	   * We cannot turn off linemode if alwayslinemode
 	   * and the ICANON bit is set.
 	   */
 	  if (lmodetype == KLUDGE_LINEMODE)
@@ -586,7 +586,7 @@ clientstat (register int code, register int parm1, register int parm2)
       break;
     }				/* end of switch */
 
-#if defined(CRAY2) && defined(UNICOS5)
+#if defined CRAY2 && defined UNICOS5
   /*
    * Just in case of the likely event that we changed the pty state.
    */
@@ -597,7 +597,7 @@ clientstat (register int code, register int parm1, register int parm2)
 
 }				/* end of clientstat */
 
-#if defined(CRAY2) && defined(UNICOS5)
+#if defined CRAY2 && defined UNICOS5
 void
 termstat ()
 {
@@ -623,7 +623,7 @@ _termstat ()
  * It calls other functions that do things that were deferred in each module.
  */
 void
-defer_terminit ()
+defer_terminit (void)
 {
 
   /*
@@ -660,7 +660,7 @@ defer_terminit ()
  * Returns true if the pty state has been processed yet.
  */
 int
-terminit ()
+terminit (void)
 {
   return (_terminit);
 
