@@ -1,7 +1,7 @@
 /*
   Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
-  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-  Foundation, Inc.
+  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free
+  Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -30,17 +30,24 @@
 #endif
 #include <time.h>
 
-#if defined(AUTHENTICATION) || defined(ENCRYPTION)
+#if defined AUTHENTICATION || defined ENCRYPTION
 # include <libtelnet/misc.h>
 # define NET_ENCRYPT net_encrypt
 #else
 # define NET_ENCRYPT()
 #endif
 
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_TERMCAP_TGETENT
+# include <termcap.h>
+#elif defined HAVE_CURSES_TGETENT
+# include <curses.h>
 # include <term.h>
 #endif
 
+#if defined HAVE_STREAMSPTY && defined HAVE_GETMSG \
+	&& defined HAVE_STROPTS_H
+# include <stropts.h>
+#endif
 
 static char netobuf[BUFSIZ + NETSLOP], *nfrontp, *nbackp;
 static char *neturg;		/* one past last byte of urgent data */
@@ -67,7 +74,9 @@ readstream (int p, char *ibuf, int bufsize)
   int flags = 0;
   int ret = 0;
   struct termios *tsp;
+#ifdef HAVE_TERMIO_H
   struct termio *tp;
+#endif
   struct iocblk *ip;
   char vstop, vstart;
   int ixon;
@@ -125,6 +134,7 @@ readstream (int p, char *ibuf, int bufsize)
 	  ixon = tsp->c_iflag & IXON;
 	  break;
 
+#ifdef HAVE_TERMIO_H
 	case TCSETA:
 	case TCSETAW:
 	case TCSETAF:
@@ -133,6 +143,7 @@ readstream (int p, char *ibuf, int bufsize)
 	  vstart = tp->c_cc[VSTART];
 	  ixon = tp->c_iflag & IXON;
 	  break;
+#endif /* HAVE_TERMIO_H */
 
 	default:
 	  errno = EAGAIN;
@@ -158,7 +169,7 @@ readstream (int p, char *ibuf, int bufsize)
 /* Net and PTY I/O functions */
 
 void
-io_setup ()
+io_setup (void)
 {
   pfrontp = pbackp = ptyobuf;
   nfrontp = nbackp = netobuf;
@@ -170,7 +181,7 @@ io_setup ()
 }
 
 void
-set_neturg ()
+set_neturg (void)
 {
   neturg = nfrontp - 1;
 }
@@ -222,19 +233,19 @@ net_output_datalen (const void *buf, size_t l)
 }
 
 int
-net_input_level ()
+net_input_level (void)
 {
   return ncc;
 }
 
 int
-net_output_level ()
+net_output_level (void)
 {
   return nfrontp - nbackp;
 }
 
 int
-net_buffer_is_full ()
+net_buffer_is_full (void)
 {
   return (&netobuf[BUFSIZ] - nfrontp) < 2;
 }
@@ -254,7 +265,7 @@ net_get_char (int peek)
 }
 
 int
-net_read ()
+net_read (void)
 {
   ncc = read (net, netibuf, sizeof (netibuf));
   if (ncc < 0 && errno == EWOULDBLOCK)
@@ -277,7 +288,7 @@ net_read ()
 /* PTY buffer functions */
 
 int
-pty_buffer_is_full ()
+pty_buffer_is_full (void)
 {
   return (&ptyobuf[BUFSIZ] - pfrontp) < 2;
 }
@@ -298,19 +309,19 @@ pty_output_datalen (const void *data, size_t len)
 }
 
 int
-pty_input_level ()
+pty_input_level (void)
 {
   return pcc;
 }
 
 int
-pty_output_level ()
+pty_output_level (void)
 {
   return pfrontp - pbackp;
 }
 
 void
-ptyflush ()
+ptyflush (void)
 {
   int n;
 
@@ -360,7 +371,7 @@ pty_input_putback (const char *str, size_t len)
 }
 
 int
-pty_read ()
+pty_read (void)
 {
   pcc = readstream (pty, ptyibuf, BUFSIZ);
   if (pcc < 0 && (errno == EWOULDBLOCK
@@ -380,8 +391,8 @@ pty_read ()
 
 
 /* io_drain ()
- *   
- *  
+ *
+ *
  *	A small subroutine to flush the network output buffer, get some data
  * from the network, and pass it through the telnet state machine.  We
  * also flush the pty input buffer (by dropping its data) if it becomes
@@ -389,7 +400,7 @@ pty_read ()
  */
 
 void
-io_drain ()
+io_drain (void)
 {
   DEBUG (debug_report, 1, debug_output_data ("td: ttloop\r\n"));
   if (nfrontp - nbackp > 0)
@@ -405,12 +416,12 @@ again:
 	  goto again;
 	}
       syslog (LOG_INFO, "ttloop:  read: %m\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   else if (ncc == 0)
     {
       syslog (LOG_INFO, "ttloop:  peer died: %m\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   DEBUG (debug_report, 1,
 	 debug_output_data ("td: ttloop read %d chars\r\n", ncc));
@@ -450,7 +461,7 @@ stilloob (int s)
 
 /*
  * nextitem()
- *  
+ *
  *	Return the address of the next "item" in the TELNET data
  * stream.  This will be the address of the next character if
  * the current address is a user data character, or it will
@@ -489,15 +500,15 @@ nextitem (char *current)
 
 /*
  * netclear()
- *  
+ *
  *	We are about to do a TELNET SYNCH operation.  Clear
  * the path to the network.
- *  
+ *
  *	Things are a bit tricky since we may have sent the first
  * byte or so of a previous TELNET command into the network.
  * So, we have to scan the network buffer from the beginning
  * until we are up to where we want to be.
- *  
+ *
  *	A side effect of what we do, just to keep things
  * simple, is to clear the urgent data pointer.  The principal
  * caller should be setting the urgent data pointer AFTER calling
@@ -509,7 +520,7 @@ nextitem (char *current)
 
 
 void
-netclear ()
+netclear (void)
 {
   register char *thisitem, *next;
   char *good;
@@ -563,7 +574,7 @@ netclear ()
  *	handling requests for urgent data.
  */
 void
-netflush ()
+netflush (void)
 {
   int n;
 
@@ -644,7 +655,7 @@ fatal (int f, char *msg)
 #endif /* ENCRYPTION */
   write (f, buf, (int) strlen (buf));
   sleep (1);
-   /*FIXME*/ exit (1);
+   /*FIXME*/ exit (EXIT_FAILURE);
 }
 
 void
@@ -664,7 +675,7 @@ static unsigned char ttytype_sbbuf[] = {
 };
 
 static void
-_gettermname ()
+_gettermname (void)
 {
   if (his_state_is_wont (TELOPT_TTYPE))
     return;
@@ -682,7 +693,7 @@ getterminaltype (char *user_name)
   int retval = -1;
 
   settimer (baseline);
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
   /*
    * Handle the Authentication option before we do anything else.
    */
@@ -821,7 +832,7 @@ terminaltypeok (char *s)
   if (terminaltype == NULL)
     return 1;
 
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_TGETENT
   if (tgetent (buf, s) == 0)
 #endif
     return 0;
@@ -836,7 +847,7 @@ terminaltypeok (char *s)
 static FILE *debug_fp = NULL;
 
 static int
-debug_open ()
+debug_open (void)
 {
   int um = umask (077);
   if (!debug_fp)
@@ -846,7 +857,7 @@ debug_open ()
 }
 
 static int
-debug_close ()
+debug_close (void)
 {
   if (debug_fp)
     fclose (debug_fp);
@@ -899,7 +910,7 @@ printsub (int direction, unsigned char *pointer, int length)
 {
   register int i;
 
-#if defined(AUTHENTICATION) && defined(ENCRYPTION)
+#if defined AUTHENTICATION && defined ENCRYPTION
   char buf[512];
 #endif
 
@@ -1358,7 +1369,7 @@ printsub (int direction, unsigned char *pointer, int length)
 	}
       break;
 
-#if defined(AUTHENTICATION)
+#if defined AUTHENTICATION
     case TELOPT_AUTHENTICATION:
       debug_output_data ("AUTHENTICATION");
 
@@ -1547,7 +1558,7 @@ printdata (register char *tag, register char *ptr, register int cnt)
     }
 }
 
-#if defined(AUTHENTICATION) || defined(ENCRYPTION)
+#if defined AUTHENTICATION || defined ENCRYPTION
 
 int
 net_write (unsigned char *str, int len)
