@@ -1,5 +1,6 @@
 /*
-  Copyright (C) 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation,
+  Inc.
 
   This file is part of GNU Inetutils.
 
@@ -21,12 +22,13 @@
 #include "libinetutils.h"
 
 #include <stdio.h>
+#include <progname.h>
 
 int
-main (void)
+main (int argc, char **argv)
 {
   const char *p = localhost ();
-
+  set_program_name (argv[0]);
   if (!p)
     return 1;
 
