@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-  2007, 2008, 2009 Free Software Foundation, Inc.
+  2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -19,9 +19,7 @@
 
 /* Written by Marco d'Itri.  */
 
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 /* System library */
 #include <stdio.h>
 #include <stdlib.h>
@@ -213,7 +211,7 @@ main (int argc, char *argv[])
   argv += index;
 
   if (argc == 0 && !nopar)	/* there is no parameter */
-    error (1, 0, "not enough arguments");
+    error (EXIT_FAILURE, 0, "not enough arguments");
 
   /* parse other parameters, if any */
   if (!nopar)
@@ -237,7 +235,7 @@ main (int argc, char *argv[])
       server = query_crsnic (sockfd, qstring);
       closeconn (sockfd);
       if (!server)
-	exit (0);
+	exit (EXIT_SUCCESS);
       printf (_("\nFound InterNIC referral to %s.\n\n"), server);
     }
 
@@ -257,10 +255,10 @@ main (int argc, char *argv[])
 		  "whois database at"));
 	case 2:
 	  puts (server + 1);
-	  exit (0);
+	  exit (EXIT_SUCCESS);
 	case 3:
 	  puts (_("This TLD has no whois server."));
-	  exit (0);
+	  exit (EXIT_SUCCESS);
 	default:
 	  if (verb)
 	    printf (_("Using server %s.\n"), server);
@@ -282,7 +280,7 @@ main (int argc, char *argv[])
   do_query (sockfd, p);
   closeconn (sockfd);
 
-  exit (0);
+  exit (EXIT_SUCCESS);
 }
 
 const char *
@@ -558,16 +556,9 @@ int
 openconn (const char *server, const char *port)
 {
   int fd;
-#ifdef HAVE_GETADDRINFO
   int i;
   struct addrinfo hints, *res, *ressave;
-#else
-  struct hostent *hostinfo;
-  struct servent *servinfo;
-  struct sockaddr_in saddr;
-#endif
 
-#ifdef HAVE_GETADDRINFO
   memset (&hints, 0, sizeof (struct addrinfo));
   hints.ai_family = AF_UNSPEC;
   hints.ai_socktype = SOCK_STREAM;
@@ -588,27 +579,6 @@ openconn (const char *server, const char *port)
   if (!res)
     err_sys ("connect");
   freeaddrinfo (ressave);
-#else
-  if ((hostinfo = gethostbyname (server)) == NULL)
-    err_quit (_("Host %s not found."), server);
-  if ((fd = socket (PF_INET, SOCK_STREAM, IPPROTO_IP)) < 0)
-    err_sys ("socket");
-  memset (&saddr, 0, sizeof (saddr));
-  saddr.sin_addr = *(struct in_addr *) hostinfo->h_addr;
-  saddr.sin_family = AF_INET;
-  if (!port)
-    {
-      saddr.sin_port = htons (43);
-    }
-  else if ((saddr.sin_port = htons (atoi (port))) == 0)
-    {
-      if ((servinfo = getservbyname (port, "tcp")) == NULL)
-	err_quit (_("%s/tcp: unknown service"), port);
-      saddr.sin_port = servinfo->s_port;
-    }
-  if (connect (fd, &saddr, sizeof (saddr)) < 0)
-    err_sys ("connect");
-#endif
   return (fd);
 }
 
@@ -618,7 +588,7 @@ closeconn (const int fd)
   close (fd);
 }
 
-RETSIGTYPE
+void
 sighandler (int signum)
 {
   closeconn (sockfd);
@@ -680,7 +650,7 @@ err_sys (const char *fmt, ...)
   vfprintf (stderr, fmt, ap);
   fprintf (stderr, ": %s\n", strerror (errno));
   va_end (ap);
-  exit (2);
+  exit (EXIT_FAILURE);
 }
 
 void
@@ -692,5 +662,5 @@ err_quit (const char *fmt, ...)
   vfprintf (stderr, fmt, ap);
   fputs ("\n", stderr);
   va_end (ap);
-  exit (2);
+  exit (EXIT_FAILURE);
 }
