@@ -1,6 +1,6 @@
 /*
   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-  2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+  2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -61,7 +61,7 @@ void do_query (const int, const char *);
 const char *query_crsnic (const int, const char *);
 int openconn (const char *, const char *);
 void closeconn (const int);
-RETSIGTYPE sighandler (int);
+void sighandler (int);
 unsigned long myinet_aton (const char *);
 int domcmp (const char *, const char *);
 int domfind (const char *, const char *[]);
